{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the definition for TjmGridColumn class.

  This class represents a single column within a zone.
}

  TjmGridColumn = class (TComponent)
    private
      { Read-Only Properties }
      FZone : TjmGridZone ;
      FColumnNumber : Cardinal ;
      FGutterSize : Cardinal ;
      FOverride : TjmGridOverrideSettings ;

      FColumnType : TjmGridColumnType ;
      FFieldName : String ;
      FHeaderText : String ;
      FFooterText : String ;
      FField : TField ;

      FEditorType : TjmGridEditorType ;
      FEditor : TWinControl ;

      FCellFrameColor : TColor ;
      FCellFrameStyle : TjmGridFrameStyle ;
      FHeaderFrameColor : TColor ;
      FHeaderFrameStyle : TjmGridFrameStyle ;
      FFooterFrameColor : TColor ;
      FFooterFrameStyle : TjmGridFrameStyle ;
      FActiveFrameColor : TColor ;
      FActiveFrameStyle : TjmGridFrameStyle ;

      { Basic Properties }
      FWidth : Cardinal ;
      FHidden : Boolean ;
      FReadOnly : Boolean ;

      { Position Properties }
      FAlignment : TAlignment ;
      FVerticalAlignment : TjmGridVerticalAlignment ;

      FHeaderAlignment : TAlignment ;
      FHeaderVerticalAlignment : TjmGridVerticalAlignment ;
      FFooterAlignment : TAlignment ;
      FFooterVerticalAlignment : TjmGridVerticalAlignment ;

      { Background Properties }
      FBackgroundColor : TColor ;
      FFooterColor : TColor ;
      FHeaderColor : TColor ;
      FInactiveColor : TColor ;
      FSelectColor  : TColor ;

      { Line Properties }
      FVerticalLineStyle : TjmGridLineStyle ;
      FVerticalLineColor : TColor ;

      { Font Properties }
      FFont : TFont ;
      FFooterFont : TFont ;
      FInactiveFont : TFont ;
      FHeaderFont : TFont ;
      FSelectFont : TFont ;

      { Property Functions }
      Procedure SetAlignment (const Align : TAlignment) ;
      Function GetBackgroundColor : TColor ;
      Procedure SetBackgroundColor (AColor : TColor) ;
      Procedure SetFont (const AFont : TFont) ;
      Procedure SetFooterAlignment (Align : TAlignment) ;
      Procedure SetFooterColor (AColor : TColor) ;
      Function GetFooterColor : TColor ;
      Procedure SetFooterFont (const AFont : TFont) ;
      Procedure SetFooterVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
      Function GetParentGrid : TColossalGrid ;
      Procedure SetHeaderAlignment (Align : TAlignment) ;
      Procedure SetHeaderColor (AColor : TColor) ;
      Function GetHeaderColor : TColor ;
      Procedure SetHeaderFont (const AFont : TFont) ;
      Procedure SetHeaderVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
      Procedure SetHidden (State : Boolean) ;
      Procedure SetInactiveColor (AColor : TColor) ;
      Function GetInactiveColor : TColor ;
      Procedure SetInactiveFont (const AFont : TFont) ;
      Procedure SetOverride (AValue : TjmGridOverrideSettings) ;
      Procedure SetSelectColor (AColor : TColor) ;
      Function GetSelectColor : TColor ;
      Procedure SetSelectFont (const AFont : TFont) ;
      Procedure SetVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
      Function GetVerticalLineColor : TColor ;
      Procedure SetVerticalLineColor (AColor : TColor) ;
      Function GetVerticalLineStyle : TjmGridLineStyle ;
      Procedure SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;
      Procedure SetWidth (AWidth : Cardinal) ;

      Function GetCellFrameColor : TColor ;
      Procedure SetCellFrameColor (AColor : TColor) ;
      Function GetCellFrameStyle : TjmGridFrameStyle ;
      Procedure SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetHeaderFrameColor : TColor ;
      Procedure SetHeaderFrameColor (AColor : TColor) ;
      Function GetHeaderFrameStyle : TjmGridFrameStyle ;
      Procedure SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetFooterFrameColor : TColor ;
      Procedure SetFooterFrameColor (AColor : TColor) ;
      Function GetFooterFrameStyle : TjmGridFrameStyle ;
      Procedure SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetActiveFrameColor : TColor ;
      Procedure SetActiveFrameColor (AColor : TColor) ;
      Function GetActiveFrameStyle : TjmGridFrameStyle ;
      Procedure SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;

      { Property Functions }
      Procedure SetColumnType (AType : TjmGridColumnType) ;
      Procedure SetFieldName (Text : String) ;
      Function GetFooterText : String ;  
      Procedure SetFooterText (Text : String) ;
      Function GetHeaderText : String ;  
      Procedure SetHeaderText (Text : String) ;
      Procedure CreateEditor ;
      Procedure SetEditorType (AType : TjmGridEditorType) ;
      Function GetEditor : TWinControl ; Virtual ;

      Procedure AdjustRectForGutter (Var ARect : TRect) ;

      Procedure FontChanged (Sender : TObject) ;

      Function StoreAppearanceProperties : Boolean ;


      Procedure FindField ;

      Function SaveToDatabase (Value : String) : Boolean ;

    protected   {*** TjmGridColumn ***}

      Procedure LayoutChanged ;

      Procedure DrawCell (ARecord : LongInt ;
                          ACanvas : TCanvas ;
                          const ARect : TRect) ; Virtual ;
      Procedure DrawCellBackground (ARecord : LongInt ;
                                    ACanvas : TCanvas ;
                                    const ARect : TRect) ; Virtual ;
      Procedure DrawFooter (ACanvas : TCanvas ;
                            const ARect : TRect) ; Virtual ;
      Procedure DrawFooterBackground (ACanvas : TCanvas ;
                                      const ARect : TRect) ; Virtual ;
      Procedure DrawHeader(ACanvas : TCanvas ;
                           const ARect : TRect) ; Virtual ;
      Procedure DrawHeaderBackground (ACanvas : TCanvas ;
                                      const ARect : TRect) ; Virtual ;

      Function GetCellText (RecordNumber : LongInt) : String ; Virtual ;
      Function GetCellEditText (RecordNumber : LongInt) : String ; Virtual ;


      Function GetReadOnly : Boolean ; Virtual ;
      Procedure SetReadOnly (AValue : Boolean) ;

      Function EffectiveFont : TFont ;
      Function EffectiveSelectFont : TFont ;
      Function EffectiveInactiveFont : TFont ;
      Function EffectiveHeaderFont : TFont ;
      Function EffectiveFooterFont : TFont ;

      Procedure RepaintHeader ;
      Procedure RepaintFooter ;

      Function UpdateValue (Value : String) : Boolean ; Virtual ;

      Property ColumnNumber : Cardinal
               Read FColumnNumber
               Write FColumnNumber
               Stored False;
      Property ParentGrid : TColossalGrid
               Read GetParentGrid
               Stored False ;
      Property ParentZone : TjmGridZone
               Read FZone
               Stored False ;



    public     {*** TjmGridColumn ***}
      Constructor Create (AZone : TComponent) ; Override ;
      Destructor Destroy ; Override ;

      Property Editor : TWinControl
               Read GetEditor ;

    Published
      Property ActiveFrameColor : TColor
               Read GetActiveFrameColor
               Write SetActiveFrameColor
               Stored StoreAppearanceProperties ;
      Property ActiveFrameStyle : TjmGridFrameStyle
               Read GetActiveFrameStyle
               Write SetActiveFrameStyle
               Stored StoreAppearanceProperties ;
      Property Alignment : TAlignment
               Read FAlignment
               Write SetAlignment
               Default taLeftJustify ;
      Property BackgroundColor : TColor
               Read GetBackgroundColor
               Write SetBackgroundColor
               Stored StoreAppearanceProperties ;
      Property CellFrameColor : TColor
               Read GetCellFrameColor
               Write SetCellFrameColor
               Stored StoreAppearanceProperties ;
      Property CellFrameStyle : TjmGridFrameStyle
               Read GetCellFrameStyle
               Write SetCellFrameStyle
               Stored StoreAppearanceProperties ;
      Property ColumnType : TjmGridColumnType
               Read FColumnType
               Write SetColumnType
               Default jmDefaultColumnType ;
      Property EditorType : TjmGridEditorType
               Read FEditorType
               Write SetEditorType
               Default etDefault ;
      Property Field : TField
               Read FField ;
      Property FieldName : String
               Read FFieldName
               Write SetFieldName ;
      Property Font : TFont
               Read FFont
               Write SetFont ;
      Property FooterAlignment : TAlignment
               Read FFooterAlignment
               Write SetFooterAlignment
               Default taLeftJustify ;
      Property FooterColor : TColor
               Read GetFooterColor
               Write SetFooterColor
               Stored StoreAppearanceProperties ;
      Property FooterFont : TFont
               Read FFooterFont
               Write SetFooterFont ;
      Property FooterFrameColor : TColor
               Read GetFooterFrameColor
               Write SetFooterFrameColor
               Stored StoreAppearanceProperties ;
      Property FooterFrameStyle : TjmGridFrameStyle
               Read GetFooterFrameStyle
               Write SetFooterFrameStyle
               Stored StoreAppearanceProperties ;
      Property FooterText : String
               Read FFooterText
               Write SetFooterText ;
      Property FooterVerticalAlignment : TjmGridVerticalAlignment
               Read FFooterVerticalAlignment
               Write SetFooterVerticalAlignment
               Default vaTop ;
      Property HeaderAlignment : TAlignment
               Read FHeaderAlignment
               Write SetHeaderAlignment
               Default taLeftJustify ;
      Property HeaderColor : TColor
               Read GetHeaderColor
               Write SetHeaderColor
               Stored StoreAppearanceProperties ;
      Property HeaderFont : TFont
               Read FHeaderFont
               Write SetHeaderFont ;
      Property HeaderFrameColor : TColor
               Read GetHeaderFrameColor
               Write SetHeaderFrameColor
               Stored StoreAppearanceProperties ;
      Property HeaderFrameStyle : TjmGridFrameStyle
               Read GetHeaderFrameStyle
               Write SetHeaderFrameStyle
               Stored StoreAppearanceProperties ;
      Property HeaderText : String
               Read FHeaderText
               Write SetHeaderText ;
      Property HeaderVerticalAlignment : TjmGridVerticalAlignment
               Read FHeaderVerticalAlignment
               Write SetHeaderVerticalAlignment
               Default vaTop ;
      Property Hidden : Boolean
               Read FHidden
               Write SetHidden
               Default False ;

      Property InactiveColor : TColor
               Read GetInactiveColor
               Write SetInactiveColor
               Stored StoreAppearanceProperties ;
      Property InactiveFont : TFont
               Read FInactiveFont
               Write SetInactiveFont ;

      Property OverrideSettings : TjmGridOverrideSettings
               Read FOverride
               Write SetOverride
               Default [] ;
      Property ReadOnly : Boolean
               Read GetReadOnly
               Write SetReadOnly
               Default False ;
      Property SelectColor : TColor
               Read GetSelectColor
               Write SetSelectColor
               Stored StoreAppearanceProperties ;
      Property SelectFont : TFont
               Read FSelectFont
               Write SetSelectFont ;
      Property VerticalAlignment : TjmGridVerticalAlignment
               Read FVerticalAlignment
               Write SetVerticalAlignment
               Default vaTop ;
      Property VerticalLineColor : TColor
               Read GetVerticalLineColor
               Write SetVerticalLineColor
               Stored StoreAppearanceProperties ;
      Property VerticalLineStyle : TjmGridLineStyle
               Read GetVerticalLineStyle
               Write SetVerticalLineStyle
               Stored StoreAppearanceProperties ;
      Property Width : Cardinal
               read FWidth
               Write SetWidth
               Default jmDefaultColumnWidth ;


    End ; {*** TjmGridColumn ***}

