{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the implementation for th TjmGridColumn class's
  methods. This objects of this class represents a column in the grid. 
}

Function EditableFieldType (FieldType : TFieldType) : Boolean ;
  Begin
  Case FieldType Of
    ftString, ftSmallint, ftInteger, ftWord, ftBoolean, ftFloat,
    ftCurrency, ftBCD, ftDate, ftTime, ftDateTime,
    ftBytes, ftVarBytes, ftMemo:
      Begin
      Result := True ;
      End
    Else
      Begin
      Result := False ;
      End ;
    End ;
  End ;

Function TjmGridColumn.GetCellText (RecordNumber : LongInt) : String ;
  Var
    OldCurrent : LongInt ;
  Begin
  If (FColumnType = gctUnbound) Or (ParentGrid.Mode = gmUnbound) Then
    Begin
    If csDesigning In ParentGrid.ComponentState Then
      Begin
      { In design mode give the user a visual clue. }
      Result := Format ('(%d, %d)', [FColumnNumber, RecordNumber]) ;
      End
    Else If Assigned (ParentGrid.FOnUnboundFetch) Then
      Begin
      ParentGrid.OnUnboundFetch (ParentGrid, ParentZone.ZoneNumber, ColumnNumber, RecordNumber, Result) ;
      End
    Else
      Begin
      Result := '' ;
      End ;
    End
  Else If ParentGrid.UsingDatabase Then
    Begin
    OldCurrent := ParentGrid.FDataLink.ActiveRecord ;
    ParentGrid.FDataLink.ActiveRecord := RecordNumber - 1 ;
    If Field <> Nil Then
      Begin
      Try
        Case Field.DataType Of
          ftMemo:
            Try
              Result := Field.AsString ;
            Except
              Result := Field.DisplayText ;
              End ;
          Else
            Result := Field.DisplayText
          End ;
      Except
        Result := '' ;
        End ;
      End
    Else
      Result := '' ;
    ParentGrid.FDataLink.ActiveRecord := OldCurrent ;
    End
  Else
    Begin
    Result := '' ;
    End ;
  End ;

Function TjmGridColumn.GetCellEditText (RecordNumber : LongInt) : String ;
  Var
    OldCurrent : LongInt ;
  Begin
  If (FColumnType = gctUnbound) Or (ParentGrid.Mode = gmUnbound) Then
    Begin
    If Assigned (ParentGrid.FOnUnboundEditFetch) Then
      Begin
      ParentGrid.OnUnboundEditFetch (ParentGrid, ParentZone.ZoneNumber, ColumnNumber, RecordNumber, Result) ;
      End
    Else If Assigned (ParentGrid.FOnUnboundFetch) Then
      Begin
      { If OnUnboundEditFetch has not been assigned then use OnUnboundFetch if possible. }
      ParentGrid.OnUnboundFetch (ParentGrid, ParentZone.ZoneNumber, ColumnNumber, RecordNumber, Result) ;
      End
    Else
      Begin
      Result := '' ;
      End ;
    End
  Else If ParentGrid.UsingDatabase Then
    Begin
    OldCurrent := ParentGrid.FDataLink.ActiveRecord ;
    ParentGrid.FDataLink.ActiveRecord := RecordNumber - 1 ;
    If Field <> Nil Then
      Begin
      Try
        Case Field.DataType Of
          ftMemo:
            Try
              Result := Field.AsString ;
            Except
              Result := Field.Text ;
              End ;
          Else
            Result := Field.Text
          End ;
      Except
        Result := '' ;
        End ;
      End
    Else
      Result := '' ;
    ParentGrid.FDataLink.ActiveRecord := OldCurrent ;
    End
  Else
    Begin
    Result := '' ;
    End ;
  End ;

Function TjmGridColumn.GetHeaderText : String ;
  Begin
  If FHeaderText <> '' Then
    Result := FHeaderText
  Else If (ParentGrid.Mode = gmDatabase) And (FColumnType = gctBound) Then
    Result := FFieldName
  Else
    Result := '' ;
  End ;

Procedure TjmGridColumn.SetHeaderText (Text : String) ;
  Begin
  If Text = FHeaderText Then
    Exit ;
  FHeaderText := Text ;

  If Not (csLoading in ComponentState) Then
    RepaintHeader ;
  End ;

Function TjmGridColumn.GetFooterText : String ;
  Begin
  Result := FFooterText
  End ;

Procedure TjmGridColumn.SetFooterText (Text : String) ;
  Begin
  If Text = FFooterText Then
    Exit ;
  FFooterText := Text ;
  If Not (csLoading in ComponentState) Then
    RepaintFooter ;
  End ;

Procedure TjmGridColumn.SetColumnType (AType : TjmGridColumnType) ;
  Begin
  If FColumnType = AType Then
    Exit ;

  FColumnType := AType ;
  LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetFieldName (Text : String) ;
  Begin
  If Text = FFieldName Then
    Exit ;

  FFieldName := Text ;
  If (ParentGrid.Mode = gmDatabase) And (FColumnType = gctBound) Then
    LayoutChanged ;

  End ;

Function TjmGridColumn.UpdateValue (Value : String) : Boolean ;
  Begin
  Result := False ;
  If (ParentGrid.UsingDatabase) And (FColumnType = gctBound) Then
    Begin
    Try
      ParentGrid.FDataLink.UpdateData ;
    Except
      On EAbort Do Result := False ;
      End ;
    End
  Else
    Begin
    If Assigned (ParentGrid.FOnValidateCell) Then
      Begin
      ParentGrid.FOnValidateCell (ParentGrid, Value, Result) ;
      If Result Then
        Exit ;
      End ;
    End ;
  End ;

Function TjmGridColumn.GetEditor : TWinControl ;
  Begin
  If ReadOnly Or ParentZone.ReadOnly Or ParentGrid.ReadOnly Then
    Begin
    { If the column is readonly then it cannot have an editor. }
    Result := Nil ;
    End
  Else If (FColumnType = gctBound) And (Field = Nil) Then
    Begin
    { If the column is bound to a nonexistant field the dataset is inactive
      the column cannot have an editor. }
    Result := Nil ;
    End
  Else
    Begin
    If FEditorType = etDefault Then
      Begin
      Result := ParentGrid.FDefaultEditor
      End
    Else
      Begin
      CreateEditor ;
      Result := FEditor ;
      End ;
    End ;
  End ;

Procedure TjmGridColumn.CreateEditor ;
  Begin
  If FEditor = Nil Then
    Begin
    Case FEditorType Of
      etEdit:
        FEditor := TjmGridEdit.Create (Nil) ;
      etMemo:
        FEditor := TjmGridMemo.Create (Nil) ;
      etMaskEdit:
        FEditor := TjmGridMaskEdit.Create (Nil) ;
      etComboBox:
        FEditor := TjmGridComboBox.Create (Nil) ;
      Else
        Raise Exception.Create ('Internal Error - Unknown editor type') ;
      End ;
    End ;
    FEditor.Parent := ParentZone ;
  End ;

Procedure TjmGridColumn.SetEditorType (AType : TjmGridEditorType) ;
  Begin
  If AType = FEditorType Then
    Exit ;

  FEditor.Free ;
  FEditorType := AType ;
  End ;

Function TjmGridColumn.GetReadOnly : Boolean ;
  Begin
  If FReadOnly Then
    Begin
    Result := true
    End
  Else If ColumnType = gctBound Then
    Begin
    If Parentgrid.UsingDatabase Then
      Begin
      If Field <> Nil Then
        Begin
        If Field.ReadOnly Or (Field Is TBlobField) Then
          Begin
          Result := True
          End
        Else
          Begin
          Result := False ;
          End ;
        End
      Else
        Begin
        { Bound column with no field assigned. }
        Result := True ;
        End ;
      End
    Else
      Begin
      { Bound field is read only if not using the database. }
      Result := True ;
      End ;
    End
  Else
    Begin
    Result := False ;
    End ;
  End ;

{
  Function: CreateAlignmentFlags

  Description:
    This is a simple routine for converting the horizontal and vertical
    alignment specifiers for a column into the flags used by the
    DrawText API function.

  Paramters:
    HAlign : Column Horizontal Alignment
    VAlign : Column Vertical Alignment

  Return Value:
    The flags setting for DrawText.

}

Function CreateAlignmentFlags (HAlign : TAlignment ;
                               VAlign : TjmGridVerticalAlignment) : Integer ;
  Begin
  Result := DT_NOPREFIX ;  { Make sure "&" does not get converted to "_" }

  Case HAlign Of
    taLeftJustify:
      Result := Result Or DT_LEFT ;
    taRightJustify:
      Result := Result Or DT_RIGHT ;
    taCenter:
      Result := Result Or DT_CENTER ;
    End ;
  Case VAlign Of
    vaTop:
      Result := Result Or DT_TOP OR DT_WORDBREAK ;
    vaMiddle:
      Result := Result Or DT_VCENTER OR DT_SINGLELINE ;
    vaBottom:
      Result := Result Or DT_BOTTOM OR DT_SINGLELINE;
    End ;
  End ;

Procedure TjmGridColumn.DrawCell (ARecord : LongInt ;
                                  ACanvas : TCanvas ;
                                  const ARect : TRect) ;
  Var
    Text : String ;
    Flags : Integer ;
    DrawRect : TRect ;
    Draw : Boolean ;
  begin
  DrawRect := ARect ;
  AdjustRectForGutter (DrawRect) ;
  Text := GetCellText (ARecord) ;

  If Not ParentGrid.IsCellSelected (ParentZone.ZoneNumber, ColumnNumber, ARecord) Then
    Begin
    ACanvas.Font.Assign (EffectiveFont) ;
    End
  Else
    Begin
    ACanvas.Font.Assign (EffectiveSelectFont) ;
    End ;
  ACanvas.Brush.Style := bsClear ;

  If Assigned (ParentGrid.OnDrawCell) Then
    Begin
    Draw := True ;
    ParentGrid.OnDrawCell (ParentGrid,
                           ParentZone.ZoneNumber,
                           ColumnNumber,
                           ARecord,
                           Text,
                           ARect,
                           ACanvas,
                           Draw) ;
    If Not Draw Then
      Exit ;
    End ;

  Flags := CreateAlignmentFlags (FAlignment, FVerticalAlignment) ;
  DrawText (ACanvas.Handle,
            PChar(Text),
            Length (Text),
            DrawRect,
            Flags) ;
  End ;

Procedure TjmGridColumn.DrawCellBackground (ARecord : LongInt ;
                                                  ACanvas : TCanvas ;
                                                  const ARect : TRect) ;
  Var
    Draw : Boolean ;
    FStyle : TjmGridFrameStyle ;
    FColor : TColor ;
  begin
  If Not ParentGrid.EditorVisible
    And ParentGrid.IsCellBoxed (ParentZone.ZoneNumber, ColumnNumber, ARecord) Then
    Begin
    FStyle := ActiveFrameStyle ;
    FColor := ActiveFrameColor ;
    End
  Else
    Begin
    FStyle := CellFrameStyle ;
    FColor := CellFrameColor ;
    End ;

  { All the canvas settings have to be done before calling the event
    procedure since we allow the caller to change those settings within
    the event.
  }
  If ParentGrid.IsCellSelected (ParentZone.ZoneNumber, ColumnNumber, ARecord) Then
    ACanvas.Brush.Color := SelectColor
  Else
    ACanvas.Brush.Color := BackgroundColor ;
  ACanvas.Brush.Style := bsSolid ;

  If Assigned (ParentGrid.FOnDrawCellBackGround) Then
    Begin
    Draw := True ;
    ParentGrid.FOnDrawCellBackGround (ParentGrid,
                                      ParentZone.ZoneNumber,
                                      ColumnNumber,
                                      ARecord,
                                      ARect,
                                      ACanvas,
                                      Draw) ;
    If Not Draw Then
      Exit ;
    End ;

  ACanvas.FillRect (ARect) ;
  DrawFrame (FStyle, FColor, ARect, ACanvas) ;

  End ;

Procedure TjmGridColumn.DrawFooter (ACanvas : TCanvas ;
                                    const ARect : TRect) ;
  Var
    Text : String ;
    Flags : Integer ;
    DrawRect : TRect ;
    Draw : Boolean ;
  begin
  DrawRect := ARect ;
  AdjustRectForGutter (DrawRect) ;
  Text := GetFooterText ;
  ACanvas.Brush.Style := bsClear ;
  If Assigned (ParentGrid.OnDrawFooter) Then
    Begin
    Draw := True ;
    ParentGrid.OnDrawFooter (ParentGrid,
                             ParentZone.ZoneNumber,
                             ColumnNumber,
                             Text,
                             ARect,
                             ACanvas,
                             Draw) ;
    If Not Draw Then
      Exit ;
    End ;

  Flags := CreateAlignmentFlags (FFooterAlignment, FFooterVerticalAlignment) ;
  DrawText (ACanvas.Handle,
            PChar(Text),
            Length (Text),
            DrawRect,
            Flags) ;
  End ;

Procedure TjmGridColumn.DrawHeader (ACanvas : TCanvas ; const ARect : TRect) ;
  Var
    Text : String ;
    Flags : Integer ;
    DrawRect : TRect ;
    Draw : Boolean ;
  begin
  DrawRect := ARect ;
  AdjustRectForGutter (DrawRect) ;
  Text := GetHeaderText ;
  ACanvas.Brush.Style := bsClear ;

  If Assigned (ParentGrid.OnDrawHeader) Then
    Begin
    Draw := True ;
    ParentGrid.OnDrawHeader (ParentGrid,
                           ParentZone.ZoneNumber,
                           ColumnNumber,
                           Text,
                           ARect,
                           ACanvas,
                           Draw) ;
    If Not Draw Then
      Exit ;
    End ;

  Flags := CreateAlignmentFlags (FHeaderAlignment, FHeaderVerticalAlignment) ;

  DrawText (ACanvas.Handle,
            PChar(Text),
            Length (Text),
            DrawRect,
            Flags) ;
 End ;

Function TjmGridColumn.GetBackgroundColor : TColor ;
  Begin
  If osBackgroundColor In FOverride Then
    Result := FBackgroundColor
  Else
    Result := ParentZone.Rows [ParentZone.ColumnAddressToRow (Self)].BackgroundColor ;
  End ;

Function TjmGridColumn.GetFooterColor : TColor ;
  Begin
  If osFooterColor In FOverride Then
    Result := FFooterColor
  Else
    Result := ParentZone.FooterColor ;
  End ;

Function TjmGridColumn.GetHeaderColor : TColor ;
  Begin
  If osHeaderColor In FOverride Then
    Result := FHeaderColor
  Else
    Result := ParentZone.HeaderColor ;
  End ;

Function TjmGridColumn.GetInactiveColor : TColor ;
  Begin
  If osInactiveColor In FOverride Then
    Result := FInactiveColor
  Else
    Result := ParentZone.InactiveColor ;
  End ;

Function TjmGridColumn.GetSelectColor : TColor ;
  Begin
  If osSelectColor In FOverride Then
    Result := FSelectColor
  Else
    Result := ParentZone.SelectColor ;
  End ;

Procedure TjmGridColumn.SetBackgroundColor (AColor : TColor) ;
  Begin
  If (osBackgroundColor In FOverride) And (AColor = FBackgroundColor) Then
    Exit ;

  FBackgroundColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osBackgroundColor) ;

  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetFooterColor (AColor : TColor) ;
  Begin
  If (osFooterColor In FOverride) And (AColor = FFooterColor) Then
    Exit ;

  FFooterColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterColor) ;

  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetHeaderColor (AColor : TColor) ;
  Begin
  If (osHeaderColor In FOverride) And (AColor = FHeaderColor) Then
    Exit ;

  FHeaderColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderColor) ;

  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetInactiveColor (AColor : TColor) ;
  Begin
  If (osInactiveColor In FOverride) And (AColor = FInactiveColor) Then
    Exit ;

  FInactiveColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osInactiveColor) ;

  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetSelectColor (AColor : TColor) ;
  Begin
  If (osSelectColor In FOverride) And (AColor = FSelectColor) Then
    Exit ;

  FSelectColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osSelectColor) ;

  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetVerticalLineStyle : TjmGridLineStyle ;
  Begin
  If osVerticalLineStyle In FOverride Then
    Result := FVerticalLineStyle
  Else
    Result := ParentZone.VerticalLineStyle ;
  End ;

Procedure TjmGridColumn.SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If (osVerticalLineStyle In FOverride) And (FVerticalLineStyle = AStyle) THen
    Exit ;

  FVerticalLineStyle := AStyle ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osVerticalLineStyle) ;

  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetVerticalLineColor : TColor ;
  Begin
  If osVerticalLineColor In FOverride Then
    Result := FVerticalLineColor
  Else
    Result := ParentZone.VerticalLineColor ;
  End ;

Procedure TjmGridColumn.SetVerticalLineColor (AColor : TColor) ;
  Begin
  If (osVerticalLineColor In FOverride) And (FVerticalLineColor = AColor) Then
    Exit ;

  FVerticalLineColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osVerticalLineColor) ;
  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.DrawHeaderBackground (ACanvas : TCanvas ;
                                              const ARect : TRect) ;
  Var
    Draw : Boolean ;
  Begin
  If Assigned (ParentGrid.OnDrawHeaderBackground) Then
    Begin
    Draw := True ;
    ParentGrid.OnDrawHeaderBackGround (ParentGrid,
                                       ParentZone.ZoneNumber,
                                       ColumnNumber,
                                       ARect,
                                       ACanvas,
                                       Draw) ;
    If Not Draw Then
      Exit ;
    End ;
  ACanvas.FillRect (ARect) ;
  DrawFrame (HeaderFrameStyle, HeaderFrameColor, ARect, ACanvas) ;
  End ;

Procedure TjmGridColumn.DrawFooterBackground (ACanvas : TCanvas ;
                                                    const ARect : TRect) ;
  Var
    Draw : Boolean ;
  Begin
  If Assigned (ParentGrid.OnDrawFooterBackground) Then
    Begin
    Draw := True ;
    ParentGrid.OnDrawFooterBackGround (ParentGrid,
                                       ParentZone.ZoneNumber,
                                       ColumnNumber,
                                       ARect,
                                       ACanvas,
                                       Draw) ;
    If Not Draw Then
      Exit ;
    End ;
  ACanvas.FillRect (ARect) ;
  DrawFrame (FooterFrameStyle, FooterFrameColor, ARect, ACanvas) ;
  End ;

Procedure TjmGridColumn.SetSelectFont (const AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FSelectFont.Free ;
    FSelectFont := Nil
    End
  Else
    Begin
    If FSelectFont = Nil Then
      Begin
      FSelectFont := TFont.Create ;
      FSelectFont.OnChange := FontChanged ;
      End ;
    FSelectFont.Assign (AFont) ;
    End ;
  End ;

Procedure TjmGridColumn.SetHeaderFont (const AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FHeaderFont.Free ;
    FHeaderFont := Nil
    End
  Else
    Begin
    If FHeaderFont = Nil Then
      Begin
      FHeaderFont := TFont.Create ;
      FHeaderFont.OnChange := FontChanged ;
      End ;
    FHeaderFont.Assign (AFont) ;
    End ;
  End ;

Procedure TjmGridColumn.SetFooterFont (const AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FFooterFont.Free ;
    FFooterFont := Nil
    End
  Else
    Begin
    If FFooterFont = Nil Then
      Begin
      FFooterFont := TFont.Create ;
      FFooterFont.OnChange := FontChanged ;
      End ;
    FFooterFont.Assign (AFont) ;
    End ;
  End ;

Procedure TjmGridColumn.SetInactiveFont (const AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FInactiveFont.Free ;
    FInactiveFont := Nil
    End
  Else
    Begin
    If FInactiveFont = Nil Then
      Begin
      FInactiveFont := TFont.Create ;
      FInactiveFont.OnChange := FontChanged ;
      End ;
    FInactiveFont.Assign (AFont) ;
    End ;
  End ;

Function TjmGridColumn.EffectiveFont : TFont ;
  Var
    Row : Cardinal ;
    Position : Cardinal ;
  Begin
  If FFont <> Nil Then
    Begin
    Result := FFont
    End
  Else
    Begin
    ParentZone.ColumnToRow (FColumnNumber, Row, Position) ;
    Result := ParentZone.Rows [Row].EffectiveFont ;
    End ;
  End ;

Function TjmGridColumn.EffectiveSelectFont : TFont ;
  Begin
  If FSelectFont <> Nil Then
    Result := FSelectFont
  Else
    Result := ParentZone.EffectiveSelectFont ;
  End ;

Function TjmGridColumn.EffectiveInactiveFont : TFont ;
  Begin
  If FInactiveFont <> Nil Then
    Result := FInactiveFont
  Else
    Result := ParentZone.EffectiveInactiveFont ;
  End ;

Function TjmGridColumn.EffectiveHeaderFont : TFont ;
  Begin
  If FHeaderFont <> Nil Then
    Result := FHeaderFont
  Else
    Result := ParentZone.EffectiveHeaderFont ;
  End ;

Function TjmGridColumn.EffectiveFooterFont : TFont ;
  Begin
  If FFooterFont <> Nil Then
    Result := FFooterFont
  Else
    Result := ParentZone.EffectiveFooterFont ;
  End ;

Procedure TjmGridColumn.AdjustRectForGutter (Var ARect : TRect) ;
  Begin
  Inc (ARect.Left, FGutterSize) ;
  Inc (ARect.Top, FGutterSize) ;
  Dec (ARect.Right, FGutterSize) ;
  Dec (ARect.Bottom, FGutterSize) ;
  End ;

Function TjmGridColumn.GetParentGrid : TColossalGrid ;
  Begin
  Result := ParentZone.ParentGrid ;
  End ;

Procedure TjmGridColumn.SetWidth (AWidth : Cardinal) ;
  Begin
  If AWidth = FWidth Then
    Exit ;
  FWidth := AWidth ;
  If Not Hidden Then
    ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetHidden (State : Boolean) ;
  Begin
  If State = FHidden Then
    Exit ;

  FHidden := State ;
  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetAlignment (const Align : TAlignment) ;
  Begin
  If Align = FAlignment Then
    Exit ;

  FAlignment := Align ;
  If Not Hidden Then
    ParentZone.LayoutChanged ;

  End ;

Procedure TjmGridColumn.SetVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
  Begin
  If FVerticalAlignment = AValue Then
    Exit ;

  FVerticalAlignment := AValue ;

  If Not Hidden Then
    ParentZone.LayoutChanged ;

  End ;

Procedure TjmGridColumn.LayoutChanged ;
  Begin
  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.RepaintHeader ;
  Begin
  If Not (csLoading in ComponentState) Then
    ParentZone.RedrawHeaderColumn (ColumnNumber) ;
  End ;

Procedure TjmGridColumn.RepaintFooter ;
  Begin
  ParentZone.RedrawFooterColumn (ColumnNumber) ;
  End ;

Procedure TjmGridColumn.SetFooterAlignment (Align : TAlignment) ;
  Begin
  If Align = FFooterAlignment Then
    Exit ;

  FFooterAlignment := Align ;
  If Not (csLoading In ComponentState) Then
    RepaintFooter ;
  End ;

Procedure TjmGridColumn.SetFooterVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
  Begin
  If AValue = FFooterVerticalAlignment Then
    Exit ;

  FFooterVerticalAlignment := AValue ;
  If Not (csLoading In ComponentState) Then
    RepaintFooter ;
  End ;

Procedure TjmGridColumn.SetHeaderAlignment (Align : TAlignment) ;
  Begin
  If Align = FHeaderAlignment Then
    Exit ;

  FHeaderAlignment := Align ;
  RepaintHeader ;
  End ;

Procedure TjmGridColumn.SetHeaderVerticalAlignment (AValue : TjmGridVerticalAlignment) ;
  Begin
  If AValue = FHeaderVerticalAlignment Then
    Exit ;

  FHeaderVerticalAlignment := AValue ;
  If Not (csLoading In ComponentState) Then
    RepaintHeader ;
  End ;

Procedure TjmGridColumn.SetFont (const AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FFont.Free ;
    FFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FFont = Nil Then
      Begin
      FFont := TFont.Create ;
      FFont.OnChange := FontChanged ;
      End ;
    FFont.Assign (AFont) ;
    End ;

  End ;

Procedure TjmGridColumn.FontChanged (Sender : TObject) ;
  Begin
  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.SetOverride (AValue : TjmGridOverrideSettings) ;
  Begin
  FOverride := AValue ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.StoreAppearanceProperties : Boolean ;
  Begin
  If OverrideSettings = [] Then
    Result := False
  Else
    Result := True ;
  End ;

Procedure TjmGridColumn.SetReadOnly (AValue : Boolean) ;
  Begin
  FreadOnly := AValue ;
  End ;

Function TjmGridColumn.GetCellFrameColor : TColor ;
  Begin
  If osCellFrameColor In FOverride Then
    Result := FCellFrameColor
  Else
    Result := ParentZone.CellFrameColor ;
  End ;

Procedure TjmGridColumn.SetCellFrameColor (AColor : TColor) ;
  Begin
  If (osCellFrameColor In FOverride) And (AColor = FCellFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osCellFrameColor) ;
  FCellFrameColor := AColor ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetCellFrameStyle : TjmGridFrameStyle ;
  Begin
  If osCellFrameStyle In FOverride Then
    Result := FCellFrameStyle
  Else
    Result := ParentZone.CellFrameStyle ;
  End ;

Procedure TjmGridColumn.SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osCellFrameStyle In FOverride) And (AStyle = FCellFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osCellFrameStyle) ;
    FCellFrameStyle := AStyle ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetHeaderFrameColor : TColor ;
  Begin
  If osHeaderFrameColor In FOverride Then
    Result := FHeaderFrameColor
  Else
    Result := ParentZone.HeaderFrameColor ;
  End ;

Procedure TjmGridColumn.SetHeaderFrameColor (AColor : TColor) ;
  Begin
  If (osHeaderFrameColor In FOverride) And (AColor = FHeaderFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderFrameColor) ;
  FHeaderFrameColor := AColor ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetHeaderFrameStyle : TjmGridFrameStyle ;
  Begin
  If osHeaderFrameStyle In FOverride Then
    Result := FHeaderFrameStyle
  Else
    Result := ParentZone.HeaderFrameStyle ;
  End ;

Procedure TjmGridColumn.SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osHeaderFrameStyle In FOverride) And (AStyle = FHeaderFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderFrameStyle) ;
  FHeaderFrameStyle := AStyle ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetFooterFrameColor : TColor ;
  Begin
  If osFooterFrameColor In FOverride Then
    Result := FFooterFrameColor
  Else
    Result := ParentZone.FooterFrameColor ;
  End ;

Procedure TjmGridColumn.SetFooterFrameColor (AColor : TColor) ;
  Begin
  If (osFooterFrameColor In FOverride) And (AColor = FFooterFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterFrameColor) ;
  FFooterFrameColor := AColor ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetFooterFrameStyle : TjmGridFrameStyle ;
  Begin
  If osFooterFrameStyle in FOverride Then
    Result := FFooterFrameStyle
  Else
    Result := ParentZone.FooterFrameStyle ;
  End ;

Procedure TjmGridColumn.SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osFooterFrameStyle In FOverride) And (AStyle = FFooterFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterFrameStyle) ;
  FFooterFrameStyle := AStyle ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetActiveFrameColor : TColor ;
  Begin
  If osActiveFrameColor In FOverride Then
    Result := FActiveFrameColor
  Else
    Result := ParentZone.ActiveFrameColor ;
  End ;

Procedure TjmGridColumn.SetActiveFrameColor (AColor : TColor) ;
  Begin
  If (osActiveFrameColor In FOverride) And (AColor = FActiveFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osActiveFrameColor) ;
  FActiveFrameColor := AColor ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridColumn.GetActiveFrameStyle : TjmGridFrameStyle ;
  Begin
  If osActiveFrameStyle In FOverride Then
    Result := FActiveFrameStyle
  Else
    Result := ParentZone.ActiveFrameStyle ;
  End ;

Procedure TjmGridColumn.SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osActiveFrameStyle In FOverride) And (AStyle = FActiveFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osActiveFrameStyle) ;
  FActiveFrameStyle := AStyle ;
  ParentZone.LayoutChanged ;
  End ;

Procedure TjmGridColumn.FindField ;
  Begin
  If ParentGrid.UsingDatabase And (FColumnType = gctBound) Then
    Begin
    FField := ParentGrid.FDataLink.DataSet.FindField (FFieldName) ;
    End
  Else
    Begin
    FField := Nil ;
    End ;
  End ;

Function TjmGridColumn.SaveToDatabase (Value : String) : Boolean ;
  Begin
  Result := False ;
  If Assigned (ParentGrid.FOnValidateCell) And (Field <> Nil) Then
    Begin
    ParentGrid.FOnValidateCell (ParentGrid, Value, Result) ;
    End ;

  If Not Result Then
    Field.AsString := Value ;
  End ;


