unit ColGrid;
{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}


{$INCLUDE COMMON}

interface

Uses
  Windows, Graphics, Classes, Controls, Messages, StdCtrls, Mask, DB, Forms,
  SysUtils, Dialogs ;

Type
  TjmGridDataLinkState = (dlNormal, dlAppend, dlEdit) ;


  TjmGridOverrideSettings = Set Of (osVerticalLineColor,
                                    osVerticalLineStyle,
                                    osBackgroundColor,
                                    osFooterColor,
                                    osHeaderColor,
                                    osUnusedColor,
                                    osInactiveColor,
                                    osSelectColor,
                                    osHorizontalLineColor,
                                    osRowHeight,
                                    osRowLineColor,
                                    osRowLineStyle,
                                    osRowCount,
                                    osCellFrameColor,
                                    osCellFrameStyle,
                                    osHeaderFrameColor,
                                    osHeaderFrameStyle,
                                    osFooterFrameColor,
                                    osFooterFrameStyle,
                                    osActiveFrameColor,
                                    osActiveFrameStyle) ;


  TjmGridColumnType = (gctUnbound, gctBound) ;
  TjmGridMode = (gmUnbound, gmDatabase) ;
  TjmGridSelectMode = (smCell, smRecord, smCellAndRecord, smNone) ;
  TjmGridDataLayoutMode = (lmAuto, lmManual) ;
  TjmGridEditorType = (etDefault, etEdit, etMemo, etMaskEdit, etComboBox) ;

  TjmGridFrameStyle = (NoFrame, SolidFrame, DashFrame, DotFrame, RaisedFrame, InsetFrame) ;

  { Grid Definitions }

  TjmGridLineStyle = (lsNone, lsSingle, lsDouble, ls3D) ;
  TjmGridScrollBars = (sbNever, sbAlways, sbWhenNeeded) ;
  TjmGridVerticalAlignment = (vaTop, vaMiddle, vaBottom) ;

  TjmGridZoneState = (gsNormal, gsColumnSizing, gsColumnMoving) ;

  TjmGridLocation = (glOutside,    { Outside the object in question }
                     glUnused,     { Unused Grid Area }
                     glZone,
                     glZoneSeparator,
                     glRecordSeparator,
                     glHeaderSeparator,
                     glCell,
                     glHeader,
                     glFooter,
                     glFooterSeparator,
                     glVerticalScrollbar,
                     glHorizontalScrollBar) ;

  TjmGridColumnLocation = (gclNone,
                           gclBody,
                           gclSeparator) ;


  TjmGridPosition = Record
    GridLocation : TjmGridLocation ;
    ColumnLocation : TjmGridColumnLocation ;
    Zone : Cardinal ;
    Column : Cardinal ;
    RecordNumber : LongInt ;
    End ;

  TjmGridCell = Record
    Zone : Cardinal ;
    Column : Cardinal ;
    RecordNumber : LongInt ;
    End ;

  TjmGridAppendEvent = Procedure (Sender : TObject) Of Object ;

  TjmGridValidateCellEvent = Procedure (Sender : TObject ;
                                    Text : String ;
                                    var Cancel : Boolean) Of Object ;

  TjmGridColumnMovedEvent = Procedure (Sender : TObject ;
                                       Zone : Cardinal ;
                                       NewColumn : Cardinal ;
                                       OldColumn : Cardinal) Of Object ;

  TjmGridDrawCellEvent = Procedure (Sender : TObject ;
                                    Zone : Cardinal ;
                                    Column : Cardinal ;
                                    RecordNumber : LongInt ;
                                    Text : String ;
                                    Rect : TRect ;
                                    Canvas : TCanvas ;
                                    var Draw : Boolean) Of Object ;

  TjmGridDrawCellBackgroundEvent = Procedure (Sender : TObject ;
                                              Zone : Cardinal ;
                                              Column : Cardinal ;
                                              RecordNumber : LongInt ;
                                              Rect : TRect ;
                                              Canvas : TCanvas ;
                                              var Draw : Boolean) Of Object ;

  TjmGridDrawHeaderEvent = Procedure (Sender : TObject ;
                                      Zone : Cardinal ;
                                      Column : Cardinal ;
                                      Text : String ;
                                      Rect : TRect ;
                                      Canvas : TCanvas ;
                                      var Draw : Boolean) Of Object ;

  TjmGridDrawHeaderBackgroundEvent = Procedure (Sender : TObject ;
                                                Zone : Cardinal ;
                                                Column : Cardinal ;
                                                Rect : TRect ;
                                                Canvas : TCanvas ;
                                                var Draw : Boolean) Of Object ;

  TjmGridLeaveCellEvent = Procedure (Sender : TObject ;
                                     NewZone : Cardinal ;
                                     NewColumn : Cardinal ;
                                     NewRecordNumber : LongInt ;
                                     var Cancel : Boolean) Of Object ;

  TjmGridLeftOffsetChangedEvent = Procedure (Sender : TObject ;
                                             Zone : Cardinal) Of Object ;

  TjmCellFetchEvent =  Procedure (Sender : TObject ;
                                 Zone : Integer ;
                                 Column : Integer ;
                                 RecordNumber : LongInt ;
                                 Var Value : String) Of Object ;



{
  These constants define default property values.  We have them up here rather
  than in the constructor.
}

const
  jmDefaultColumnWidth = 40 ;
  jmDefaultRowHeight = 20 ;
  jmDefaultUnusedColor = cl3DLight ;

  jmDefaultBorderStyle = bsSingle ;
  jmDefaultHorizontalLineColor = clBlack ;
  jmDefaultHorizontalLineStyle = lsDouble ;
  jmDefaultVerticalLineStyle = ls3D ;
  jmDefaultVerticalLineColor = clBlack ;
  jmDefaultBackgroundColor = clInfoBk ;
  jmDefaultHeaderColor = clActiveCaption ;
  jmDefaultFooterColor = clBtnFace ;
  jmDefaultInactiveColor = clInactiveCaption ;

  jmDefaultHorizontalScrollBarSeparatorStyle = lsSingle ;
  jmDefaultHorizontalScrollBarSeparatorColor = clBlack ;
  jmDefaultVerticalScrollBarSeparatorStyle = lsSingle ;
  jmDefaultVerticalScrollBarSeparatorColor = clBlack ;

  jmDefaultZoneSeparatorStyle = lsDouble ;
  jmDefaultZoneSeparatorColor = clBlack ;

  jmDefaultHeaderSeparatorStyle = lsSingle ;
  jmDefaultHeaderSeparatorColor = clBlack ;
  jmDefaultFooterSeparatorStyle = lsSingle ;
  jmDefaultFooterSeparatorColor = clBlack ;

  jmDefaultSelectColor = clHighlight ;
  jmDefaultHeaderSelectColor = clHighlight ;
  jmDefaultFooterSelectColor = clHighlight ;
  jmDefaultRowLineStyle = lsSingle ;
  jmDefaultRowLineColor = clRed ;

  jmDefaultColumnType = gctUnbound ;
  jmDefaultGridMode = gmUnbound ;
  jmDefaultSelectMode = smCell ;
  jmDefaultLayoutMode = lmAuto ;

Type
  TjmGridZone = Class ;
  TjmGridColumn = Class ;
  TjmGridRow = Class ;
  TColossalGrid = Class ;
  TjmGridZoneList = Class ;
  TjmGridRowList = Class ;
  TjmGridColumnList = Class ;

{$INCLUDE LISTDEFINITION}
{$INCLUDE COLUMNDEFINITION}
{$INCLUDE ROWDEFINITION}
{$INCLUDE ZONEDEFINITION}
{$INCLUDE EDITORDEFINITION}
{$INCLUDE DATALINKDEFINITION}
{$INCLUDE GRIDDEFINITION}

Implementation

Uses
  Winprocs, DBTables, Bde ;

Const
  DoubleLineDelta : Integer = 3 ;
  GridVersion = '1-005' ;

{
  Function: GetEditorText

  Description:  Returns the text from a grid editor.

  Parameters:
    Editor : The grid editor

  Return Value:
    The editor's text string
}

Function GetEditorText (Editor : TWinControl) : String ;
  Begin
  If Editor Is TjmGridEdit Then
    Result := (Editor As TjmGridEdit).Text
  Else If Editor Is TjmGridMemo Then
    Result := (Editor AS TjmGridMemo).Lines.Text
  Else If Editor Is TjmGridMaskEdit Then
    Result := (Editor AS TjmGridMaskEdit).Text
  Else If Editor Is TjmGridComboBox Then
    Result := (Editor AS TjmGridComboBox).Text
  Else
    Raise Exception.Create ('Internal Error - Unknown Editor Type') ;
  End ;

{
  Procedure: SetEditorText

  Description:  Sets text in a grid editor.

  Parameters:
    Editor : The grid editor
    Text : The text to be placed in the text editor.

}

Procedure SetEditorText (Editor : TWinControl ; Text : String) ;
  Begin
  If Editor Is TjmGridEdit Then
    Begin
    (Editor As TjmGridEdit).Text := Text ;
    (Editor As TjmGridEdit).Modified := False ;
    End
  Else If Editor Is TjmGridMemo Then
    Begin
    (Editor AS TjmGridMemo).Lines.Text := Text ;
    (Editor As TjmGridMemo).Modified := False ;
    End
  Else If Editor Is TjmGridMaskEdit Then
    Begin
    (Editor AS TjmGridMaskEdit).Text := Text ;
    (Editor As TjmGridMaskEdit).Modified := False ;
    End
  Else If Editor Is TjmGridComboBox Then
    Begin
    If (Editor As TjmGridComboBox).Style = csDropDownList Then
      Begin
      (Editor As TjmGridComboBox).ItemIndex
        := (Editor As TjmGridComboBox).Items.IndexOf (Text) ;
      End
    Else
      (Editor AS TjmGridComboBox).Text := Text ;
    End
  Else
    Raise Exception.Create ('Internal Error - Unknown Editor Type') ;
  End ;

{
  Procedure:  DrawLine

  Descrption: Draws a line in the grid.

  Parameters:
              ACanvas : The Canvas To Draw On
              X1, Y1: Line Starting Point
              X2, Y2: Line Ending Point
              AColor: Line Color (Does not apply to 3d)
              AStyle: Line Style

}
Procedure DrawLine (ACanvas : TCanvas ;
                    X1, Y1, X2, Y2 : Integer ;
                    AColor : TColor ;
                    AStyle : TjmGridLineStyle) ;
  Procedure DrawSingle ;
    Begin
    ACanvas.Pen.Color := AColor ;
    ACanvas.Pen.Style := psSolid ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;
    ACanvas.MoveTo (X1, Y1) ;
    ACanvas.LineTo (X2, y2) ;
    End ;

  Procedure DrawDouble ;
    Begin
    ACanvas.Pen.Color := AColor ;
    ACanvas.Pen.Style := psSolid ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;
    ACanvas.MoveTo (X1, Y1) ;
    ACanvas.LineTo (X2, y2) ;
    If X1 = X2 Then
      Begin
      ACanvas.MoveTo (X1 + DoubleLineDelta, Y1) ;
      ACanvas.LineTo (X2 + DoubleLineDelta, y2) ;
      ACanvas.Pen.Color := ACanvas.Brush.Color ;
      ACanvas.Pen.Width := DoubleLineDelta - 1;
      ACanvas.MoveTo (X1 + DoubleLineDelta - 1, Y1) ;
      ACanvas.LineTo (X2 + DoubleLineDelta - 1, y2) ;
      End
    Else If Y1 = Y2 Then
      Begin
      ACanvas.MoveTo (X1, Y1 + DoubleLineDelta) ;
      ACanvas.LineTo (X2, y2 + DoubleLineDelta) ;
      ACanvas.Pen.Color := ACanvas.Brush.Color ;
      ACanvas.Pen.Width := DoubleLineDelta - 1 ;
      ACanvas.MoveTo (X1, Y1 + DoubleLineDelta - 1) ;
      ACanvas.LineTo (X2, y2 + DoubleLineDelta - 1) ;
      End
    End ;

  Procedure Draw3D ;
    Begin
    ACanvas.Pen.Color := cl3DDKShadow ;
    ACanvas.Pen.Style := psSolid ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;

    ACanvas.MoveTo (X1, Y1) ;
    ACanvas.LineTo (X2, y2) ;
    If X1 = X2 Then { Vertical Line }
      Begin
      ACanvas.Pen.Color := cl3DLight ;
      ACanvas.MoveTo (X1 + ACanvas.Pen.Width, Y1) ;
      ACanvas.LineTo (X2 + ACanvas.Pen.Width, y2) ;
      End
    Else IF Y1 = y2 Then { Horizontal Line }
      Begin
      ACanvas.Pen.Color := cl3DLight ;
      ACanvas.MoveTo (X1, Y2 + ACanvas.Pen.Width) ;
      ACanvas.LineTo (X2, Y2 + ACanvas.Pen.Width) ;
      End ;
    End ;

  Begin
  Case AStyle of
    lsSingle:
      DrawSingle ;
    lsDouble:
      DrawDouble ;
    ls3D:
      Draw3D ;
    lsNone:
      ;
    End ;
  End ;


Procedure DrawFrame (AStyle : TjmGridFrameStyle ;
                     AColor : TColor ;
                     ARect : TRect ;
                     ACanvas : TCanvas) ;

  Procedure DrawRaised3dBox ;
    Begin
    ACanvas.Pen.Color := cl3DLight ;
    ACanvas.Pen.Style := psSolid ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;

    ACanvas.MoveTo (ARect.Left, ARect.Bottom - 1) ;
    ACanvas.LineTo (ARect.Left, ARect.Top) ;            { Left }
    ACanvas.LineTo (ARect.Right - 1, ARect.Top) ;       { Top }
    ACanvas.Pen.Color := cl3DDKShadow ;
    ACanvas.LineTo (ARect.Right - 1, ARect.Bottom - 1) ;  { Right }
    ACanvas.LineTo (ARect.Left, ARect.Bottom - 1) ;       { Bottom }
    End ;

  Procedure DrawInset3dBox ;
    Begin
    ACanvas.Pen.Color := cl3DDKShadow ;
    ACanvas.Pen.Style := psSolid ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;

    ACanvas.MoveTo (ARect.Left, ARect.Bottom - 1) ;
    ACanvas.LineTo (ARect.Left, ARect.Top) ;            { Left }
    ACanvas.LineTo (ARect.Right - 1, ARect.Top) ;       { Top }
    ACanvas.Pen.Color := cl3DLight ;
    ACanvas.LineTo (ARect.Right - 1, ARect.Bottom - 1) ;  { Right }
    ACanvas.LineTo (ARect.Left, ARect.Bottom - 1) ;       { Bottom }
    End ;

  Procedure DrawFlatFrame (Style : TPenStyle) ;
    Begin
    ACanvas.Pen.Color := AColor ;
    ACanvas.Pen.Style := Style ;
    ACanvas.Pen.Width := 1 ;
    ACanvas.Pen.Mode := pmCopy ;

    ACanvas.MoveTo (ARect.Left, ARect.Bottom - 1) ;
    ACanvas.LineTo (ARect.Left, ARect.Top) ;              { Left }
    ACanvas.LineTo (ARect.Right - 1, ARect.Top) ;         { Top }
    ACanvas.LineTo (ARect.Right - 1, ARect.Bottom - 1) ;  { Right }
    ACanvas.LineTo (ARect.Left, ARect.Bottom - 1) ;       { Bottom }
    End ;

  Begin
  Case AStyle Of
    NoFrame: ; { NoOp }
    RaisedFrame: DrawRaised3dBox ;
    InsetFrame: DrawInset3dBox ;
    SolidFrame: DrawFlatFrame (psSolid) ;
    DashFrame: DrawFlatFrame (psDash) ;
    DotFrame: DrawFlatFrame (psDot) ;
    Else ;
    End ;
  End ;

{
  Function:  LineWidth

  Description:  Centralized routine for determining the width in pixels
                for a given line style.


  Parameters:
              AStyle : The line style

  Return Value:
              The width of a line in the given line style.
}
Function LineWidth (AStyle : TjmGridLineStyle) : Cardinal ;
  Begin
  Case AStyle Of
    ls3D:
      Result := 2 ;
    lsSingle:
      Result := 1 ;
    lsDouble:
      Result := 1 + DoubleLineDelta ;
    lsNone:
      Result := 0 ;
    Else
      Raise Exception.Create ('Unknown line style') ;
    End ;
  End ;

{
  This function determines if two rectangles overlap.
}
Function IsIntersectionNull (const Rect1 : TRect ; const Rect2 : TRect) : Boolean ;
  Begin
  If (Rect1.Left > Rect2.Right)
     Or (Rect1.Top > Rect2.Bottom)
     Or (Rect1.Right < Rect2.Left)
     Or (Rect1.Bottom < Rect2.Top) Then
    Begin
    Result := True
    End
  Else
    Begin
    Result := False
    End ;
  End ;


Function ZeroRect : TRect ;
  Begin
  Result.Left := 0 ;
  Result.Top := 0 ;
  Result.Right := 0 ;
  Result.Bottom := 0 ;
  End ;

{
  This function determines if a scroll bar control is visible.
}

Function VisibleScrollBar (AScrollBar : TScrollBar) : Boolean ;
  Begin
  If (AScrollBar = Nil) Or Not AScrollBar.Visible Then
    Result := False
  Else
    Result := True ;
  End ;

Constructor TColossalGrid.Create (AOwner : TComponent) ;
  Begin
  Inherited Create (AOwner) ;
  FDataLink := TjmGridDataLink.Create (Self) ;
  FMode := jmDefaultGridMode ;
  FLayoutMode := jmDefaultLayoutMode ;
  FSelectMode := jmDefaultSelectMode ;
  FOnUnboundFetch := Nil ;
  FDataZone := 1 ;

  ControlStyle := ControlStyle + [csOpaque, csFramed] ;

  FBorderStyle := jmDefaultBorderStyle ;

  FZones := TjmGridZoneList.Create ;

  FVariableWidthZone := 1 ;
  FDefaultRowCount := 1 ;
  FActiveZone := 1 ;
  FActive := True ;
  FReadOnly := False ;

  Width := 150 ;
  Height := 150 ;

  FHorizontalLineColor := jmDefaultHorizontalLineColor ;
  FHorizontalLineStyle := jmDefaultHorizontalLineStyle ;
  FVerticalLineStyle := jmDefaultVerticalLineStyle ;
  FVerticalLineColor := jmDefaultVerticalLineColor ;
  FBackgroundColor := jmDefaultBackgroundColor ;
  FHeaderColor := jmDefaultHeaderColor ;
  FFooterColor := jmDefaultFooterColor ;
  FInactiveColor := jmDefaultInactiveColor ;

  FDisableColumnResize := False ;
  FDisableColumnMove := False ;
  FDisableRecordResize := True ;
  FDisableRecordMove := True ;

  Font.Name := 'MS SANS SERIF' ;

  FSelectFont := TFont.Create ;
  FSelectFont.OnChange := FontChanged ;
  FSelectFont.Assign (Font) ;
  FSelectFont.Color := clHighLightText ;

  FHeaderFont := TFont.Create ;
  FHeaderFont.OnChange := FontChanged ;
  FHeaderFont.Assign (Font) ;
  FHeaderFont.Color := clCaptionText ;

  FFooterFont := TFont.Create ;
  FFooterFont.OnChange := FontChanged ;
  FFooterFont.Assign (Font) ;
  FFooterFOnt.Color := clWindowText ;

  FInactiveFont := TFont.Create ;
  FInactiveFont.OnChange := FontChanged ;
  FInactiveFont.Assign (Font) ;
  FInactiveFont.Color := clInactivecaptionText ; ;

  FShowVerticalScrollBar := sbWhenNeeded ;
  FShowHorizontalScrollBar := sbWhenNeeded ;

  FUnusedColor := jmDefaultUnusedColor ;

  FHScrollBar := TScrollBar.Create (Nil) ;
  FHScrollBar.TabStop := False ;
  FHScrollBar.Visible := False ;
  FHScrollBar.Kind := sbHorizontal ;

  FVScrollBar := TScrollBar.Create (Nil) ;
  FVScrollBar.TabStop := False ;
  FVScrollBar.Visible := False ;
  FVScrollBar.Kind := sbVertical ;

  FUseTabs := True ;
  FUseArrows := True ;

  FRowHeight := jmDefaultRowHeight ;

  { Lines to separate scroll bars from zones }
  FHorizontalScrollBarSeparatorStyle := jmDefaultHorizontalScrollBarSeparatorStyle ;
  FHorizontalScrollBarSeparatorColor := jmDefaultHorizontalScrollBarSeparatorColor ;
  FVerticalScrollBarSeparatorStyle := jmDefaultVerticalScrollBarSeparatorStyle ;
  FVerticalScrollBarSeparatorColor := jmDefaultVerticalScrollBarSeparatorColor ;

  { Lines between Zones }
  FZoneSeparatorStyle := jmDefaultZoneSeparatorStyle ;
  FZoneSeparatorColor := jmDefaultZoneSeparatorColor ;

  { Lines between header/footer and records }
  FHeaderSeparatorStyle := jmDefaultHeaderSeparatorStyle ;
  FHeaderSeparatorColor := jmDefaultHeaderSeparatorColor ;
  FFooterSeparatorStyle := jmDefaultFooterSeparatorStyle ;
  FFooterSeparatorColor := jmDefaultFooterSeparatorColor ;

  { Highlight Colors }
  FSelectColor := jmDefaultSelectColor ;

  FRowLineStyle := jmDefaultRowLineStyle ;
  FRowLineColor := jmDefaultRowLineColor ;

  FHasHeader := True ;
  FHasFooter := True ;

  FEditorVisible := False ;
  FActiveEditor := Nil ;

  FTopVisibleRecord := 1 ;
  FRecordCount := 0 ;
  Ctl3D := true ;
  ParentCtl3d := true ;

  TabStop := True ;

  FCurrentCell.Zone := 0 ;
  FCurrentCell.Column := 0 ;
  FCurrentCell.RecordNumber := 0 ;

  FOnAppend := Nil ;
  FOnDrawCell := Nil ;

  FCellFrameColor := clBlack ;
  FCellFrameStyle := NoFrame ;
  FHeaderFrameColor := clBlack ;
  FHeaderFrameStyle := NoFrame ;
  FFooterFrameColor := clBlack ;
  FFooterFrameStyle := NoFrame ;
  FActiveFrameColor := clYellow ;
  FActiveFrameStyle := DotFrame ;


  If (GetOwnerForm = Nil) OR Not (csLoading in GetOwnerForm.ComponentState) Then
    AddZone ;

  End ;

Destructor TColossalGrid.Destroy ;
  Begin
  FDefaultEditor.Free ;
  FDefaultEditor := Nil ;
  FSelectFont.Free ;
  FSelectFont := Nil ;
  FHeaderFont.Free ;
  FHeaderFont := Nil ;
  FFooterFont.Free ;
  FFooterFont := Nil ;
  FInactiveFont.Free ;
  FInactiveFOnt := Nil ;
  FHScrollBar.Free ;
  FHScrollBar := Nil ;
  FVScrollBar.Free ;
  FVScrollBar := Nil ;
  FDataLink.Free ;
  FDataLink := Nil ;

  { We cannot delete the zones until all of the editors are deleted. }
  DeleteZones ;
  FZones.Free ;
  FZones := Nil ;

  Inherited Destroy ;
  End ;

Constructor TjmGridZone.Create (AGrid : TComponent) ;
  Begin
  Inherited Create (Nil) ;
  FNeedsHScrollBar := False ;

  { We need csOpaque in the control style or we get flicker. }
  ControlStyle := ControlStyle + [csOpaque] ;

  FGrid := AGrid As TColossalGrid ;
  Parent := FGrid ;

  FOverride := [] ;
  FState := gsNormal ;

  FRows := TjmGridRowList.Create ;
  FColumns := TjmGridColumnList.Create ;

  FDisableColumnResize := False ;
  FDisableColumnMove := False ;

  FSelectable := True ;
  FExtendRightColumn := True ;
  FZoneWidth := 150 ;

  FShowHorizontalScrollBar := sbWhenNeeded ;
  FFont := Nil ;
  FSelectFont := Nil ;
  FInactiveFont := Nil ;
  FHeaderFont := Nil ;
  FFooterFont := Nil ;

  If Not (csLoading in ParentGrid.ComponentState) Then
    Begin
    AdjustRowCount (ParentGrid.DefaultRowCount) ;
    End ;

  { The last step is to tell the grid that the zone has been created. }
  FGrid.AttachZone (Self) ;
  End ;

Destructor TjmGridZone.Destroy ;
  Begin
  IF FRows <> Nil Then
    Begin
    While FRows.Count > 0 Do
      FRows.Delete (1) ;
    FRows.Free ;  FRows := Nil ;
    End ;

  If FColumns <> Nil Then
    Begin
    While FColumns.Count > 0 Do
      Begin
      FColumns.Delete (1) ;
      End ;
    FColumns.Free ; FColumns := Nil ;
    End ;

  FFont.Free ;  FFont := Nil ;
  FInactiveFont.Free ; FInactiveFont := Nil ;
  FSelectFont.Free ;  FSelectFont := Nil ;
  FHeaderFont.Free ;  FHeaderFont := Nil ;
  FFooterFont.Free ;  FFooterFont := Nil ;

  Inherited Destroy ;
  End ;

Constructor TjmGridRow.Create (AZone : TComponent) ;
  Begin
  Inherited Create (Nil) ;
  FZone := AZone As TjmGridZone ;
  FOverride := [] ;
  FMaxColumns := 0 ;
  FZone.AddRow (Self) ;
  FFont := Nil ;
  End ;

Destructor TjmGridRow.Destroy ;
  Begin
  Inherited Destroy ;
  FFont.Free ;
  FFont := Nil ;
  FZone := Nil ;
  FMaxColumns := 0 ;
  FOverride := [] ;
  End ;

Constructor TjmGridColumn.Create (AZone : TComponent) ;
  Begin
  Inherited Create (Nil) ;

  FZone := AZone As TjmGridZone ;
  FColumnNumber := High (Integer) ;
  FGutterSize := 2 ;
  FOverride := [] ;

  FColumnType := jmDefaultColumnType ;
  FieldName := '' ;
  HeaderText := '' ;
  FooterText := '' ;
  FField := Nil ;

  FEditorType := etDefault ;
  FEditor := Nil ;

  FCellFrameColor := FZone.CellFrameColor ;
  FCellFrameStyle := FZone.CellFrameStyle ;
  FHeaderFrameColor := FZone.HeaderFrameColor ;
  FHeaderFrameStyle := FZone.HeaderFrameStyle ;
  FFooterFrameColor := FZone.FooterFrameColor ;
  FFooterFrameStyle := FZone.FooterFrameStyle ;
  FActiveFrameColor := FZone.ActiveFrameColor ;
  FActiveFrameStyle := FZone.ActiveFrameStyle ;

  FWidth := jmDefaultColumnWidth ;
  FHidden := False ;
  FReadOnly := False ;

  FFont := Nil ;
  FSelectFont := Nil ;
  FHeaderFont := Nil ;
  FFooterFont := Nil ;
  FInactiveFont := Nil ;

  FAlignment := taLeftJustify ;
  FVerticalAlignment := vaTop ;
  FHeaderAlignment := taLeftJustify ;
  FHeaderVerticalAlignment := vaTop ;
  FFooterAlignment := taLeftJustify ;
  FFooterVerticalAlignment := vaTop ;

  FBackgroundColor := FZone.BackgroundColor ;
  FFooterColor := FZone.FooterColor ;
  FHeaderColor := FZone.HeaderColor ;
  FInactiveColor := FZone.InactiveColor ;
  FSelectColor  := FZone.SelectColor ;

  FZone.AddColumn (Self) ;
  End ;

Destructor TjmGridColumn.Destroy ;
  Begin
  FFont.Free ;
  FSelectFont.Free ;
  FHeaderFont.Free ;
  FFooterFont.Free ;
  FInactiveFont.Free ;
  FEditor.Free ;

  Inherited Destroy ;
  End ;

Constructor TjmGridDataLink.Create (AGrid : TColossalGrid) ;
  Begin
  Inherited Create ;
  FGrid := AGrid ;
  FScrollOffset := 0 ;
  FVisibleRecords := 0 ;
{$IFDEF TRACE}
  FTrace := True ;
{$ENDIF}
  End ;


{$INCLUDE GRIDPROPERTIES}
{$INCLUDE GRIDEVENTS}
{$INCLUDE GRIDOVERRIDES }
{$INCLUDE GRIDMEASUREMENT}
{$INCLUDE GRIDDRAWING}
{$INCLUDE GRIDSCROLLBARS}
{$INCLUDE GRIDINFO}
{$INCLUDE GRIDACTIONS}
{$INCLUDE GRIDEDITING}
{$INCLUDE GRIDEDITOR}
{$INCLUDE GRIDLISTS}
{$INCLUDE GRIDDATAPROCS}

{$INCLUDE ZONEOVERRIDES}
{$INCLUDE ZONEEVENTS}
{$INCLUDE ZONEPROPERTIES}
{$INCLUDE ZONEMEASUREMENT}
{$INCLUDE ZONEDRAWING}
{$INCLUDE ZONESCROLLING}
{$INCLUDE ZONECONVERSION}
{$INCLUDE ZONEACTIONS}

{$INCLUDE ROWPROCS}

{$INCLUDE COLUMNPROCS}

{$INCLUDE DATALINKPROCS}

Initialization
  RegisterClasses ([TColossalGrid, TjmGridColumn, TjmGridZone, TjmGridRow, TDataSource]) ;

end.
