{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the implementation of the TjmgridDataLink class. This
  class is the TDataLink-derived link with the database.
}

Function DataState (Link : TjmGridDataLink) : String ;
  Begin
  Result := '' ;
  IF Link.Active Then Result := Result + ' Active' ;
  IF Link.DataSourceFixed Then Result := Result + ' DSFixed' ;
  IF Link.Editing Then Result := Result + ' ED' ;
  IF Link.ReadOnly Then Result := Result + ' RO' ;
  Result := Result + ' AR = ' + IntToStr (Link.ActiveRecord) ;
  Result := Result + ' BC = ' + IntToStr (Link.BufferCount) ;
  Result := Result + ' RC = ' + IntToStr (Link.RecordCount) ;
  Result := Result + ' DRC = ' + IntToStr (Link.DataSet.RecordCount) ;
  If Link.DataSet.EOF Then
    Result := Result + ' EOF ' ;
  End ;

{
  This overridden prodedure gets called when the data link becomes
  active or inactive.
}
procedure TjmGridDataLink.ActiveChanged ;
  Begin
  If (FTrace) Then WriteLn ('ActiveChanged ' + DataState (Self)) ;
  { All we have to do here is to notify the grid of the change. }
  If Active Then
    FGrid.DataLinkActive
  Else
    FGrid.DataLinkInactive ;
  End ;

{
  I have not determined what this overriden procedure does.  I suspect that
  its purpose is to give the application the opportunity to abort a database
  operation by calling "abort".

  We use it to assist in determining when a record gets deleted.  One of the
  times this procedure does get called it right before a delete occurs.  We
  save the count of records so that it can be compared to the recordcount.

}
procedure TjmGridDataLink.CheckBrowseMode ;
  Begin
  If (FTrace) Then WriteLn ('CheckBrowseMode ' + DataState (Self)) ;
  End ;

{
  This overridden procedure gets called for all sorts of events.
  So far it appears that these are:

    Moving to the start of the dataset
    Moving to the end of the dataset
    Inserting or Appending to the dataset
    Deleting a record from the dataset
    Canceling the editing of a record
    Updating a record

  The code here attempts to decipher what occured from the data of the
  dataset and the additional clues that we add to the datalink.

  If we had not overridden this procedure TDataLink would simply call:

    RecordChanged (Nil)

}
Procedure TjmGridDataLink.DataSetChanged ;
  Var
    Cell : TJmGridCell ;
  Procedure MoveFirst ;
    Begin
    If (RecordCount <> 0) Then
      Begin
      FGrid.DataChangeRecord (1) ;
      End
    Else
      Begin
      FGrid.ChangeRecordCount (0) ;
      FGrid.RedrawRecordArea ;
      End ;
    End ;

  Procedure MoveLast ;
    Begin
    FGrid.DataChangeRecord (RecordCount) ;
    End ;

  Begin
  If (FTrace) Then WriteLn ('DataSetChanged ' + DataState (Self)) ;

  If DataSet.BOF Then
    Begin
    MoveFirst ;
    End
  Else If DataSet.EOF Then
    Begin
    If Not Editing Then
      Begin
      MoveLast ;  { Move to the end of the dataset }
      FGrid.RedrawRecordArea ;
      End
    Else
      Begin
      FGrid.RedrawRecordArea ;  { Appending a record to the end of the dataset. }
      End ;
    End
  Else
    Begin
    FGrid.LayoutChanged ;
    FGrid.RedrawRecordArea ;
    End ;

  If Editing Then
    Begin
    Cell := FGrid.CurrentCell ;
    If Cell.Zone = 0 Then
      Cell.Zone := 1 ;
    If Cell.Column = 0 Then
      Cell.Column := 1 ;
    Cell.RecordNumber := ActiveRecord + 1 ;
    FGrid.CurrentCell := Cell ;
    If FGrid.HasControl Then
      FGrid.EditorVisible := True ;
    End ;

  End ;

{
  This overridden procedure gets called any time the current record in the
  dataset changes through a Next or Prior operation.

  The Distance parameter indicates how much the buffer window in the dataset
  has moved.  The ActiveRecord member indicates the offset within the buffer
  window.

  The default action from TDataLink would be to call DataSetChanged.
}
procedure TjmGridDataLink.DataSetScrolled(Distance: Integer) ;
  Begin
  If (Distance < 0) Or (Distance > 0) Then
    FGrid.RedrawRecordArea ; // Maybe too drastic

  If (FTrace) Then WriteLn ('DataSetScrolled ', Distance, ' ', ActiveRecord, ' ' + DataState (Self)) ;

  FGrid.DataChangeRecord (ActiveRecord + 1) ;

  End ;

{
  No idea what this overriden procedure does.  It does not even
  seem to get called.
}
procedure TjmGridDataLink.FocusControl(Field: TFieldRef) ;
  Begin
  If (FTrace) Then WriteLn ('FocusCOntrol ' + DataState (Self)) ;
  End ;

{
  This overriden procedure gets called when the data link enters or exits
  the editing state.

}
procedure TjmGridDataLink.EditingChanged ;
  Begin
  If (FTrace) Then WriteLn ('Editing Changed ' + DataState (Self)) ;
  End ;

{
  This overriden procedure gets called when the dataset's column layout
  changes.
}
procedure TjmGridDataLink.LayoutChanged ;
  Begin
  FGrid.DataLayoutChanged ;
  End ;

{
  This overridden procedure appears to get called when:

    o The current record is edited
    o The record's text has changed
        If the Field parameter is non-nil then this seems
        to mean that one field has changed.
}
procedure TjmGridDataLink.RecordChanged(Field : TField) ;
  Begin
  If (FTrace) Then WriteLn ('RecordChanged ' + DataState (Self)) ;
  FGrid.RedrawRecord (ActiveRecord + 1) ;
  End ;

{
  This overriden procedure gets called before a record is written to the
  database.
}
procedure TjmGridDataLink.UpdateData ;
  Begin
  If (FTrace) Then WriteLn ('UpdateData ' + DataState (Self)) ;

  FGrid.UpdateData ;
  End ;

{
  This function returns the number of records to be displayed.
}
Function TjmGridDataLink.GetRecordSetCount : LongInt ;
  { The TDataSet.RecordCount is too slow. }
  Begin
  Result := 0 ;
  If DataSet <> Nil Then
    Begin
    { The RecordCount property for TBDEDataSet objects is implemented
      using the DbiGetExactRecordCount. This function is too
      slooooooooooow for our purposes.
    }
    If Dataset Is TBDEDataSet Then
      DbiGetRecordCount((Dataset As TBDEDataSet).Handle, Result)
    Else
      Result := Dataset.RecordCount ;
    End ;

  If Editing Then
    Result := Result + 1 ;
  End ;

Function TjmGridDataLink.GetCurrentRecordNumber : LongInt ;
  Begin
  If DataSet.BOF Then
    Begin
    Result := 1 ;
    End
  Else If DataSet.EOF Then
    Begin
    Result := GetRecordSetCount ;
    End
  Else
    Begin
    If Dataset Is TBDEDataSet Then
      Begin
      Result := (Dataset AS TBDEDataSet).RecNo ;
      End
    Else
      Begin
      Result := FScrollOffset + ActiveRecord ;
      End ;
    End ;
  End ;

