{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains TColossalGrid methods that are catagorized as "actions".
}

{
  Procedure: TColossalGrid.AddZone

  Description:

  This procedure adds a zone to the grid.  CreateZone is a virtual function
  that creates a zone.  The zone's constructor attaches the zone to the grid.

  Having this as a separate procedure means that the places that need to add a
  zone do not have to declare a zone variable.

}

Procedure TColossalGrid.AddZone ;
  Begin
  CreateZone ;
  End ;


{
  Procedure: TColossalGrid.AttachZone

  Description:

  This procedure is called by a zone's constructor to link the zone to the
  grid.  While the obvious method for doing this would be to simply have the
  grid create link when it creates the zone.  Unfortunately when the grid is
  read from a stream the zone is created by VCL so the grid would not know
  the the zone has been created.


  Parameters:
          Zone : The grid zone to attach to the grid.

}
Procedure TColossalGrid.AttachZone (Zone : TjmGridZone) ;
  Begin
  FZones.Add (Zone) ;
  Zone.FZoneNumber := FZones.Count ;
  If Not (csLoading in ComponentState) Then
    PositionZones ;
  End ;

{
  Function: TColossalGrid.CreateZone

  Description:

  We use a virtual function to create zones so descendant classes can use
  their own zone classes.  This implementation is only used for testing.  The
  actual grid classes need to override this method.

  Parameters: None
}
Function TColossalGrid.CreateZone : TjmGridZone ;
  Begin
  Result := TjmGridZone.Create (Self) ;
  End ;

{
  Procedure: TColossalGrid.RepaintHeader

  Description:

  Procedure to invalidate the header area of the grid.  This needs to be
  done whenever the grid gains or loses input focus.

  Parameters: None
}
Procedure TColossalGrid.RedrawHeader ;
  Var
    Rect : TRect ;
  Begin
  Rect := GetGridRect ;
  Rect.Bottom := Rect.Top + EffectiveHeaderHeight ;
  RedrawRect (Rect) ;
  End ;

{
  Function: TColossalGrid.CreateColumn

  Description:

  We use a virtual function to create columns so descendant classes can use
  their own column classes.  This implementation is only used for testing.

  Parameters:
    AZone : The zone the column is added to.

  Return Value:
    The address of the column that is created.
}

Function TColossalGrid.CreateColumn (AZone : TjmGridZone) : TjmGridColumn ;
  Begin
  Result := TjmGridColumn.Create (AZone) ;
  End ;

{
  Procedure:  TColossalGrid.RepaintCell

  Description:

  This procedure invalidates a cell which causes Windoze to repaint the cell.

  Parameters: None
}

Procedure TColossalGrid.RedrawCell (ACell : TjmGridCell) ;
  Begin
  If (ACell.RecordNumber < 1) Or (ACell.RecordNumber > RecordCount) Then
    Exit ;
  Zones [ACell.Zone].RedrawCell (ACell.Column, ACell.RecordNumber) ;
  End ;

{
  Procedure:  TColossalGrid.ChangeCurrentCell

  Description:

  This procedure is used to change the currently active cell.

  Parameters:
    NewCell : The cell that is to become the current cell.
}

Procedure TColossalGrid.ChangeCurrentCell (NewCell : TjmGridCell) ;
  Var
    OldCell : TjmGridCell ;
    Cancel : Boolean ;
  Begin
  If Assigned (OnLeaveCell) Then
    Begin
    Cancel := False ;
    OnLeaveCell (Self, NewCell.Zone, NewCell.Column, NewCell.RecordNumber, Cancel) ;
    If Cancel Then
      Exit ;
    End ;

  OldCell := FCurrentCell ;
  FCurrentCell := NewCell ;

  { If the editor is visible we need to hide it before changing the cell. }
  If EditorVisible Then
    EditorVisible := False ;

  { Repaint the current cell. }
  If (OldCell.RecordNumber <> 0) And (OldCell.Zone <> 0) And (OldCell.Column <> 0) Then
    Begin
    If (FSelectMode = smCellAndRecord) OR (FSelectMode = smRecord) Then
      RedrawRecord (OldCell.RecordNumber)
    Else
      RedrawCell (OldCell) ;
    End ;

  { This is where we make the actual switch to the new cell. }
  FCurrentCell := NewCell ;

  { Repaint the new current cell. }
  If (FCurrentCell.RecordNumber <> 0) And (FCurrentCell.Zone <> 0) And (FCurrentCell.Column <> 0)Then
    Begin
    If (FSelectMode = smCellAndRecord) OR (FSelectMode = smRecord) Then
      RedrawRecord (FCurrentCell.RecordNumber)
    Else
      RedrawCell (FCurrentCell) ;
    End ;

  { Adjust the scrollbar. }
  If VisibleScrollBar (FVScrollBar) Then
    Begin
    If FMode = gmDatabase Then
      FVScrollBar.Position := FDataLink.GetCurrentRecordNumber 
    Else
      FVScrollBar.Position := NewCell.RecordNumber ;
    End ;

  If Assigned (OnEnterCell) Then
    OnEnterCell (Self) ;

  End ;

{
  Procedure:  TColossalGrid.LayoutChanged

  Description:

    This procedure is called whenever a property that affects the grid's layout
    is changed.

  Parameters: None
}

Procedure TColossalGrid.LayoutChanged ;
  Begin
  If (FActive And HandleAllocated) And Not (csReading in ComponentState) Then
    Begin
    If UsingDatabase Then
      UpdateDataRecordCount ;
    PositionColumnsInRows ;
    PositionZones ;
    If EditorVisible Then
      PositionEditor ;
    Repaint ;
    End ;
  End ;

{
  Procedure:  TColossalGrid.CellsMoved

  Description:

    This procedure is called each time the grid is scrolled.

    The main function of this procedure to to eliminate unwanted double clicks.
    If the user clicks on a partial record it will scroll up to become
    completely visible.  If the user clicks on the same point rapidly then we
    do not want to do double click processing.  By comaring the FScrollCount
    value we can tell if the user is clicking on the same cell.

  Parameters: None

}
Procedure TColossalGrid.CellsMoved ;
  Begin
  Inc (FScrollCount) ;
  End ;

{
  Procedure:  TColossalGrid.RepaintRecord

  Description:

    This procedure invalidates a complete record in the grid.

  Parameters:
    ARecord : The number of the record to invalidate.

}

Procedure TColossalGrid.RedrawRecord (ARecord : LongInt) ;
  Var
    RecordAreaRect : TRect ;
    RecordRect : TRect ;
    Offset : Integer ;
  Begin
  If (ARecord < 1) Or (ARecord > RecordCount) Then
    Raise Exception.CreateFmt ('Record number (%d) out of range 1..%d', [ARecord, RecordCount]) ;

  If (ARecord < TopVisibleRecord) Or (ARecord > BottomVisibleRecord) Then
    Exit ;

  RecordAreaRect := GetGridRect ;
  RecordAreaRect.Top := RecordAreaYPosition ;
  RecordAreaRect.Bottom := RecordAreaRect.Top + RecordAreaHeight ;
  If VisibleScrollBar (FVScrollBar) Then
    Dec (RecordAreaRect.Right, FVScrollBar.Width) ;

  Offset := (ARecord - TopVisibleRecord) * (RecordHeight + LineWidth (HorizontalLineStyle)) ;
  RecordRect := RecordAreaRect ;
  Inc (RecordRect.Top, Offset) ;
  RecordRect.Bottom := RecordRect.Top + (RecordHeight + LineWidth (HorizontalLineStyle)) ;

  { If the record is clipped then eliminate the clipped part. }
  If RecordRect.Top < RecordAreaRect.Top Then
    RecordRect.Top := RecordAreaRect.Top ;
  If RecordRect.Bottom > RecordAreaRect.Bottom Then
    RecordRect.Bottom := RecordAreaRect.Bottom ;

  RedrawRect (RecordRect) ;
  
  End ;

{
  Procedure:  TColossalGrid.FontChanged

  Description:
    This procedure is used as the OnChange handler for all of the grid's fonts.
    This procedure lets the grid know when a font has changed so it can repaint
    itself.

  Parameters:
    Sender : The font that was changed.

}

Procedure TColossalGrid.FontChanged (Sender : TObject) ;
  Begin
  { In the futrue we could do more checking so that we only repaint the areas
    affected by the font change. }
  LayoutChanged ;
  End ;


{
  Procedure: TColossalGrid.MoveZone

  This is a public procedure that allows the user to change the zone order.
  It moves a zone to a new position in the zone list.

  Parameters:
          OldPosition:  The index of the zone to move (1 Based)
          NewPosition:  The new index for the zone (1 Based)

  If NewPosition is greater than the number of zones then we treat is as if
  it is zone count.  This allows the user to do something like
  MoveZone (X, 9999999) ; to move it to the end of the list.

}

Procedure TColossalGrid.MoveZone (OldPosition : Cardinal ; NewPosition : Cardinal) ;
  Var
    IZone : Integer ;
  Begin
  If (OldPosition < 1) Or (OldPosition > ZoneCount) Then
    Raise Exception.CreateFmt ('Old Zone Position (%d) outside range 1..%d', [OldPosition, ZoneCount]) ;

  If (NewPosition < 1) Then
    Raise Exception.CreateFmt ('New Zone Position Zone (%d) outside range 1..%d', [NewPosition, ZoneCount]) ;

  If (OldPosition = NewPosition) Or ((OldPosition = ZoneCount) And (NewPosition > ZoneCount)) Then
    Exit ;

  If NewPosition > ZoneCount Then
    NewPosition := ZoneCount ;

  FZones.Move (OldPosition, NewPosition) ;

  { Let the zones know what position they are in. }
  For IZone := 1 To ZoneCount Do
    Begin
    { If this is the zone containing the current cell then move the current
      cell to the zone's new position. }
    If FCurrentCell.Zone = Zones [IZone].FZoneNumber THen
      FCurrentCell.Zone := IZone ;
    Zones [IZone].FZoneNumber := IZone ;
    End ;
  PositionZones ;
  Repaint ;
  End ;

{
  Procedure:  TColossalGrid.Resize

  Description:

    This procedure is called whenever the grid is resized.  Generally this is
    the WM_SIZE handler.  The code is separated here because this is a virtual
    procedure.  Descendants can override this to add their own resize handling.

  Parameters: None
}
Procedure TColossalGrid.ResizeGrid ;
  Begin
  If ZoneCount > 0 Then
    PositionZones ;


  FindRecordHeight ;
  
  PositionScrollBars ;

  If UsingDatabase Then
    Begin
    UpdateDataRecordCount ;
    End
  Else If (RecordCount <> 0)
          And (TopVisibleRecord + MaxVisibleRecords > RecordCount)
          And (MaxVisibleRecords < RecordCount) Then
    Begin
    TopVisibleRecord := RecordCount - MaxVisibleRecords + 1 ;
    End ;

  If EditorVisible Then
    PositionEditor ;

  End ;

{
  Procedure:  TColossalGrid.ChangeCurrentRecord

  Description:

    This virtual procedure is used to change the current selected record
    in the grid.

  Parameters:
    NewRecord : The record to move to.

}

Procedure TColossalGrid.ChangeCurrentRecord (NewRecord : LongInt) ;
  Var
    NewCell : TjmGridCell ;
  Begin

  If UsingDatabase Then
    Begin
    { We have to let the database events update the cell }
    If FDataLink.Editing Then
      FDataLink.DataSet.Post ;
    If (NewRecord <> FDataLink.ActiveRecord - 1) Then
      Begin
      FDataLink.DataSet.MoveBy (NewRecord - FDataLink.ActiveRecord - 1) ;
      End
    Else
      Begin
      NewCell := CurrentCell ;
      NewCell.RecordNumber := newRecord ;
      ChangeCurrentCell (NewCell) ;
      End ;
    End
  Else
    Begin
    If NewRecord < 1 Then
      NewRecord := 1 ;

    If NewRecord > RecordCount Then
      NewRecord := RecordCount ;

    If NewRecord = CurrentCell.RecordNumber Then
      Exit ;
    NewCell := CurrentCell ;
    NewCell.RecordNumber := NewRecord ;
    If (NewRecord <> 0) Then
      Begin
      If (NewRecord < TopVisibleRecord) Then
        Begin
        TopVisibleRecord := NewRecord ;
        End
      Else If (NewRecord > BottomVisibleRecord) Then
        Begin
        TopVisibleRecord := TopVisibleRecord + (NewRecord - BottomVisibleRecord) ;
        End ;
      End ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

{
  Procedure:  TColossalGrid.RepaintRecordArea

  Description:

    This procedure invalidates the area of the grid containing records.

  Parameters:

    None
}

Procedure TColossalGrid.RedrawRecordArea ;
  Var
    IZone : Integer ;
  Begin
  For IZone := 1 To ZoneCount Do
    Zones [IZone].RedrawRecordArea ;
  End ;

{
  Procedure:   TColossalGrid.DeleteZones

  Description:

    This procedure deletes all of the zones in the grid.  It is intended to
    be called from the grid's destructor to free all the memory allocated to
    zones and their columns and rows.

  Parameters:

    None
}
Procedure TColossalGrid.DeleteZones ;
  Begin
  If FZones <> Nil Then
    Begin
    While FZones.Count <> 0 Do
      FZones.Delete (1) ;
    End ;
  End ;

{
  Procedure:  TColossalGrid.AdjustRowCount

  Description:

    This procedure is called whenever the grid's row count changes.  It use
    used to tell the zones to update their row count.

  Parameters:

    None
}

Procedure TColossalGrid.AdjustRowCount ;
  Var
    IZone : Integer ;
  Begin
  For IZone := 1 To ZoneCount Do
    Begin
    Zones [IZone].AdjustRowCount (DefaultRowCount) ;
    End ;
  End ;

{
  Procedure:  TColossalGrid.PositionColumnsInRows

  Description:

    This procedure is called after the grid is loaded from the stream.  It
    tells all the zones to assign their columns to rows.

  Parameters:

    None
}

Procedure TColossalGrid.PositionColumnsInRows ;
  Var
    IZone : Integer ;
  Begin
  For IZone := 1 To ZoneCount Do
    Begin
    Zones [IZone].PositionColumnsInRows ;
    End ;
  End ;

{
  Procedure:   TColossalGrid.VisibleRecordsChanged

  Description
    This virtual function is called whenever a property changes that affects
    the number of visible records.

  Parameters:

    None
}

{
  Procedure: TColossalGrid.FindRecordHeight

  Description:
    This is one of proverbial "efficient" procedures.  It is used to find and
    save the record height for the grid and call each each time the grid layout
    changes.  If we did this as part of the RecordHeight property we would have
    to do this calculation each time a zone paints a record.

  Parameters:
    None

}
Procedure TColossalGrid.FindRecordHeight ;
  Var
    IZone : Cardinal ;
    Height : Cardinal ;
  Begin
  FRecordHeight := LineWidth (FHorizontalLineStyle) ;
  For IZone := 1 To ZoneCount Do
    Begin
    Height := Zones [IZone].DesiredRecordHeight ;
    If Height > FRecordHeight Then
      FRecordHeight := Height ;
    End ;
  End ;

{
    This function removes the editor without updating.
}
Procedure TColossalGrid.CancelEditing ;
  Begin
  HideEditor (True) ;
  If UsingDatabase Then
    FDataLink.RecordChanged (Nil) ;
  End ;


Procedure TColossalGrid.ChangeRecordCount (ACount : LongInt) ;
  Var
    OldCount : LongInt ;
    OldTop : LongInt ;
    OldBottom : LongInt ;
    NewTop : LongInt ;
    Rect : TRect ;
  Begin
  If ACount < 0 Then
    Raise Exception.Create ('Invalid record count') ;

  If ACount = FRecordCount Then
    Exit ;

  IF Not WindowValid Then
    Begin
    FRecordCount := ACount ;
    Exit ;
    End ;

  { Save the previous state of the grid. We use these values to determine
    what needs to be redrawn. }
  OldCount := FRecordCount ;
  OldTop := TopVisibleRecord ;
  OldBottom := BottomVisibleRecord ;

  FRecordCount := ACount ;

  If FRecordCount = 0 Then
    FTopVisibleRecord := 0
  Else If (TopVisibleRecord = 0) Then
    FTopVisibleRecord := 1 ;
  { See if the current cell has become invalid. }
  If RecordCount < FCurrentCell.RecordNumber Then
    Begin
    FCurrentCell.RecordNumber := RecordCount ;
    End
  Else If (RecordCount > 0) And (FCurrentCell.RecordNumber = 0) Then
    Begin
    FCurrentCell.RecordNumber := 1 ;
    End ;

  If HandleAllocated And (VerticalScrollbarsAreNeeded <> VisibleScrollBar (FVScrollBar)) Then
    Begin
    LayoutChanged ;
    Exit ;
    End ;

  If OldCount = 0 Then
    Begin

    Rect := GetGridRect ;
    Rect.Top :=  RecordAreaYPosition ;
    Rect.Bottom := Rect.Top + RecordCount * RecordHeight ;
    If Rect.Bottom > (RecordAreaYPosition + RecordAreaHeight) Then
      Rect.Bottom := RecordAreaYPosition + RecordAreaHeight ;
    FCurrentCell.RecordNumber := 1 ;
    FCurrentCell.Zone := ActiveZone ;
    If Not (csLoading in ComponentState) And (Zones [ActiveZone].FColumns.Count > 0) Then
      FCurrentCell.Column := 1 ;
    RedrawRect (Rect) ;
    End
  Else If RecordCount > OldCount Then
    Begin { Adding Records }
    If ((OldBottom - OldTop + 1) * RecordHeight) < RecordAreaHeight Then
      Begin { Records do not go all the way to the bottom }
      Rect := GetGridRect ;
      Rect.Top :=  RecordAreaYPosition + (OldBottom - OldTop + 1) * RecordHeight ;
      Rect.Bottom := Rect.Top + (RecordCount - OldTop) * RecordHeight ;
      If Rect.Bottom > (RecordAreaYPosition + RecordAreaHeight) Then
        Rect.Bottom := RecordAreaYPosition + RecordAreaHeight ;
      RedrawRect (Rect) ;
      End
    End
  Else
    Begin {  Deleting Rows }
    If RecordCount < OldBottom Then
      Begin { The change will be visible on the grid. }
      Rect := GetGridRect ;
      Rect.Top :=  RecordAreaYPosition + (RecordCount - OldTop + 1) * RecordHeight ;
      If Rect.Top < RecordAreaYPosition Then
        Rect.Top := RecordAreaYPosition ;
      Rect.Bottom := RecordAreaYPosition + (OldBottom - OldTop + 1) * RecordHeight ;
      If Rect.Bottom > RecordAreaYPosition + RecordAreaHeight Then
        Rect.Bottom := RecordAreaYPosition + RecordAreaHeight ;
      RedrawRect (Rect) ;
      If RecordCount = 0 Then
        Begin
        NewTop := 0
        End
      Else
        Begin
        NewTop := RecordCount - OldBottom + OldTop ;
        If NewTop < 1 Then
          NewTop := 1 ;
        End ;

      TopVisibleRecord := NewTop ;
      End ;
    End ;

  If VerticalScrollBarsAreNeeded <> VisibleScrollBar (FVScrollBar) Then
    LayoutChanged
  Else
    SetVerticalScrollRange ;
  End ;


Procedure TColossalGrid.FindFields ;
  Var
    IZone : Cardinal ;
  Begin
  For IZone := 1 To ZoneCount Do
    Zones [IZone].FindFields ;
  End ;

