{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains the definition of the TColossalGrid class.
}

  TColossalGrid = class (TCustomControl)
    Private
      FMode : TjmGridMode ;
      FDataLink : TjmGridDataLink ;
      FSelectMode : TjmGridSelectMode ;
      FLayoutMode : TjmGridDataLayoutMode ;

      FOnUnboundFetch : TjmCellFetchEvent ;
      FOnUnboundEditFetch : TjmCellFetchEvent ;
      FOnLayoutChanged : TNotifyEvent ;

      FDataZone : Cardinal ;

      { Grid State Variable }

      FDisableColumnResize : Boolean ;
      FDisableColumnMove : Boolean ;
      FDisableRecordResize : Boolean ;
      FDisableRecordMove : Boolean ;

      FRecordHeight : Cardinal ;  { Calculated in FindRecordHeight. }

      FReadOnly : Boolean ;
      FZones : TjmGridZoneList ;  { List of Zones that make up the grid }
      FVariableWidthZone : Cardinal ;   { Zone that expands to fill available space. }
      FActiveZone : Cardinal ;
      FCurrentCell : TjmGridCell ;

      FDefaultEditor : TWinControl ;

      FEditorVisible : Boolean ;
      FActiveEditor : TWinControl ;
      FActive : Boolean ;
      FScrollCount : Integer ;
      FUseArrows : Boolean ;
      FUseTabs : Boolean ;

      { Font Properties }
      FSelectFont : TFont ;
      FHeaderFont : TFont ;
      FFooterFont : TFont ;
      FInactiveFont : TFont ;

      { Scrolling Properties }
      FZoneOffset : Integer ;     { Offset of the leftmost zone. }
      FTopVisibleRecord : LongInt ;
      FShowVerticalScrollBar : TjmGridScrollBars ;
      FShowHorizontalScrollBar : TjmGridScrollBars ;
      FVScrollBar : TScrollBar ;
      FHScrollBar : TScrollBar ;

      FDefaultRowCount : Cardinal ;

      { Font Properties Values }
      FRowLineStyle : TjmGridLineStyle ;
      FRowLineColor : TColor ;
      FHasHeader : Boolean ;
      FHasFooter : Boolean ;

      { Background Color Property Values }
      FUnusedColor : TColor ;
      FBackgroundColor : TColor ;
      FHeaderColor : Tcolor ;
      FFooterColor : Tcolor ;
      FInactiveColor : Tcolor ;
      FSelectColor : TColor ;

      { Lines between zones and scroll bars }
      FHorizontalScrollBarSeparatorStyle : TjmGridLineStyle ;
      FHorizontalScrollBarSeparatorColor : TColor ;
      FVerticalScrollBarSeparatorStyle : TjmGridLineStyle ;
      FVerticalScrollBarSeparatorColor : TColor ;

      { Lines between header/footer and records }
      FHeaderSeparatorStyle : TjmGridLineStyle ;
      FHeaderSeparatorColor : TColor ;
      FFooterSeparatorStyle : TjmGridLineStyle ;
      FFooterSeparatorColor : TColor ;

      { Line Property Values }
      { Lines Between Zones }
      FZoneSeparatorStyle : TjmGridLineStyle ;
      FZoneSeparatorColor : TColor ;
      FVerticalLineColor : TColor ;
      FHorizontalLineStyle : TjmGridLineStyle ;
      FVerticalLineStyle : TjmGridLineStyle ;
      FHorizontalLineColor : TColor ;

      FBorderStyle : TBorderStyle ;
      FRecordCount : LongInt ;
      FRowHeight : Cardinal ;

      { Frame Properties }

      FCellFrameColor : TColor ;
      FCellFrameStyle : TjmGridFrameStyle ;
      FHeaderFrameColor : TColor ;
      FHeaderFrameStyle : TjmGridFrameStyle ;
      FFooterFrameColor : TColor ;
      FFooterFrameStyle : TjmGridFrameStyle ;
      FActiveFrameColor : TColor ;
      FActiveFrameStyle : TjmGridFrameStyle ;

      { Events }
      FOnAppend : TjmGridAppendEvent ;
      FOnDrawCell : TjmGridDrawCellEvent ;
      FOnDrawCellBackground : TjmGridDrawCellBackgroundEvent ;
      FOnDrawHeader : TjmGridDrawHeaderEvent ;
      FOnDrawHeaderBackground : TjmGridDrawHeaderBackgroundEvent ;
      FOnDrawFooter : TjmGridDrawHeaderEvent ;
      FOnDrawFooterBackground : TjmGridDrawHeaderBackgroundEvent ;
      FOnLeaveCell : TjmGridLeaveCellEvent ;
      FOnStartEdit : TNotifyEvent ;
      FOnEndEdit : TNotifyEvent ;
      FOnLeftOffsetChanged : TjmGridLeftOffsetChangedEvent ;
      FOnValidateCell : TjmGridValidateCellEvent ;
      FOnColumnMoved : TjmGridColumnMovedEvent ;
      FOnEnterCell : TNotifyEvent ;

      Function GetOwnerForm : TForm ;

      Procedure ChangeRecordCount (ACount : LongInt) ;


      Function GetColumns (Index : Cardinal) : TjmGridColumn ;
      Procedure SetDataSource (ASource : TDataSource) ;
      Function GetDataSource : TDataSource ;
      Procedure SetMode (AMode : TjmGridMode) ;
      Function GetRecordCount : LongInt ;
      Procedure SetRecordCount (ACount : LongInt) ;
      Procedure SetSelectMode (AMode : TjmGridSelectMode) ;
      Function GetZones (Index : Cardinal) : TjmGridZone ;
      Function GetUsingDatabase : Boolean ;
      Function GetVersion : String ;
      Procedure SetVersion (AVersion : String) ;

      Procedure DataLayoutChanged ;
      Procedure DataLinkActive ;
      Procedure DataLinkInactive ;
      Procedure UpdateDataRecordCount ;
      Procedure DataChangeRecord (ARecord : LongInt) ;
      Procedure UpdateData ;

      { This group exists for debugging.  They could be replaced by direct calls }
      Procedure SetTopVisibleRecord (ARecord : LongInt) ;
      Function GetTopVisibleRecord : LongInt ;

      Procedure SetDataZone (AZone : Cardinal) ;

      { Editor Funcions }
      Procedure EditorKeyDown (Editor : TWinControl ; Var Key : Word ; Shift : TShiftState) ;
      Procedure EditorKeyPress (Editor : TWinControl ; Var Key : Char) ;
      Procedure EditorKillFocus (Message : TWMKillFocus) ;
      Function  UpdateTextFromEditor : Boolean ;

      { Measurement Functions }
      Function EffectiveFooterHeight : Cardinal ;
      Function EffectiveHeaderHeight : Cardinal ;
      Function EffectiveHorizontalScrollbarHeight : Cardinal ;
      Function EffectiveHorizontalZoneScrollBarHeight : Cardinal ;
      Function EffectiveVerticalScrollbarWidth : Cardinal ;
      Function FooterYPosition : Cardinal ;
      Function GetGridRect : Windows.TRect ;
      Function RecordAreaTop : Integer ;
      Function RecordAreaHeight : Cardinal ;
      Function RecordAreaYPosition : Integer ;
      Function VirtualWidth : Cardinal ;
      Function ZoneClientAreaHeight : Cardinal ;

      { Drawing Functions }
      Procedure RedrawRect (const ARect : TRect) ;
      Procedure PositionScrollBars ;
      Procedure PositionZones ;


      { Information Functions }
      Function AnyZonesHaveScrollBars : Boolean ;
      Function HasControl : Boolean ;
      Function WindowValid : Boolean ;

      { Property Functions }
      Procedure SetActive (Value : Boolean) ;
      Procedure SetActiveZone (AZone : Cardinal) ;
      Procedure SetBackgroundColor (AColor : TColor) ;
      Procedure SetBorderStyle (Value : TBorderStyle) ;
      Function GetBottomVisibleRecord : LongInt ;
      Function GetColumnCount : Cardinal ;
      Procedure SetColumnCount (ACount : Cardinal) ;
      Procedure SetRecordIndex (NewRecord : LongInt) ;
      Procedure SetZoneIndex (AZone : Cardinal) ;
      Procedure SetEditorVisible (Mode : Boolean) ;
      Procedure SetFooterColor (AColor : TColor) ;
      Procedure SetFooterFont (AFont : TFont) ;
      Function GetFooterHeight : Cardinal ;
      Procedure SetFooterSeparatorColor (AColor : TColor) ;
      Procedure SetFooterSeparatorStyle (AStyle : TjmGridLineStyle) ;
      Function GetGridColumnCount : Cardinal ;
      Procedure SetHasFooter (AValue : Boolean) ;
      Procedure SetHasHeader (AValue : Boolean) ;
      Procedure SetHeaderColor (AColor : TColor) ;
      Procedure SetHeaderFont (AFont : TFont) ;
      Function GetHeaderHeight : Cardinal ;
      Procedure SetHeaderSeparatorColor (AColor : TColor) ;
      Procedure SetHeaderSeparatorStyle (AStyle : TjmGridLineStyle) ;
      Procedure SetHorizontalLineColor (AColor : TColor) ;
      Procedure SetHorizontalLineStyle (AStyle : TjmGridLineStyle) ;
      Procedure SetInactiveColor (AColor : TColor) ;
      Procedure SetInactiveFont (AFont : TFont) ;
      Function GetMaxVisibleRecords : Cardinal ;
      Procedure SetReadOnly (AState : Boolean) ;
      Procedure SetDefaultRowCount (ACount : Cardinal) ;
      Procedure SetRowHeight (AHeight : Cardinal) ;
      Procedure SetRowLineColor (AColor : TColor) ;
      Procedure SetRowLineStyle (AStyle : TjmGridLineStyle) ;
      Function GetRows (Index : Cardinal) : TjmGridRow ;
      Procedure SetRowCount (ACount : Cardinal) ;
      Function GetRowCount : Cardinal ;
      Procedure SetSelectColor (AColor : TColor) ;
      Procedure SetSelectFont (AFont : TFont) ;
      Procedure SetShowHorizontalScrollBar (Mode : TjmGridScrollBars) ;
      Procedure SetShowVerticalScrollBar (Mode : TjmGridScrollBars) ;
      Procedure SetUnusedColor (AColor : TColor) ;
      Procedure SetVariableWidthZone (AZone : Cardinal) ;
      Procedure SetVerticalLineColor (AColor : TColor) ;
      Procedure SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;
      Function GetZoneCount : Cardinal ;
      Procedure SetZoneCount (ACount : Cardinal) ;
      Procedure SetZoneSeparatorColor (AColor : TColor) ;
      Procedure SetZoneSeparatorStyle (AStyle : TjmGridLineStyle) ;

      Procedure SetCurrentCell (ACell : TjmGridCell) ;
      Procedure SetHorizontalScrollBarSeparatorStyle (AStyle : TjmGridLineStyle) ;
      Procedure SetHorizontalScrollBarSeparatorColor (AColor : TColor) ;
      Procedure SetVerticalScrollBarSeparatorStyle (AStyle : TjmGridLineStyle) ;
      Procedure SetVerticalScrollBarSeparatorColor (AColor : TColor) ;

      Procedure SetCellFrameColor (AColor : TColor) ;
      Procedure SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
      Procedure SetHeaderFrameColor (AColor : TColor) ;
      Procedure SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
      Procedure SetFooterFrameColor (AColor : TColor) ;
      Procedure SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
      Procedure SetActiveFrameColor (AColor : TColor) ;
      Procedure SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;

      { Scrolling Functions }
      Procedure DisplayHorizontalScrollBar (AShow : Boolean) ;
      Procedure DisplayVerticalScrollBar (AShow : Boolean) ;
      Procedure HorizontalScrollHandler (Sender : TObject ;
                                         ScrollCode : TScrollCode ;
                                         var ScrollPos : Integer) ;
      Procedure SetVerticalScrollRange ;
      Procedure VerticalScrollHandler (Sender : TObject ;
                                       ScrollCode : TScrollCode ;
                                       var ScrollPos : Integer) ;
      Function VerticalScrollbarsAreNeeded : Boolean ;

      { Grid Actions }
      Procedure AddZone ;
      Procedure CellsMoved ;
      Procedure FontChanged (Sender : TObject) ;
      Procedure AttachZone (Zone : TjmGridZone) ;
      Procedure AdjustRowCount ;
      Procedure PositionColumnsInRows ;
      Procedure FindRecordHeight ;

      { Editing Functions }
      Procedure MoveDown ;
      Procedure MoveLeft ;
      Procedure MoveRight ;
      Procedure MoveUp ;
      Procedure HideEditor (GridSetFocus : Boolean) ;

      {--Message Procedures--}
      Procedure WMGetDlgCode (var Message : TwmGetDlgCode) ; Message WM_GETDLGCODE ;
      Procedure WMSize (var Message : TWmSize) ; Message WM_SIZE ;
      Procedure WMSetFocus (var Message : TWMSetFocus) ; Message WM_SETFOCUS ;
      Procedure WMKillFocus (var Message : TWMKillFocus) ; Message WM_KILLFOCUS ;
      Procedure CNExit (var Message : TCMExit) ; Message CM_EXIT ;

      Procedure FindFields ;
    Protected   {*** TColossalGrid ***}
      {--Overriden Procedures--}
      Procedure Notification (AComponent : TComponent ;
                              Operation : TOperation) ; Override ;
      procedure CreateParams (var Params : TCreateParams) ; Override ;
      Procedure CreateWnd ; Override ;
      Procedure KeyDown (var Key : Word ; Shift : TShiftState) ; Override ;
      Procedure KeyPress(var Key : Char) ; Override ;

      procedure MouseDown (Button : TMouseButton ;
                           Shift : TShiftState ;
                           X , Y : Integer) ; Override ;
      Procedure Paint ; Override ;
      Procedure GetChildren (Proc : TGetChildProc ; Root : TComponent) ; Override ;
      Function GetChildOwner : TComponent ; Override ;
      Function GetChildParent : TComponent ; Override ;
      procedure Loaded ; Override ;


      { Grid Actions }
      Procedure DeleteZones ;
      Procedure RedrawCell (ACell : TjmGridCell) ;
      Procedure RedrawHeader ;
      Procedure RedrawRecord (ARecord : LongInt) ;
      Procedure RedrawRecordArea ;
      Procedure CancelEditing ; Virtual ;

      { These functions are intended to be overriden }
      Procedure ChangeCurrentCell (NewCell : TjmGridCell) ; Virtual ;
      Procedure ChangeCurrentRecord (NewRecord : LongInt) ; Virtual ;
      Function CreateColumn (AZone : TjmGridZone) : TjmGridColumn ; Virtual ;
      Function CreateZone : TjmGridZone ; Virtual ;
      Procedure DrawBackground ; Virtual ;
      Function GridControlsWindow (Window : HWND) : Boolean ; Virtual ;
      Function GridTop : Integer ; Virtual ;
      Function GridLeft : Integer ; Virtual ;
      Function GridHeight : Integer ; Virtual ;
      Function GridWidth : Integer ; Virtual ;
      Function IsCellBoxed (AZone : Cardinal ; ACol : Cardinal ; ARecord : LongInt) : Boolean ; Virtual ;
      Function IsCellSelected (AZone : Cardinal ; ACol : Cardinal ; ARecord : LongInt) : Boolean ; Virtual ;
      Function IsCurrentCell (AZone : Cardinal ; ACol : Cardinal ; ARecord : LongInt) : Boolean ; Virtual ;
      Procedure LayoutChanged ; Virtual ;
      Procedure MoveToNextRecord (AColumn : Cardinal) ; Virtual ;
      Procedure MoveToPreviousRecord (AColumn : Cardinal) ; Virtual ;
      Procedure MoveToFirstRecord ; Virtual ;
      Procedure MoveToLastRecord ; Virtual ;
      Procedure MovePageDown ; Virtual ;
      Procedure MovePageUp ; Virtual ;
      Procedure PositionEditor ; Virtual ;
      Procedure ResizeGrid ; 

      { Properties }


      { Events }

    Public     {*** TjmColossalGrid ***}
      Constructor Create (AOwner : TComponent) ; Override ;
      Destructor Destroy ; Override ;

      Procedure MoveZone (OldPosition : Cardinal ; NewPosition : Cardinal) ;
      Function CellRect (Zone : Cardinal ; Column : Cardinal ; RecordNumber : LongInt) : TRect ;
      Function GridPosition (XPos : Integer ;
                             YPos : Integer) : TjmGridPosition ;

      Property Active : Boolean
               Read FActive
               Write SetActive
               Stored False ;
      { Must not be published. }
      Property ActiveEditor : TWinControl
               Read FActiveEditor ;
      { Must not be published. }
      Property Canvas ;
      Property ColumnIndex : Cardinal
               Read FCurrentCell.Column
               Stored False ;
      Property Columns [Index : Cardinal] : TjmGridColumn
               Read GetColumns ;
      Property CurrentCell : TjmGridCell
               Read FCurrentCell
               Write SetCurrentCell
               Stored False ;
      Property EditorVisible : Boolean
               Read FEditorVisible
               Write SetEditorVisible
               Stored False ;
      { Must not be published. }
      Property RecordIndex : LongInt
               Read FCurrentCell.RecordNumber
               Write SetRecordIndex
               Stored False ;
      Property Rows [Index : Cardinal] : TjmGridRow
               Read GetRows ;
      { Must not be published. }
      Property UsingDatabase : Boolean
               Read GetUsingDatabase ;
      Property Zones [Index : Cardinal] : TjmGridZone
               Read GetZones ;
      Property ZoneIndex : Cardinal
               Read FCurrentCell.Zone
               Write SetZoneIndex
               Stored False ;

    Published    {*** TColossalGrid ***}
      Property Align ;
      Property ActiveFrameColor : TColor
               Read FActiveFrameColor
               Write SetActiveFrameColor ;
      Property ActiveFrameStyle : TjmGridFrameStyle
               Read FActiveFrameStyle
               Write SetActiveFrameStyle ;
      Property ActiveZone : Cardinal
               Read FActiveZone
               Write SetActiveZone
               Default 1 ;
      Property BackgroundColor : TColor
               Read FBackgroundColor
               Write SetBackgroundColor
               Default jmDefaultBackgroundColor ;
      Property BorderStyle : TBorderStyle
               Read FBorderStyle
               Write SetBorderStyle
               Default jmDefaultBorderStyle ;
      Property BottomVisibleRecord : LongInt
               Read GetBottomVisibleRecord
               Stored False ;
      Property CellFrameColor : TColor
               Read FCellFrameColor
               Write SetCellFrameColor ;
      Property CellFrameStyle : TjmGridFrameStyle
               Read FCellFrameStyle
               Write SetCellFrameStyle ;
      Property Color ;
      Property ColumnCount : Cardinal
               Read GetColumnCount
               Write SetColumnCount
               Stored False ;
      Property Ctl3D ;
      Property Cursor ;
      Property DataSource : TDataSource
               Read GetDataSource
               Write SetDataSource ;
      Property DataZone : Cardinal
               Read FDataZone
               Write SetDataZone ;
      Property DefaultRowCount : Cardinal
               Read FDefaultRowCount
               Write SetDefaultRowCount
               Default 1 ;
      Property DisableColumnMove : Boolean
               Read FDisableColumnMove
               Write FDisableColumnMove
               Default False ;
      Property DisableColumnResize : Boolean
               Read FDisableColumnResize
               Write FDisableColumnResize
               Default False ;
      Property DisableRecordResize : Boolean
               Read FDisableRecordResize
               Write FDisableRecordResize
               Default True ;
      Property DisableRecordMove : Boolean
               Read FDisableRecordMove
               Write FDisableRecordMove
               Default True ;
      Property DragCursor ;
      { Whatever you do, do not publish the drag mode property. It
        screws everything up. }
      Property Enabled ;
      Property Font ;
      Property FooterColor : TColor
               Read FFooterColor
               Write SetFooterColor
               Default jmDefaultFooterColor ;
      Property FooterFont : TFont
               Read FFooterFont
               Write SetFooterFont ;
      Property FooterFrameColor : TColor
               Read FFooterFrameColor
               Write SetFooterFrameColor ;
      Property FooterFrameStyle : TjmGridFrameStyle
               Read FFooterFrameStyle
               Write SetFooterFrameStyle ;
      Property FooterHeight : Cardinal
               Read GetFooterHeight
               Stored False ;
      Property FooterSeparatorColor : Tcolor
               Read FFooterSeparatorColor
               Write SetFooterSeparatorColor
               Default jmDefaultFooterSeparatorColor ;
      Property FooterSeparatorStyle : TjmGridLineStyle
               Read FFooterSeparatorStyle
               Write SetFooterSeparatorStyle
               Default jmDefaultFooterSeparatorStyle ;
      Property GridColumnCount : Cardinal
               Read GetGridColumnCount
               Stored False ;
      Property HasFooter : Boolean
               Read FHasFooter
               Write SetHasFooter
               Default True ;
      Property HasHeader : Boolean
               Read FHasHeader
               Write SetHasHeader
               Default True ;
      Property HeaderColor : TColor
               Read FHeaderColor
               Write SetHeaderColor
               Default jmDefaultHeaderColor ;
      Property HeaderFont  : TFont
               Read FHeaderFont
               Write SetHeaderFont ;
      Property HeaderFrameColor : TColor
               Read FHeaderFrameColor
               Write SetHeaderFrameColor ;
      Property HeaderFrameStyle : TjmGridFrameStyle
               Read FHeaderFrameStyle
               Write SetHeaderFrameStyle ;
      Property HeaderHeight : Cardinal
               Read GetHeaderHeight
               Stored False ;
      Property HeaderSeparatorColor : Tcolor
               Read FHeaderSeparatorColor
               Write SetHeaderSeparatorColor
               Default jmDefaultHeaderSeparatorColor ;
      Property HeaderSeparatorStyle : TjmGridLineStyle
               Read FHeaderSeparatorStyle
               Write SetHeaderSeparatorStyle
               Default jmDefaultHeaderSeparatorStyle ;
      Property HelpContext ;
      Property Hint ;
      Property HorizontalLineColor : TColor
               Read FHorizontalLineColor
               Write SetHorizontalLineColor
               Default jmDefaultHorizontalLineColor ;
      Property HorizontalLineStyle : TjmGridLineStyle
               read FHorizontalLineStyle
               Write SetHorizontalLineStyle
               Default jmDefaultHorizontalLineStyle ;
      Property HorizontalScrollBarSeparatorStyle : TjmGridLineStyle
               Read FHorizontalScrollBarSeparatorStyle
               Write SetHorizontalScrollBarSeparatorStyle
               Default jmDefaultHorizontalScrollBarSeparatorStyle ;
      Property HorizontalScrollBarSeparatorColor : TColor
               Read FHorizontalScrollBarSeparatorColor
               Write SetHorizontalScrollBarSeparatorColor
               Default jmDefaultHorizontalScrollBarSeparatorColor ;
      Property InactiveColor : Tcolor
               Read FInactiveColor
               Write SetInactiveColor
               Default jmDefaultInactiveColor ;
      Property InactiveFont : TFont
               Read FInactiveFont
               Write SetInactiveFont ;
      Property LayoutMode : TjmGridDataLayoutMode
               Read FLayoutMode
               Write FLayoutMode
               Default jmDefaultLayoutMode ;
      Property MaxVisibleRecords : Cardinal
               Read GetMaxVisibleRecords
               Stored False ;
      Property Mode : TjmGridMode
               Read FMode
               Write SetMode
               Default jmDefaultGridMode ;
      Property ParentColor ;
      Property ParentCtl3D ;
      Property ParentFont ;
      Property ParentShowHint ;
      Property PopupMenu ;
      Property ReadOnly : Boolean
               Read FReadOnly
               Write SetReadonly
               Default False ;
      Property RecordCount : LongInt
               Read GetRecordCount
               Write SetRecordCount
               Default 0 ;
      Property RecordHeight : Cardinal
               Read FRecordHeight
               Stored False ;
      Property RowCount : Cardinal
               Read GetRowCount
               Write SetRowCount
               Stored False ;
      Property RowHeight : Cardinal
               Read FRowHeight
               Write SetRowHeight
               Default jmDefaultRowHeight ;
      Property RowLineColor : TColor
               Read FRowLineColor
               Write SetRowLineColor
               Default jmDefaultRowLineColor ;
      Property RowLineStyle : TjmGridLineStyle
               Read FRowLineStyle
               Write SetRowLineStyle
               Default jmDefaultRowLineStyle ;
      Property SelectColor : TColor
               Read FSelectColor
               Write SetSelectColor
               Default jmDefaultSelectColor ;
      Property SelectFont : TFont
               Read FSelectFont
               Write SetSelectFont ;
      Property SelectMode : TjmGridSelectMode
               Read FSelectMode
               Write SetSelectMode
               Default jmDefaultSelectMode ;
      Property ShowHint ;
      Property ShowHorizontalScrollBar : TjmGridScrollBars
               Read FShowHorizontalScrollBar
               Write SetShowHorizontalScrollBar
               Default sbWhenNeeded ;
      Property ShowVerticalScrollBar : TjmGridScrollBars
               read FShowVerticalScrollBar
               Write SetShowVerticalScrollBar
               Default sbWhenNeeded ;
      Property TabOrder ;
      Property TabStop ;
      Property Tag ;
      Property TopVisibleRecord : LongInt
               Read GetTopVisibleRecord
               Write SetTopVisibleRecord
               Stored False ;
      Property UnusedColor : TColor
               Read FUnusedColor
               Write SetUnusedColor
               Default jmDefaultUnusedColor ;
      Property UseArrows : Boolean
               Read FUseArrows
               Write FUseArrows
               Default True ;
      Property UseTabs : Boolean
               Read FUseTabs
               Write FUseTabs
               Default True ;
      Property VariableWidthZone : Cardinal
               Read FVariableWidthZone
               Write SetVariableWidthZone
               Default 1 ;
      Property Version : String
               Read GetVersion
               Write SetVersion
               Stored False ;
      Property VerticalLineColor : TColor
               Read FVerticalLineColor
               Write SetVerticalLineColor
               Default jmDefaultVerticalLineColor ;
      Property VerticalLineStyle : TjmGridLineStyle
               Read FVerticalLineStyle
               Write SetVerticalLineStyle
               Default jmDefaultVerticalLineStyle ;
      Property VerticalScrollBarSeparatorStyle : TjmGridLineStyle
               Read FVerticalScrollBarSeparatorStyle
               Write SetVerticalScrollBarSeparatorStyle
               Default jmDefaultVerticalScrollBarSeparatorStyle ;
      Property VerticalScrollBarSeparatorColor : TColor
               Read FVerticalScrollBarSeparatorColor
               Write SetVerticalScrollBarSeparatorColor
               Default jmDefaultVerticalScrollBarSeparatorColor ;
      Property Visible ;  
      Property ZoneCount : Cardinal
               Read GetZoneCount
               Write SetZoneCount
               Stored False ;
      property ZoneSeparatorColor : TColor
               Read FZOneSeparatorColor
               Write SetZoneSeparatorColor
               Default jmDefaultZoneSeparatorColor ;
      Property ZoneSeparatorStyle : TjmGridLineStyle
               Read FZoneSeparatorStyle
               Write SetZoneSeparatorStyle
               Default jmDefaultZoneSeparatorStyle ;

      { Events }
      { Called whenever the user attempts to move down while on the last record. }
      Property OnAppend : TjmGridAppendEvent
               Read FOnAppend
               Write FOnAppend ;

      { Called when a column is moved. }
      Property OnColumnMoved : TjmGridColumnMovedEvent
               Read FOnColumnMoved
               Write FOnColumnMoved ;

      { Called before each cell is drawn. }
      Property OnDrawCell : TjmGridDrawCellEvent
               Read FOnDrawCell
               Write FOnDrawCell ;
      { Called before each cell background is drawn. }
      Property OnDrawCellBackground : TjmGridDrawCellBackgroundEvent
               Read FOnDrawCellBackground
               Write FOnDrawCellBackground ;

      { Called before each header is drawn. }
      Property OnDrawHeader : TjmGridDrawHeaderEvent
               Read FOnDrawHeader
               Write FOnDrawHeader ;
      { Called before each header background is drawn. }
      Property OnDrawHeaderBackground : TjmGridDrawHeaderBackgroundEvent
               Read FOnDrawHeaderBackground
               Write FOnDrawHeaderBackground ;

      { Called before each footer is drawn. }
      Property OnDrawFooter : TjmGridDrawHeaderEvent
               Read FOnDrawFooter
               Write FOnDrawFooter ;
      { Called before each footer background is drawn. }
      Property OnDrawFooterBackground : TjmGridDrawHeaderBackgroundEvent
               Read FOnDrawFooterBackground
               Write FOnDrawFooterBackground ;

      Property OnEnterCell : TNotifyEvent
               Read FOnEnterCell
               Write FOnEnterCell ;

      { Called before changing the current cell. }
      Property OnLeaveCell : TjmGridLeaveCellEvent
               Read FOnLeaveCell
               Write FOnLeaveCell ;

      { Called just before the cell editor is shown. }
      Property OnStartEdit : TNotifyEvent
               Read FOnStartEdit
               Write FOnStartEdit ;
      { called just before the cell editor is hidden. }
      Property OnEndEdit : TNotifyEvent
               Read FOnEndEdit
               Write FOnEndEdit ;

      { Called when a zone is scrolled. }
      Property OnLeftOffsetChanged : TjmGridLeftOffsetChangedEvent
               Read FOnLeftOffsetChanged
               Write FOnLeftOffsetChanged ;

      Property OnValidateCell : TjmGridValidateCellEvent
               Read FOnValidateCell
               Write FOnvalidateCell ;

      Property OnClick ;
      Property OnDblClick ;
      Property OnDragDrop ;
      Property OnDragOver ;
      Property OnKeyDown ;
      Property OnKeyPress ;
      Property OnKeyUp ;
      Property OnEnter ;
      Property OnExit ;
      Property OnLayoutChanged : TNotifyEvent
               Read FOnLayoutChanged
               Write FOnLayoutChanged ;
      Property OnMouseDown ;
      Property OnMouseMove ;
      Property OnMouseUp ;
//OnPaint????

      { Retreives text for editing. }
      Property OnUnboundEditFetch : TjmCellFetchEvent
               Read FOnUnboundEditFetch
               Write FOnUnboundEditFetch ;
      { Retreives text for display. }
      Property OnUnboundFetch : TjmCellFetchEvent
               Read FOnUnboundFetch
               Write FOnUnboundFetch ;


    End ;        {*** TColossalGrid ***}


