{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains TColossalGrid methods that deal with drawing.
}

{
  Procedure:  TColossalGrid.PositionZones

  Description: Assigns the zones their physical location in the grid.

  Parameters: None

}
Procedure TColossalGrid.PositionZones ;
    Var
    IZone : Integer ;
    AvailableSpace : Integer ;
    Offset : Integer ;
    HScroll : Boolean ;
    GridRect : TRect ;
    ZoneHeight : Integer ;

  { Nested function for hiding/showing the grid horizontal scroll bar. }
  Procedure DetermineHorizontalScrollbarState ;
    Begin
    {  We need a scroll bar if the minimum zone space is greater than the
       available space or of the zones are scrolled.
    }
    If ((VirtualWidth > AvailableSpace) Or (FZoneOffset <> 0))
       And (FZones.Count > 1) Then
      Begin { Zone widths dictate scroll bars. }
      If Self.HandleAllocated Then
        Begin
        If ShowHorizontalScrollBar <> sbNever Then
          Begin
          DisplayHorizontalScrollbar (True) ;
          FHScrollBar.Min := 0 ;
          FHScrollBar.Max := VirtualWidth ;
          FHScrollBar.PageSize := AvailableSpace ;
          FHScrollBar.Position := FZoneOffset ;
          End
        Else
          Begin { The user never wants scroll bars. }
          DisplayHorizontalScrollbar (False) ;
          End ;
        End ;
      End
    Else
      Begin { Scroll bars are not required by the zone widths. }
      If ShowHorizontalScrollBar = sbAlways Then
        Begin
        DisplayHorizontalScrollBar (True) ;
        FHScrollBar.Min := 0 ;
        FHScrollBar.Max := 0 ;
        FHScrollBar.Position := 0 ;
        FHScrollBar.PageSize := (GridRect.Right - GridRect.Left);
        End
      Else
        Begin { The user wants scroll bars no matter what. }
        DisplayHorizontalScrollBar (False) ;
        End ;
      End ;
    End ;

  Begin

  { There's nothing we can do until a window has been created. }
  If Not HandleAllocated Then
     Begin
     exit ;
     End ;

  GridRect := GetGridRect ;

  DisplayVerticalScrollBar (VerticalScrollBarsAreNeeded) ;

  { Find the width of the client area allowing for a scroll bar. }
  AvailableSpace := GridRect.Right - GridRect.Left
                                   - LongInt (EffectiveVerticalScrollBarWidth) ;

  { If minimum sizes for all of the zones is greater than the size of
    the control then we need a horizontal scroll bar for the control.
  }

  DetermineHorizontalScrollbarState ;

  { Find the height of the client area allowing for a scroll bar. }
  ZoneHeight := GridRect.Bottom - GridRect.Top
                - LongInt (EffectiveHorizontalScrollBarHeight) ;

  { Adjust the zone widths }

  If ZoneCount = 1 Then
    Begin { If there is only one zone then we do not want a grid scroll bar. }
    Zones [1].Top := GridRect.Top ;
    Zones [1].Left := GridRect.Left ;
    Zones [1].Width := AvailableSpace ;
    FZoneOffset := 0 ;
    End
  Else
    Begin { Width adjustments for more than one zone. }
    Offset := GridRect.Left ;
    For IZone := 1 To ZoneCount Do
      Begin

      { Allow space for a line between the zones. }
      If (IZone <> 1) Then
        Inc (Offset, LineWidth (FZoneSeparatorStyle)) ;

      Zones [IZone].Top := GridRect.Top ;
      Zones [IZone].Left := Offset - FZoneOffset ;

      If (IZone = LongInt (FVariableWidthZone)) And
        (VirtualWidth < AvailableSpace) Then
        Begin { We have extra space so make the variable zone fill it. }
        Zones [IZone].Width := AvailableSpace - VirtualWidth + Zones [IZone].ZoneWidth ;
        End
      Else
        Begin
        { Make sure the last visible zone ends at the grid border }
        If Zones [IZone].Left + Zones [IZone].ZoneWidth < AvailableSpace Then
          Begin
          Zones [IZone].Width := Zones [IZone].ZoneWidth ;
          End
        Else
          Begin
          Zones [IZone].Width := AvailableSpace - Zones [IZone].Left ;
          End ;
        End ;
      Inc (Offset, Zones [IZone].Width) ;
      End ; { Width Loop }
    End ; { Width Adjustments. }

  { When the size of the grid is increased and a zone has been scrolled it
    may need to be shifted over. }
  For IZone := 1 To ZoneCount Do
    Begin
    If Zones [IZone].RowWidth <= Zones [IZone].Width Then
      Begin
      Zones [IZone].FLeftOffset := 0 ;
      End
    Else If (Zones [IZone].FLeftOffset + Zones [IZone].Width) > Zones [IZone].RowWidth Then
      Begin
      Zones [IZone].FLeftOffset := Zones [IZone].RowWidth - Zones [IZone].Width ;
      End ;
    End ;

  HScroll := AnyZonesHaveScrollBars ;
  For IZone := 1 To ZoneCount Do
    Begin
    { If any zones have scroll bars then the zones without scroll
     bars need to have their height reduced so they will be even
     with the others. }
    If HScroll And Not Zones [IZone].HasHorizontalScrollBar Then
      Zones [IZone].Height := ZoneHeight - FHScrollBar.Height
    Else
      Zones [IZone].Height := ZoneHeight ;
    Zones [IZone].ResizeZone ; { Move the scroll bars }
    End ; { Height Adjustment for Scroll Bars }

  PositionScrollBars ;
  End ; { PositionZones }

Procedure TColossalGrid.PositionScrollBars ;
  Var
    Rect : TRect ;
    LargeVertical : Integer ;
  Begin
  Rect := GetGridRect ;

  If Assigned (FVScrollBar) Then
    Begin
    FVScrollBar.Kind := sbVertical ;
    FVScrollBar.Top := Rect.Top + Self.EffectiveHeaderHeight ;
    FVScrollBar.Left := Rect.Right - FVScrollBar.Width ;
    FVScrollBar.Height := RecordAreaHeight ;
    If RecordHeight + LineWidth (HorizontalLineStyle) > 0 Then
      Begin
      LargeVertical := RecordAreaHeight
                    Div (RecordHeight + LineWidth (HorizontalLineStyle)) - 1 ;
      End
    Else
      Begin
      LargeVertical := 1 ;
      End ;
      
    If LargeVertical < 1 Then
       LargeVertical := 1 ;

    FVScrollBar.LargeChange := LargeVertical ;
    End ;

  If Assigned (FHScrollBar) Then
    Begin
    FHScrollBar.Kind := sbHorizontal ;
    FHScrollBar.Top := Rect.Bottom - FHScrollBar.Height ;
    FHScrollBar.Left := Rect.Left ;
    FHScrollBar.Width := Rect.Right - Rect.Left
                       - EffectiveVerticalScrollBarWidth ;
    End ;

  SetVerticalScrollRange ;
  End ;

Procedure TColossalGrid.DrawBackground ;
  Var
    DrawRect : TRect ;
    GridRect : TRect ;
    Rect : TRect ;
  Begin
  Canvas.Brush.Color := Color ;
  Canvas.Brush.Style := bsSolid ;

  If VisibleScrollBar (FVScrollbar) Then
    Begin
    { We draw around the scroll bar to keep it from making
      a funny flash the occcurs when the scroll bar is redrawn
      because we just drew over it.
    }
    DrawRect := GetGridRect ;
    DrawRect.Left := DrawRect.Right - FVScrollBar.Width ;
    DrawRect.Bottom := FVScrollBar.Top ;
    Canvas.FillRect (DrawRect) ;

    DrawRect := GetGridRect ;
    DrawRect.Left := DrawRect.Right - FVScrollBar.Width ;
    DrawRect.Top := FVScrollBar.Top + FVScrollBar.Height ;

    If VisibleScrollBar (FHScrollBar) Then
      Dec (DrawRect.Bottom, FHScrollBar.Height) ;
    Canvas.FillRect (DrawRect) ;
    End ; { Vertical Scrollbar Visible }

  GridRect := GetGridRect ;
  Rect := ClientRect ;
  If Rect.Left < GridRect.Left Then
    Begin
    DrawRect := Rect ;
    DrawRect.Right := GridRect.Left ;
    Canvas.FillRect (DrawRect) ;
    End ;
  If Rect.Right > GridRect.Right Then
    Begin
    DrawRect := Rect ;
    DrawRect.Left := GridRect.Right ;
    Canvas.FillRect (DrawRect) ;
    End ;
  If Rect.Top < GridRect.Top Then
    Begin
    DrawRect := Rect ;
    DrawRect.Bottom := GridRect.Top ;
    Canvas.FillRect (DrawRect) ;
    End ;
  If Rect.Bottom > GridRect.Top Then
    Begin
    DrawRect := Rect ;
    DrawRect.Top := GridRect.Bottom ;
    Canvas.FillRect (DrawRect) ;
    End ;
  End ;

Procedure TColossalGrid.RedrawRect (const ARect : TRect) ;
  Begin
  InvalidateRect (Handle, @ARect, False) ;
  End ;

