{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the TColossalGrid methods that deal with editing.
}

Procedure TColossalGrid.MoveUp ;
  Var
    NewCol : Cardinal ;
    NewCell : TjmGridCell ;
  Begin
  If (FCurrentCell.Zone = 0)
     Or (FCurrentCell.Column = 0)
     Or (FCurrentCell.RecordNumber = 0) Then
    Begin
    Exit ;  { No Current Cell }
    End ;

  { Make sure the user wants to the user set this value. }
  If EditorVisible Then
    Begin
    If Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column].UpdateValue (GetEditorText (FActiveEditor)) Then
      Exit ;
    End ;

  NewCol := 0 ;
  If Zones [FCurrentCell.Zone].MoveUp (FCurrentCell.Column, FCurrentCell.RecordNumber, NewCol) Then
    Begin
    { A true return value means the zone wants us to move to the previous record. }
    If (FCurrentCell.RecordNumber > 1) OR (FMode = gmDatabase) Then
      MoveToPreviousRecord (NewCol) ;
    End
  Else If (RecordCount > 0)  Then
    Begin
    { A false return value means the zone wants us to stay on the same record. }
    NewCell := FCurrentCell ;
    NewCell.Column := NewCol ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

Procedure TColossalGrid.MoveDown ;
  Var
    NewCol : Cardinal ;
    NewCell : TjmGridCell ;
    Column : TjmGridColumn ;
  Begin
  If (FCurrentCell.Zone = 0)
     Or (FCurrentCell.Column = 0)
     Or (FCurrentCell.RecordNumber = 0) Then
    Begin
    Exit ;  { No Current Cell }
    End ;

  { Make sure the user wants to the user set this value. }
  If EditorVisible Then
    Begin
    Column := Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column] ;
    If Column.UpdateValue (GetEditorText (FActiveEditor)) Then
      Exit ;
    End ;

  NewCol := 0 ;
  If Zones [FCurrentCell.Zone].MoveDown (FCurrentCell.Column, FCurrentCell.RecordNumber, NewCol) Then
    Begin
    { A true return value means the zone wants us to move to the next record. }
    If (RecordCount = 0) Or (FCurrentCell.RecordNumber = RecordCount)
       And Assigned (FOnAppend) Then
      Begin
      FOnAppend (Self) ;
      Zones [FCurrentCell.Zone].MoveDown (FCurrentCell.Column, FCurrentCell.RecordNumber, NewCol) ;
      End ;

    If (RecordCount > 0) And (FcurrentCell.RecordNumber < RecordCount) THen
      Begin
      MoveToNextRecord (NewCol) ;
      End
    End
  Else
    Begin
    { A false return value means the zone wants us to stay on the same record. }
    If (RecordCount > 0) THen
      Begin
      NewCell := FCurrentCell ;
      NewCell.Column := NewCol ;
      ChangeCurrentCell (NewCell) ;
      End
    End ;

  End ;

Procedure TColossalGrid.MoveLeft ;
  Var
    NewCol : Cardinal ;
    NewCell : TjmGridCell ;

  Procedure ZoneHandledMove ;
    Begin
    NewCell.Column := NewCol ;
    If NewCell.RecordNumber > BottomVisibleRecord Then
      TopVisibleRecord := TopVisibleRecord + 1 ;
    ChangeCurrentCell (NewCell) ;
    End ;
  { The zone was not able to change the current cell. We need to move to
    the previous zone. }
  Procedure GridHandlesMove ;
    Var
      IZone : Integer ;
    Begin
    For IZone := FCurrentCell.Zone - 1 DownTo 1 Do
      Begin
      If Zones [IZone].Selectable And (Zones [IZone].ColumnCount > 0) Then
        Begin
        NewCell.Zone := IZone ;
        NewCell.Column := Zones [IZone].ColumnCount ;
        Zones [IZone].MakeColumnVisible (NewCell.Column) ;
        ChangeCurrentCell (NewCell) ;
        Exit ;
        End ;
      End ;
    End ;

  Begin
  If (FCurrentCell.Zone = 0)
     Or (FCurrentCell.Column = 0)
     Or (FCurrentCell.RecordNumber = 0) Then
    Begin
    Exit ;  { No Current Cell }
    End ;

  { Make sure the user wants to the user set this value. }
  If EditorVisible Then
    Begin
    If Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column].UpdateValue (GetEditorText (FActiveEditor)) Then
      Exit ;
    End ;

  NewCell := FCurrentCell ;
  If Zones [FCurrentCell.Zone].MoveLeft (FCurrentCell.Column, NewCol) Then
    ZoneHandledMove
  Else
    GridHandlesMove ;
  End ;

{
  This functiojn moves the current cell to the right. This is performed
  in response to the right arrow key.
}
Procedure TColossalGrid.MoveRight ;
  Var
    NewCol : Cardinal ;
    NewCell : TjmGridCell ;
    Column : TJmGridColumn ;

  { This function is used when the zone is able to move the current cell. }
  Procedure ZoneHandledMove ;
    Begin
    NewCell.Column := NewCol ;
    If NewCell.RecordNumber > BottomVisibleRecord Then
      TopVisibleRecord := TopVisibleRecord + 1 ;
    ChangeCurrentCell (NewCell) ;
    End ;

  { This function is used to move the current cell to the right when
    the zone was not able to move the current cell. }
  Procedure GridHandlesMove ;
    Var
      IZone : Integer ;
    Begin
    { We need to more for the first column in the next zone. }
    For IZone := FCurrentCell.Zone + 1 To ZoneCount Do
      Begin
      If Zones [IZone].Selectable And (Zones [IZone].ColumnCount > 0) Then
        Begin
        NewCell.Zone := IZone ;
        NewCell.Column := 1 ;
        Zones [IZone].MakeColumnVisible (NewCell.Column) ;
        ChangeCurrentCell (NewCell) ;
        Exit ;
        End ;
      End ;
    End ;

  Begin
  If (FCurrentCell.Zone = 0)
     Or (FCurrentCell.Column = 0)
     Or (FCurrentCell.RecordNumber = 0) Then
    Begin
    Exit ;  { No Current Cell }
    End ;
  { Make sure the user wants to the user set this value. }
  If EditorVisible Then
    Begin
    Column := Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column] ;
    If Column.UpdateValue (GetEditorText (FActiveEditor)) Then
      Exit ;
    End ;

  { See if the current can handle the move operation. }
  NewCell := FCurrentCell ;
  If Zones [FCurrentCell.Zone].MoveRight (FCurrentCell.Column, NewCol) Then
    ZoneHandledMove
  Else
    GridHandlesMove ;
  End ;

Procedure TColossalGrid.PositionEditor ;
  Var
    Zone : TjmGridZone ;
  Begin
  If (FCurrentCell.Zone = 0)
     Or (FCurrentCell.Column = 0)
     Or (FCurrentCell.RecordNumber = 0) Then
    Begin
    Exit ;
    End ;

  If (FCurrentCell.Zone > ZoneCount) Then
    Raise Exception.CreateFmt ('Current zone number (%d) out of range 1..%d', [FCurrentCell.Zone, ZoneCount]) ;
  If (FCurrentCell.Column > Zones [FCurrentCell.Zone].ColumnCount) Then
    Raise Exception.CreateFmt ('Current cell number (%d) out of range 1..%d', [FCurrentCell.Column, Zones [FCurrentCell.Zone].ColumnCount]) ;
  If (FCurrentCell.RecordNumber > RecordCount) Then
    Raise Exception.CreateFmt ('Current record number (%d) out of range 1..%d', [FCurrentCell.RecordNumber, RecordCount]) ;

  Zone := Zones [FCurrentCell.Zone] ;

  { Under any of these conditions editing of the cell is not allowed. }
  If ReadOnly Or Zone.ReadOnly Or Zone.Columns [FCurrentCell.Column].ReadOnly
     Or (csDesigning In ComponentState)
     Or (csLoading In ComponentState) Then
   Begin
   Exit ;
   End ;

  FActiveEditor := Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column].Editor ;
  { If the column is read only then the editor will be nil. }
  If FActiveEditor <> Nil Then
    Begin
    FActiveEditor.Parent := Zones [FCurrentCell.Zone] ;
    FEditorVisible := True ;
    Zones [FCurrentCell.Zone].PositionEditor (FActiveEditor, FCurrentCell.Column, FCurrentCell.RecordNumber) ;
    Application.ProcessMessages ;
    FActiveEditor.SetFocus ;
    End ;
  End ;

Procedure TColossalGrid.MoveToNextRecord (AColumn : Cardinal) ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    NewCell := CurrentCell ;
    NewCell.Column := AColumn ;
    ChangeCurrentCell (NewCell) ;
    FDataLink.DataSet.Next ;
    End
  Else
    Begin
    NewCell := FCurrentCell ;
    NewCell.Column := AColumn ;
    Inc (NewCell.RecordNumber) ;
    If BottomVisibleRecord < NewCell.RecordNumber Then
      TopVisibleRecord := TopVisibleRecord + NewCell.RecordNumber - BottomVisibleRecord ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

Procedure TColossalGrid.MoveToPreviousRecord (AColumn : Cardinal) ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    NewCell := CurrentCell ;
    NewCell.Column := AColumn ;
    ChangeCurrentCell (NewCell) ;
    If (FDataLink.DataSet.State = dsInsert)
       And Not FDataLink.DataSet.Modified
       And FDataLink.DataSet.EOF Then
      Begin
      FDataLink.DataSet.Cancel ;
      End
    Else
      Begin
      FDataLink.DataSet.Prior ;
      End ;
    End
  Else
    Begin
    If FCurrentCell.RecordNumber > 1 Then
      Begin
      NewCell := FCurrentCell ;
      NewCell.Column := AColumn ;
      Dec (NewCell.RecordNumber) ;
      If TopVisibleRecord > NewCell.RecordNumber Then
        TopVisibleRecord := NewCell.RecordNumber ;

      ChangeCurrentCell (NewCell) ;
      End ;
    End ;
  End ;

Procedure TColossalGrid.MoveToFirstRecord ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    UpdateDataRecordCount ;
    FDataLink.DataSet.First
    End
  Else
    Begin
    If RecordCount = 0 Then
      Exit ;

    NewCell := FCurrentCell ;
    NewCell.RecordNumber := 1 ;
    TopVisibleRecord := 1 ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

Procedure TColossalGrid.MoveToLastRecord ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    UpdateDataRecordCount ;
    FDataLink.DataSet.Last
    End
  Else
    Begin
    If RecordCount = 0 Then
      Exit ;

    NewCell := FCurrentCell ;
    NewCell.RecordNumber := RecordCount ;
    TopVisibleRecord := RecordCount ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;
{
  This function handles page down movements. It is used to implement
  the response to the Page Down/Next key.
}
Procedure TColossalGrid.MovePageDown ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    UpdateDataRecordCount ;
    FDataLink.DataSet.MoveBy (MaxVisibleRecords-1)
    End
  Else
    Begin
    If RecordCount = 0 Then
      Exit ;

    NewCell := FCurrentCell ;
    If NewCell.RecordNumber = BottomVisibleRecord Then
      Begin
      Inc (NewCell.RecordNumber, MaxVisibleRecords - 1) ;
      If NewCell.RecordNumber > RecordCount Then
        NewCell.RecordNumber := RecordCount ;
      If NewCell.RecordNumber > MaxVisibleRecords Then
        TopVisibleRecord := NewCell.RecordNumber - MaxVisibleRecords + 1 ;
      End
    Else
      Begin
      NewCell.RecordNumber := BottomVisibleRecord ;
      End ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

{
  This function handles page up movements. It is used to implement
  the response to the Page Up key.
}
Procedure TColossalGrid.MovePageUp ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If UsingDatabase Then
    Begin
    UpdateDataRecordCount ;
    If RecordIndex = TopVisibleRecord Then
      FDataLink.DataSet.MoveBy (- Integer (MaxVisibleRecords))
    Else
      FDataLink.DataSet.MoveBy (TopVisibleRecord - RecordIndex) ;
    End
  Else
    Begin
    If RecordCount = 0 Then
      Exit ;

    NewCell := FCurrentCell ;
    If NewCell.RecordNumber = TopVisibleRecord Then
      Begin
      Dec (NewCell.RecordNumber, MaxVisibleRecords - 1) ;
      If NewCell.RecordNumber < 1 Then
        NewCell.RecordNumber := 1 ;
      TopVisibleRecord := NewCell.RecordNumber ;
      End
    Else
      Begin
      NewCell.RecordNumber := TopVisibleRecord ;
      End ;
    ChangeCurrentCell (NewCell) ;
    End ;
  End ;

Procedure TColossalGrid.HideEditor (GridSetFocus : Boolean) ;
  Begin
  If Not FEditorVisible Then
    Exit ;

  { USER EVENT CALL }
  If Assigned (FOnEndEdit) Then
    FOnEndEdit (Self) ;
  { Set the FEditorVisible flag to false *BEFORE* hiding the control.  Otherwise
    we will get a WM_KILLFOCUS event for the editor with the flag still set. }
  FEditorVisible := False ;
  FActiveEditor.Hide ;

  RedrawCell (CurrentCell) ;
  If GridSetFocus Then
    SetFocus ;
  End ;


Procedure TColossalGrid.EditorKeyDown (Editor : TWinControl ; Var Key : Word ; Shift : TShiftState) ;
  Begin
  If Not UpdateTextFromEditor Then
    Begin
    HideEditor (True) ;
    KeyDown (Key, Shift) ;
    End ;
  End ;

Procedure TColossalGrid.EditorKeyPress (Editor : TWinControl ; Var Key : Char) ;
  Begin
  Case Key Of
    #27:  { Escape }
      Begin
      CancelEditing ;
      Key := #0 ;
      End ;
    #9: { Tab }
      Begin
      MoveRight ;
      Key := #0 ;
      End ;
    End ;
  End ;

Procedure TColossalGrid.EditorKillFocus (Message : TWMKillFocus) ;
  Begin
  HideEditor (False) ;
  If Not GridControlsWindow (Message.FocusedWnd) Then
    Begin
    RedrawHeader ;
    End ;
  End ;

{
  Return Value:
    True => The operation was canceled
    False => The operation was completed
}

Function TColossalGrid.UpdateTextFromEditor : Boolean ;
  Begin
  If EditorVisible Then
    Result := Zones [FCurrentCell.Zone].Columns [FCurrentCell.Column].UpdateValue (GetEditorText (FActiveEditor))
  Else
    Result := False ;
  End ;

