{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains TColossalGrid methods that deal with measurements.
}
{
  Function: TColossalGrid.EffectiveFooterHeight

  Description:

    Returns the actual height of the footer in the grid.

  Parameters:

    None

  Return Value:
    0 => The footer is not being displayed
    Other => The height of the footer.

}
Function TColossalGrid.EffectiveFooterHeight : Cardinal ;
  {
    This function returns the height that should be used for
    the grid footer.  This will be the same as FooterHeight
    unless there is no footer in the grid or there is not enough
    room.
  }
  Var
    Space : Integer ;
  Begin

  If HasFooter Then
    Begin
    Space := ZoneClientAreaHeight - EffectiveHeaderHeight ;
    Result := FooterHeight + LineWidth (FFooterSeparatorStyle) ;

    If LongInt (Result) > Space Then
      Result := Space ;
    End
  Else
    Result := 0 ;
  End ;

{
  Function: TColossalGrid.EffectiveHeaderHeight

  Description:

    Returns the actual height of the header in the grid.

  Parameters:

    None

  Return Value:
    0 => The header is not being displayed
    Other => The height of the header.

}
Function TColossalGrid.EffectiveHeaderHeight : Cardinal ;
  { This function returns the height the header will be in a zone.
    This is the value set for HeaderHeight unless the zone is not
    large enough or the grid does not have a header.
  }
  Var
    CltHeight : Cardinal ;
  Begin

  If Self.HasHeader Then
    Begin
    Result := HeaderHeight + LineWidth (FHeaderSeparatorStyle) ;
    CltHeight := ZoneClientAreaHeight ;

    If Result > CltHeight Then
      Begin
      Result := Cltheight ;
      End ;
    End
  Else
    Result := 0 ;
  End ;

{
  Function: TColossalGrid.GridWidth (Virtual)

  Description:

    Returns the width that should be used for the grid.  Classes that inherit
    from this one can make the grid width smaller than the control width if they
    want to paint the extra area.

  Parameters: None

  Return Value:

    The width of the grid.

}

Function TColossalGrid.GridWidth : Integer ;
  Begin
  Result := Self.ClientWidth ;
  End ;

{
  Function: TColossalGrid.GridHeight (Virtual)

  Description:

    Returns the height that should be used for the grid.  Classes that inherit
    from this one can make the grid height smaller than the control width if
    they want to paint the extra area.

  Parameters: None

  Return Value:

    The height of the grid.

}
Function TColossalGrid.GridHeight : Integer ;
  Begin
  Result := Self.ClientHeight ;
  End ;

{
  Function: TColossalGrid.GridTop (Virtual)

  Description:

    Returns the top position for the grid. Classes that inherit from this one
    can move the grid position so that they can manage the area outisde of the
    grid.

  Parameters: None

  Return Value:

    The top position of the grid.

}
Function TColossalGrid.GridTop : Integer ;
  Begin
  Result := 0 ;
  End ;

{
  Function: TColossalGrid.GridLeft (Virtual)

  Description:

    Returns the left position for the grid. Classes that inherit from this one
    can move the grid position so that they can manage the area outisde of the
    grid.

  Parameters: None

  Return Value:

    The left position of the grid.

}
Function TColossalGrid.GridLeft : Integer ;
  Begin
  Result := 0 ;
  End ;

{
  Function: TColossalGrid.GetGridRect (Virtual)

  Description:

    This function returns the Rect where the grid is to be contained on the
    control.  If GridTop, GridLeft, GridHeight, and GridWidth are not
    overridden then this value should be the same as ClientRect.

  Parameters: None

  Return Value:

    The Grid Rect relative to the control.
}
Function TColossalGrid.GetGridRect : Windows.TRect ;
  Begin
  Result.Left := Self.GridLeft ;
  Result.Right := Result.Left + Self.GridWidth ;
  Result.Top := Self.GridTop ;
  Result.Bottom := Result.Top + Self.GridHeight ;
  End ;

{
  Function: TColossalGrid.VirtualWidth

  Description:

    This function determines the amount of horizontal space that all of the
    zones require and is used to determine if a horizontal scroll bar is
    required. This value may be greater than the width of the grid.

  Parameters: None

  Return Value:

    The virtual width of the grid.

}
Function TColossalGrid.VirtualWidth : Cardinal ;
  Var
    IZone : Integer ;
  Begin
  Result := 0 ;
  For IZone := 1 To ZoneCount DO
    Begin
    Inc (Result,  Zones [IZone].ZoneWidth) ;
    End ;
  Inc (Result, (ZoneCount - 1) * LineWidth(FZoneSeparatorStyle)) ;
  End ;

{
  Function: TColossalGrid.EffectiveVerticalScrollbarWidth

  Description:

    This function determines the amount of horizontal space that must be set
    aside for a vertical scroll bar in the grid.

  Parameters: None

  Return Value:

    The effective scroll bar width

}
Function TColossalGrid.EffectiveVerticalScrollbarWidth : Cardinal ;
  Begin
  If VisibleScrollBar (FVScrollBar) Then
    Begin
    Result := FVScrollBar.Width
           + LongInt (LineWidth (FVerticalScrollBarSeparatorStyle))
    End
  Else
    Begin
    Result := 0 ;
    End ;
  End ;

{
  This function determines the amount of horizontal space the must be set aside
  for a vertical scroll bar in the grid.
}
Function TColossalGrid.EffectiveHorizontalScrollbarHeight : Cardinal ;
  Begin
  If VisibleScrollBar (FHScrollBar) Then
    Begin
    Result := FHScrollBar.Height
           + LongInt (LineWidth (FHorizontalScrollBarSeparatorStyle))
    End
  Else
    Begin
    Result := 0 ;
    End
  End ;

{
  This function determines the amount of vertical space that must be set aside
  for horizontal scroll bars within the zones.
}
Function TColossalGrid.EffectiveHorizontalZoneScrollBarHeight : Cardinal ;
  Begin
  If AnyZonesHaveScrollBars Then
    Begin
    Result := GetSystemMetrics (SM_CYHSCROLL)
              + LongInt (LineWidth (FHorizontalScrollBarSeparatorStyle)) ;
    End
  Else
    Begin
    Result := 0 ;
    End ;
  End ;

Function TColossalGrid.RecordAreaTop : Integer ;
  Begin
  Result := LongInt (EffectiveHeaderHeight) + GetGridRect.Top ;
  End ;

{
  Returns the height of the vertically scrolling area of the grid.
}
Function TColossalGrid.RecordAreaHeight : Cardinal ;
  Var
    GridRect : TRect ;
    Value : Integer ;
  Begin
  GridRect := GetGridRect ;

  If RecordHeight + LineWidth (HorizontalLineStyle) > 0 Then
    Begin
    Value := (GridRect.Bottom - GridRect.Top
            - EffectiveHeaderHeight - EffectiveFooterHeight)
            - EffectiveHorizontalScrollbarHeight
            - EffectiveHorizontalZoneScrollBarHeight ;
    Value := Value - Value
            Mod (RecordHeight + LineWidth (HorizontalLineStyle))
            - LongInt (LineWidth (HorizontalLineStyle)) ;
    End
  Else
    Begin
    Value := 0 ;
    End ;
    
  If Value > 0 Then
    Result := Value
  Else
    Result := 0 ;
  End ;

Function TColossalGrid.GridPosition (XPos : Integer ;
                                     YPos : Integer) : TjmGridPosition ;
  Var
    Rect : TRect ;
    Offset : Integer ;
    IZone : Integer ;
  Begin
  Result.Zone := 0 ;
  Result.Column := 0 ;
  Result.RecordNumber := 0 ;

  Result.ColumnLocation := gclNone ;

  Rect := GetGridRect ;
  { See if the point is outside the grid. }
  If (XPos < Rect.Left) Or (XPos >= Rect.Right)
     Or (YPos < Rect.Top) or (YPos >= Rect.Bottom) Then
    Begin
    Result.GridLocation := glOutside ;
    Exit ;
    End ;

  { See if the point is on the grid's horizontal scroll bar. }
  If (YPos >= Rect.Bottom - LongInt (EffectiveHorizontalScrollbarHeight)) Then
    Begin
    Result.GridLocation := glHorizontalScrollBar ;
    Exit ;
    End ;

  { Set if the point is on the grid's vertical scroll bar. }
  If (XPos >= Rect.Right - LongInt (EffectiveVerticalScrollBarWidth)) Then
    Begin
    Result.GridLocation := glVerticalScrollBar ;
    Exit ;
    End ;

  Dec (Rect.Bottom, EffectiveHorizontalScrollBarHeight) ;

  { Determine the zone under the mouse. }
  Offset := - FZoneOffset ;  { Account for scrolling of the zones. }
  For IZone := 1 to ZoneCount Do
    begin
    If XPos < (Offset + Zones [IZone].Width) Then
      Begin
      { If this zone does not have a scroll bar then see if the point is on the
        empty space where the scroll bar would be.
      }
      If (Not Zones [IZone].HasHorizontalScrollBar)
         And (YPos >= Rect.Bottom - EffectiveHorizontalZoneScrollBarHeight) Then
        Begin
        Result.GridLocation := glUnused ;
        Exit ;
        End ;
      Result := Zones [IZone].ZonePosition [XPos - Offset, YPos] ;
      Exit ;
      End ;
    Inc (Offset, Zones [IZone].Width) ;
    If XPos < Offset + LineWidth (ZoneSeparatorStyle) Then
      Begin
      Result.Zone := IZone ;
      Result.GridLocation := glZoneSeparator ;
      Exit ;
      End ;
    Inc (Offset, LineWidth (ZoneSeparatorStyle)) ;
    End ;
  If not (csDesigning in ComponentState) Then
    Raise Exception.Create ('Nowhere') ;
  End ; { TColossalGrid.GridLocation }

Function TColossalGrid.ZoneClientAreaHeight : Cardinal ;
  {  This function return the value of what the zone client
     area height should be. }
  Var
    GridRect : TRect ;
    Value : Integer ;
  Begin
  GridRect := GetGridRect ;
  Value := GridRect.Bottom - GridRect.Top
            - EffectiveHorizontalScrollBarHeight
            - EffectiveHorizontalZoneScrollBarHeight ;
  If Value > 0 Then
    Result := Value
  Else
    Result := 0 ;
  End ;

Function TColossalGrid.FooterYPosition : Cardinal ;
  Var
    Rect : TRect ;
  Begin
  Rect := GetGridRect ;
  Result := Rect.Top + EffectiveHeaderHeight + RecordAreaHeight ;
  End ;

Function TColossalGrid.RecordAreaYPosition : Integer ;
  Var
    Rect : TRect ;
  Begin
  Rect := GetGridRect ;
  Result := Rect.Top + LongInt (EffectiveHeaderHeight) ;
  End ;


