{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains VCL methods that have been overridden by the
  TColossalGrid.
}

Procedure TColossalGrid.CreateParams (var Params : TCreateParams) ;
  Begin
  Inherited CreateParams (Params) ;
  Params.Style := Params.Style OR WS_TABSTOP ;
  If FBorderStyle = bsSingle Then
    Params.Style := Params.Style OR WS_BORDER ;

  { Any time we recreate the window then set the horizontal Scroll
    position to zero }
  FZoneOffset := 0 ;
  End ;

Procedure TColossalGrid.CreateWnd ;
  Begin
  Inherited CreateWnd ;
  FVScrollBar.Parent := Self ;
  FVScrollBar.Kind := sbVertical ;

  FHScrollBar.Parent := Self ;
  FHScrollBar.Kind := sbHorizontal ;

  SetVerticalScrollRange ;
  FHScrollBar.OnScroll := HorizontalScrollHandler ;
  FVScrollBar.OnScroll := VerticalScrollHandler ;

  { If CTL3D is set then alignments get all screwed up. }
  FHScrollBar.Ctl3D := false ;
  FVScrollBar.Ctl3D := false ;

  FDefaultEditor := TjmGridEdit.Create (Nil) ;
  End ;

Procedure TColossalGrid.Paint ;
  Var
    DrawRect : TRect ;
    IZone : Integer ;
    Xpos, YPos : Integer ;

  Begin

  If Not Active Then
    Exit ;

  Inherited Paint ;

  If ZoneCount = 0 Then
    Begin
    AddZone ;
    PositionZones ;
    End ;

  Canvas.Pen.Color := clBlack ;
  Canvas.Pen.Style := psSolid ;
  Canvas.Pen.Width := 1 ;
  Canvas.Pen.Mode := pmCopy ;


  DrawRect := GetGridRect ;

  DrawBackground ;

  { Draw Lines between the zones and the scrollbars. }
  If VisibleScrollBar (FVScrollBar) Then
    Begin
    Xpos := LongInt (DrawRect.Right) - LongInt (FVScrollBar.Width)
            - LongInt (LineWidth (FVerticalScrollBarSeparatorStyle)) ;
    DrawLine (Canvas,
              XPos,
              DrawRect.Top,
              XPos,
              DrawRect.Bottom,
              FVerticalScrollBarSeparatorColor,
              FVerticalScrollBarSeparatorStyle) ;
    End ;

  If VisibleScrollBar (FHScrollBar) Then
    Begin
    Ypos := LongInt (DrawRect.Bottom) - LongInt (FHScrollBar.Height)
            - LongInt (LineWidth (FHorizontalScrollBarSeparatorStyle));
    DrawLine (Canvas,
              DrawRect.Left,
              YPos,
              DrawRect.Right,
              YPos,
              FHorizontalScrollBarSeparatorColor,
              FHorizontalScrollBarSeparatorStyle) ;
    End ;

  { Draw lines between the zones. }
  For IZone := 2 to ZoneCount Do
    Begin
    XPos := LongInt (Zones [IZone].Left)
            - LongInt (LineWidth (FZoneSeparatorStyle)) ;
    If (XPos >= DrawRect.Left) And (XPos < DrawRect.Right) Then
      Begin
      DrawLine (Canvas,
                XPos, DrawRect.Top, XPos,
                DrawRect.Top + DrawRect.Bottom - LongInt (EffectiveHorizontalScrollBarHeight) - 1,
                FZoneSeparatorColor,
                FZoneSeparatorStyle) ;
      End ;
    End ;
  End ;


Procedure TColossalGrid.KeyDown (var Key : Word ; Shift : TShiftState) ;
  Begin
  Inherited KeyDown (Key, Shift) ;
  Case Key Of
    VK_LEFT:
      MoveLeft ;
    VK_RIGHT:
      Begin
      MoveRight ;
      Key := 0 ;
      End ;
    VK_TAB:
      If ssShift In Shift Then
        MoveLeft
      Else
        MoveRight ;
    VK_UP:
      MoveUp ;
    VK_DOWN:
      MoveDown ;
    VK_HOME:
      If ssCtrl In Shift Then
        MoveToFirstRecord ;
    VK_END:
      If ssCtrl In Shift Then
        MoveToLastRecord ;
    VK_PRIOR:
      MovePageUp ;
    VK_NEXT:
      MovePageDown ;
    End ;
  End ;

Procedure TColossalGrid.KeyPress(var Key : Char) ;
  Begin
  Inherited KeyPress (Key) ;
  Case Key Of
    #13:
      Begin
      EditorVisible := True ;
      Key := #0 ;
      End ;
    #32 .. #126, #128 .. #255:
      Begin
      EditorVisible := True ;
      { If PositionEditor Fails then FActiveEditor will be Nil. }
      If (FActiveEditor <> Nil) Then
        SendMessage (FActiveEditor.Handle, WM_CHAR, Ord (Key), 0) ;
      Key := #0 ;
      End ;
    End ;

  End ;


procedure TColossalGrid.MouseDown (Button : TMouseButton ;
                                   Shift : TShiftState ;
                                   X , Y : Integer) ;
  Begin
  Inherited MouseDown (Button, Shift, X, Y) ;
  If CanFocus And Not EditorVisible Then
    SetFocus ;
  End ;

{
  Procedure:  TColossalGrid.GetChildren

  Description:  Determine the controls to be written to the stream.

  Parameters:
      Proc : The procedure to be called for each child control.

  This procedure override the one define in TComponent.  This procedure is used
  by the Delphi classes for storing object to a stream to find all the controls
  that are owned by by the zone.

  If this procedure did not exist the zones would not get written to the
  stream.
}
Procedure TColossalGrid.GetChildren (Proc : TGetChildProc ; Root : TComponent) ;
  Var
    IZone : Integer ;
  Begin
  For IZone := 1 To ZoneCount Do
    Proc (Zones [IZone]) ;
  End ;

{
  Function:  TColossalGridZone.GetChildOwner

  Description:  Specifies the owner for child controls.

  Parameters:
      None

  Return Value:
      The address of the zone.

  At first glance this function appears to have no reason to exist.  However,
  the Delphi classes for reading an object from a stream use
  TComponent.GetChildOwner to determine which object should own child classes
  stored in a stream.  If this function were not overriden then the form would
  own the grid's zones after they were read from the stream.
}

Function TColossalGrid.GetChildOwner : TComponent ;
  Begin
  Result := Self ;
  End ;

Function TColossalGrid.GetChildParent : TComponent ;
  Begin
  Result := Self ;
  End ;

procedure TColossalGrid.Loaded ;
  Begin
  Inherited Loaded ;

  If Usingdatabase Then
    Begin
    UpdateDataRecordCount ;
    DataLayoutChanged ;
    End ;

  { In theory this should never get executed.  However if something is wrong
    with the stream we need to ensure that at least one zone exists. }
  If ZoneCount = 0 Then
    AddZone ;

  { Again in theory we should not need to do this.  However if the rows have
    not been stored and read property we need to ensure that each zone has
    the number of rows expected by the grid. }
  AdjustRowCount ;

  { These functions always need to be done after reading from the stream. }
  PositionZones ;
  PositionColumnsInRows ;

  { These values should be saved correctly but just in case something gets
    fouled up we set them here. }
  // For whatever reason, Tlist.coun is an Integer rather than a cardinal.
  If (FActiveZone < 1) Or (LongInt (FActiveZone) > FZones.Count) Then
    FActiveZone := 1 ;
  If (FVariableWidthZone < 1) Or (LongInt (FVariableWidthZone) > FZones.Count) Then
    FVariableWidthZone := 1 ;

  End ;

{
  Component Notification Procedure

  Called whenever a component as added or removed from a form.

  Parameters:
             AComponent : The component being added or removed
             Operation : opRemove or opInsert

  The DataSource property is used to reference another control.  We need to know
  if the datasource is deleted so we can remove the reference.
}
Procedure TColossalGrid.Notification (AComponent : TComponent; Operation : TOperation) ;
  Begin
  If (Operation = opRemove) And (AComponent = DataSource) Then
    DataSource := Nil ;
  End ;



