{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains TColossalGrid methods for getting and setting properties.

  Sorry for the lack of comments in this file. The functions in this file
  are all pretty trivial.
}
Procedure TColossalGrid.SetDefaultRowCount (ACount : Cardinal) ;
  Begin
  If ACount < 1 Then
    Raise Exception.CreateFmt ('Invalid Row count (%d) - must be greater than zero', [ACount]) ;

  If FDefaultRowCount = ACount Then
    Exit ;

  FDefaultRowCount := ACount ;

  If Not (csReading in ComponentState) Then
    AdjustRowCount ;

  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetZoneCount (ACount : Cardinal) ;
  Var
    Delta : Integer ;
    II : Integer ;
  Begin
  If (csReading In ComponentState) Then
    Exit ;

  If ACount < 1 Then
    Raise Exception.CreateFmt ('Invalid Zone count (%d) - must be greater than zero', [ACount]) ;

  If ACount = ZoneCount Then
    Exit ; { No change }

  Delta := ACount - ZoneCount ;
  If Delta > 0 Then
    Begin { Adding Zones }
    For II := 1 to Delta Do
      Begin
      AddZone ;
      End ;
    End
  Else
    Begin { Removing Zones }
    For II := 1 to - Delta Do
      Begin
      FZones.Delete (FZones.Count) ;
      End ;
    End ;

  If FActiveZone > ACount Then
    FActiveZone := 1 ;

  LayoutChanged ;
  End ; { TColossalGrid.SetZoneCount }

Function TColossalGrid.GetGridColumnCount : Cardinal ;
  Var
    IZone : Integer ;
  Begin
  Result := 0 ;
  For IZone := 1 To ZoneCount Do
    Begin
    Inc (Result, Zones [IZone].ColumnCount) ;
    End ;
  End ;

Procedure TColossalGrid.SetActiveZone (AZone : Cardinal) ;
  Begin
  If (AZone < 1) Or (AZone > ZoneCOunt) And Not (csReading in ComponentState) Then
    Raise Exception.CreateFmt ('Zone index (%d) outsize range 1..%d', [AZone, ZoneCount]) ;

  FActiveZone := AZone ;
  End ;

Function TColossalGrid.GetColumns (Index : Cardinal) : TjmGridColumn ;
  Begin
  Result := Zones [ActiveZone].Columns [Index] ;
  End ;

Function TColossalGrid.GetColumnCount : Cardinal ;
  Begin
  Result := Zones [ActiveZone].ColumnCount ;
  End ;

Procedure TColossalGrid.SetColumnCount (ACount : Cardinal) ;
  Begin
  { Let the zones load themselves. }
  If Not (csReading in ComponentState) Then
    Zones [ActiveZone].ColumnCount := ACount ;
  End ;


Procedure TColossalGrid.SetTopVisibleRecord (ARecord : LongInt) ;
  Var
    IZone : Integer ;
  Begin
  If ((ARecord < 1) Or (ARecord > RecordCount)) And (RecordCount > 0) Then
    raise Exception.CreateFmt ('Requested top visible record (%d) is out of the range 1..%d', [ARecord, RecordCount]) ;

  If FTopVisibleRecord = ARecord Then
    Exit ;

  For IZone := 1 To ZoneCount Do
    Begin
    Zones [IZone].MakeTopRecord (ARecord) ;
    End ;

  FTopVisibleRecord := ARecord ;

  CellsMoved ;

  End ;

Function TColossalGrid.GetTopVisibleRecord : LongInt ;
  Var
    MinTop : LongInt ;
  Begin
  If (RecordCount = 0) Or (FMode = gmDatabase) Then
    Begin
    FTopVisibleRecord := 1 ;
    End
  Else
    Begin
    MinTop := LongInt (RecordCount) - LongInt (MaxVisibleRecords) + 1 ;
    If MinTop < FTopVisibleRecord Then
      Begin
      If MinTop < 1 Then
        FTopVisibleRecord := 1
      Else
        FTopVisibleRecord := MinTop ;
      End ;
    End ;

  result := FTopVisibleRecord ;
  End ;


Function TColossalGrid.GetBottomVisibleRecord : LongInt ;
  Begin
  If FMode = gmDatabase Then
    Begin
    Result := FDataLink.RecordCount ;
    End
  Else If RecordCount = 0 Then
    Begin
    Result := -1 ;
    End
  Else
    Begin
    Result := LongInt (FTopVisibleRecord) + LongInt (MaxVisibleRecords) - 1 ;
    If  Result > RecordCount Then
      Result := RecordCount ;
    End ;
  End ;

Procedure TColossalGrid.SetBorderStyle (Value : TBorderStyle) ;
  Begin
  If FBorderStyle <> Value Then
    Begin
    FBorderStyle := Value ;
    If HandleAllocated Then
      RecreateWnd ;
    End ;
  End ;

Procedure TColossalGrid.SetActive (Value : Boolean) ;
  Begin
  If Value = FActive Then
    Exit ;

  FActive := Value ;
  If FActive Then
    LayoutChanged ;
  End ;

Procedure TColossalGrid.SetRowHeight (AHeight : Cardinal) ;
  Begin
  FRowHeight := AHeight ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetVariableWidthZone (AZone : Cardinal) ;
  Begin
  If (AZone < 1) Or (AZone > ZoneCount) And Not (csReading In ComponentState) Then
    Raise Exception.CreateFmt ('Zone number (%d) out of range 1..%d', [AZone, ZoneCount]) ;


  If AZone = FVariableWidthZone Then
    Exit ;

  FVariableWidthZone := AZone ;
  LayoutChanged ;
  End ;

Function TColossalGrid.GetRows (Index : Cardinal) : TjmGridRow ;
  Begin
  Result := Zones [ActiveZone].Rows [Index] ;
  End ;

Procedure TColossalGrid.SetRowCount (ACount : Cardinal) ;
  Begin
  Zones [ActiveZone].RowCount := ACount ;
  End ;

Function TColossalGrid.GetRowCount : Cardinal ;
  Begin
  Result := Zones [ActiveZone].RowCount ;
  End ;

Procedure TColossalGrid.SetShowHorizontalScrollBar (Mode : TjmGridScrollBars) ;
  Begin
  If Mode = FShowHorizontalScrollBar Then
    Exit ;

  FShowHorizontalScrollBar := Mode ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetShowVerticalScrollBar (Mode : TjmGridScrollBars) ;
  Begin
  If Mode = FShowVerticalScrollBar Then
    Exit ;

  FShowVerticalScrollBar := Mode ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHasFooter (AValue : Boolean) ;
  Begin
  If AValue = FHasFooter Then
    Exit ;

  FHasFooter := AValue ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHasHeader (AValue : Boolean) ;
  Begin
  If AValue = FHasHeader Then
    Exit ;

  FHasHeader := AValue ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterColor (AColor : TColor) ;
  Begin
  If AColor = FFooterColor Then
    Exit ;

  FFooterColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderColor (AColor : TColor) ;
  Begin
  If AColor = FHeaderColor Then
    Exit ;

  FHeaderColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetInactiveColor (AColor : TColor) ;
  Begin
  If ACOlor = FInactiveColor Then
    Exit ;

  FInactiveColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderSeparatorColor (AColor : TColor) ;
  Begin
  If FHeaderSeparatorColor = AColor Then
    Exit ;

  FHeaderSeparatorColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterSeparatorColor (AColor : TColor) ;
  Begin
  If FFooterSeparatorColor = AColor Then
    Exit ;

  FFooterSeparatorColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterSeparatorStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If AStyle = FFooterSeparatorStyle Then
    Exit ;

  FFooterSeparatorStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderSeparatorStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If AStyle = FHeaderSeparatorStyle Then
    Exit ;

  FHeaderSeparatorStyle := AStyle ;
  LayoutChanged ;
  End ;


Procedure TColossalGrid.SetReadOnly (AState : Boolean) ;
  Begin
  If AState = FReadOnly Then
    Exit ;

  FReadOnly := AState ;
  If FReadOnly = False Then
    EditorVisible := false ;
  End ;

Procedure TColossalGrid.SetEditorVisible (Mode : Boolean) ;
  Var
    Zone : TjmGridZone ;
  Begin
  If Mode = FEditorVisible Then
    Exit ;

  If (ColumnIndex = 0)
     Or (ZoneIndex = 0)
     Or (RecordIndex = 0) And Mode Then
    Begin
    Raise Exception.Create ('Cannot show editor when no cell is selected') ;
    End ;

  Zone := Zones [ZoneIndex] ;
  If (Zone.Columns [ColumnIndex].Editor = Nil) And (Mode = True) Then
    Exit ;

  If Mode Then
    Begin
    If ReadOnly Or Zone.ReadOnly Or Zone.Columns [FCurrentCell.Column].ReadOnly
       Or (csDesigning In ComponentState)
       Or (csLoading In ComponentState) Then
      Begin
      Exit ;
      End ;

    { USER EVENT CALL }
    If Assigned (FOnStartEdit) Then
      FOnStartEdit (Self) ;

    If UsingDatabase Then
      FDataLink.Edit ;
    PositionEditor ;
    FEditorVisible := True ;
    End
  Else
    Begin
    HideEditor (True) ; { True => Set focus back to grid. }
    End ;
  End ; { TColossalGrid.SetEditorVisible }

Function TColossalGrid.GetHeaderHeight : Cardinal ;
  Begin
  Result := RecordHeight ;
  End ;

Function TColossalGrid.GetFooterHeight : Cardinal ;
  Begin
  Result := RecordHeight ;
  End ;

Procedure TColossalGrid.SetZoneSeparatorColor (AColor : TColor) ;
  Begin
  If FZoneSeparatorColor = AColor Then
    Exit ;

  FZoneSeparatorColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetZoneSeparatorStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FZoneSeparatorStyle = AStyle Then
    Exit ;

  FZoneSeparatorStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHorizontalLineColor (AColor : TColor) ;
  Begin
  If FHorizontalLineColor = AColor Then
    Exit ;

  FHorizontalLineColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHorizontalLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FHorizontalLineStyle = AStyle Then
    Exit ;

  FHorizontalLineStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetVerticalLineColor (AColor : TColor) ;
  Begin
  If FVerticalLineColor = AColor Then
    Exit ;

  FVerticalLineColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FVerticalLineStyle = AStyle Then
    Exit ;

  FVerticalLineStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetRowLineColor (AColor : TColor) ;
  Begin
  If FRowLineColor = AColor Then
    Exit ;

  FRowLineColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetRowLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FRowLineStyle = AStyle Then
    Exit ;

  FRowLineStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetBackgroundColor (AColor : TColor) ;
  Begin
  If AColor = FBackgroundColor Then
    Exit ;

  FBackgroundColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetSelectColor (AColor : TColor) ;
  Begin
  If AColor = FSelectColor Then
    Exit ;

  FSelectColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetUnusedColor (AColor : TColor) ;
  Begin
  If AColor = FUnusedColor Then
    Exit ;

  FUnusedColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderFont (AFont : TFont) ;
  Begin
  FHeaderFont.Assign (AFont) ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterFont (AFont : TFont) ;
  Begin
  FFooterFont.Assign (AFont) ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetInactiveFont (AFont : TFont) ;
  Begin
  FInactiveFont.Assign (AFont) ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetSelectFont (AFont : TFont) ;
  Begin
  FSelectFont.Assign (AFont) ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetRecordIndex (NewRecord : LongInt) ;
  Begin
  ChangeCurrentRecord (NewRecord) ;
  End ;

Function TColossalGrid.GetMaxVisibleRecords : Cardinal ;
  Begin
  If RecordHeight + LineWidth (HorizontalLineStyle) = 0 Then
    Result := 0
  Else
    Result := (RecordAreaHeight + LineWidth (HorizontalLineStyle))
                Div (RecordHeight + LineWidth (HorizontalLineStyle)) ;
  End ;

Function TColossalGrid.GetZoneCount : Cardinal ;
  Begin
  Result := FZones.Count ;
  End ;

Procedure TColossalGrid.SetCellFrameColor (AColor : TColor) ;
  Begin
  If AColor = FCellFrameColor Then
    Exit ;
  FCellFrameColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If AStyle = FCellFrameStyle Then
    Exit ;
  FCellFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderFrameColor (AColor : TColor) ;
  Begin
  If AColor = FHeaderFrameColor Then
    Exit ;
  FHeaderFrameColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If AStyle = FHeaderFrameStyle Then
    Exit ;
  FHeaderFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterFrameColor (AColor : TColor) ;
  Begin
  If AColor = FFooterFrameColor Then
    Exit ;
  FFooterFrameColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If AStyle = FFooterFrameStyle Then
    Exit ;
  FFooterFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetActiveFrameColor (AColor : TColor) ;
  Begin
  If AColor = FActiveFrameColor Then
    Exit ;
  FActiveFrameColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If AStyle = FActiveFrameStyle Then
    Exit ;
  FActiveFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Function TColossalGrid.CellRect (Zone : Cardinal ; Column : Cardinal ; RecordNumber : LongInt) : TRect ;
  Begin
  If (Zone < 1) Or (Zone > ZoneCount) Then
    Raise Exception.CreateFmt ('Zone index (%d) outside range [1..%d]', [Zone, ZoneCount]) ;

  If (Column < 1) Or (Column > Zones [Zone].ColumnCount) Then
    Raise Exception.CreateFmt ('Column index (%d) outside range [1..%d]', [Column, ColumnCount]) ;

  If (RecordNumber < 1) Or (RecordNumber > RecordCount) Then
    Raise Exception.CreateFmt ('Record number (%d) outside range [1..%d]', [RecordNumber, RecordCount]) ;
  Result := Zones [Zone].CellRect [Column, RecordNumber] ;
  End ;

Procedure TColossalGrid.SetZoneIndex (AZone : Cardinal) ;
  Var
    NewCell : TjmGridCell ;
  Begin
  If (AZone < 1) Or (AZone > ZoneCount) Then
    Raise Exception.CreateFmt ('Zone index (%d) outside range [1..%d]', [AZone, ZoneCount]) ;
  NewCell := CurrentCell ;
  NewCell.Zone := AZone ;
  ChangeCurrentCell (NewCell) ;
  End ;

Procedure TColossalGrid.SetCurrentCell (ACell : TjmGridCell) ;
  Begin
  If (ACell.Zone > ZoneCount) Or (ACell.Zone < 1) Then
    Raise Exception.CreateFmt ('Zone index (%d) outside range [1..%d]', [ACell.Zone, ZoneCount]) ;

  If (ACell.Column > Zones [ACell.Zone].ColumnCount)  Then
    Raise Exception.CreateFmt ('Column index (%d) outside range [1..%d]', [ACell.Column, Zones [ACell.Zone].ColumnCount]) ;

  If (ACell.RecordNumber < 1) Or (ACell.RecordNumber > RecordCount) Then
    Raise Exception.CreateFmt ('Record index (%d) outside range [1..%d]', [ACell.RecordNumber, RecordCount]) ;

  ChangeCurrentCell (ACell) ;
  End ;

Procedure TColossalGrid.SetHorizontalScrollBarSeparatorStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FHorizontalScrollBarSeparatorStyle = AStyle Then
    Exit ;

  FHorizontalScrollBarSeparatorStyle := AStyle ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetHorizontalScrollBarSeparatorColor (AColor : TColor) ;
  Begin
  If FHorizontalScrollBarSeparatorColor = AColor Then
    Exit ;

  FHorizontalScrollBarSeparatorColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TColossalGrid.SetVerticalScrollBarSeparatorStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If FVerticalScrollBarSeparatorStyle = AStyle Then
    Exit ;

  FVerticalScrollBarSeparatorStyle := AStyle ;
  End ;

Procedure TColossalGrid.SetVerticalScrollBarSeparatorColor (AColor : TColor) ;
  Begin
  If FVerticalScrollBarSeparatorColor = AColor Then
    Exit ;

  FVerticalScrollBarSeparatorColor := AColor ;
  LayoutChanged ;
  End ;

Function TColossalGrid.GetZones (Index : Cardinal) : TjmGridZone ;
  Begin
  Result := FZones [Index] ;
  End ;

Procedure TColossalGrid.SetDataZone (AZone : Cardinal) ;
  Begin
  If (AZone > ZoneCount) And Not (csLoading in ComponentState) Then
    Raise Exception.CreateFmt ('Zone (%d) out of range [0..%d]', [AZone, ZoneCount]) ;

  FDataZone := AZone ;
  End ;

Function TColossalGrid.GetVersion : String ;
  Begin
  Result := GridVersion ;
  End ;

Procedure TColossalGrid.SetVersion (AVersion : String) ;
  Begin { NoOp }
  End ;

Procedure TColossalGrid.SetSelectMode (AMode : TjmGridSelectMode) ;
  Begin
  If AMode = FSelectMode Then
    Exit ;

  FSelectMode := AMode ;
  If not (csReading In ComponentState) Then
    Repaint ;
  End ;

Function TColossalGrid.GetDataSource : TDataSource ;
  Begin
  IF FDataLink <> Nil Then
    Result := FDataLink.DataSource
  Else
    Result := Nil ;
  End ;

Procedure TColossalGrid.SetDataSource (ASource : TDataSource) ;
  Begin
  If ASource = FDataLink.DataSource Then
    Exit ;

  FDataLink.DataSource := ASource ;

  If Not (csReading In ComponentState) Then
    Begin
    { Be sure we go to a known state. }
    If FDataLink.Active Then
      FDataLink.DataSet.First ;

    If (Mode = gmDatabase) Then
      Begin
      UpdateDataRecordCount ;
      DataLayoutChanged ;
      End ;
    End ;

  End ;

Procedure TColossalGrid.SetMode (AMode : TjmGridMode) ;
  Begin
  If FMode = AMode Then
    Exit ;
  FMode := AMode ;
  If not (csReading In ComponentState) Then
    Begin
    If UsingDatabase Then
      Begin
      UpdateDataRecordCount ;
      DataLayoutChanged ;
      End
    Else
      Begin
      Repaint ;
      End ;
    End ;
  End ;

Procedure TColossalGrid.SetRecordCount (ACount : LongInt) ;
  Begin
  If FMode = gmDatabase Then
    Begin
    If Not (csReading In ComponentState) Then
      Raise Exception.Create ('Record count cannot be set in database mode') ;
    End ;

  If ACount < 0 Then
    Raise Exception.Create ('Invalid record count') ;

  If ACount = FRecordCount Then
    Exit ;

  ChangeRecordCOunt (ACount) ;

  End ;

Function TColossalGrid.GetRecordCount : LongInt ;
  Begin
  If FMode = gmDatabase Then
    Begin
    Result := FDataLink.GetRecordSetCount ;
    End
  Else
    Begin
    Result := FRecordCount ;
    End ;
  End ;


