{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains the implemenation for TColossalGrid methods that deal
  with scrolling.
}

{
 Function: TColossalGrid.VerticalScrollBarsAreNeeded
 Purpose:  Determines if vertical scroll bars should be displayed.
 Parameters:          None
 Return Value:
        True => The grid should display vertical scroll bars
        False => The grid does not need scroll bars


 Description:
     There are two cases when the grid should display vertical scroll bars:

       1. There are two many rows in the grid to be displayed at once.
       2. The top record is not the first record.
}
Function TColossalGrid.VerticalScrollBarsAreNeeded : Boolean ;
  Var
    RequiredHeight : LongInt ;
  Begin
  FindRecordHeight ;  { Save the record height. }

  Case ShowVerticalScrollBar Of
    sbAlways:
      Result := True ;
    sbNever:
      Result := False ;
    sbWhenNeeded:
      Begin
      RequiredHeight :=
          RecordCount * (RecordHeight + LineWidth (HorizontalLineStyle))
            - LineWidth (HorizontalLineStyle) ;
      If (RequiredHeight > RecordAreaHeight)
        Or ((RecordCount <> 0) And (TopVisibleRecord <> 1)) Then
        Result := True
      Else
        Result := False ;
      End ;
    Else
      Raise Exception.Create ('Internal Error - Invalid Vertical Scroll Property') ;
    End ;
  End ;

Procedure TColossalGrid.SetVerticalScrollRange ;
  Var
    GridRect : TRect ;
    PageSize : Integer ;
    SI : TScrollInfo ;
  Begin
  If Not VisibleScrollBar (FVScrollBar) Then
    Exit ;

  If FVScrollBar.Parent = Nil Then
    Exit ;
    
  GridRect := GetGridRect ;

  PageSize := MaxVisibleRecords ;
  If RecordCount > 0 Then
    Begin
    SI.cbSize := Sizeof (SI) ;
    SI.fMask := SIF_ALL ;
    GetScrollInfo (FVScrollBar.Handle, SB_CTL, SI) ;
    SI.nMin := 1 ;
    SI.nMax := RecordCount + PageSize - 1;
    SI.nPage := PageSize ;
    SetScrollInfo (FVScrollBar.Handle, SB_CTL, SI, False) ;
    End
  Else
    Begin
    FVScrollBar.Min := 0 ;
    FVScrollBar.Max := 0 ;
    End ;
  End ;

Procedure TColossalGrid.HorizontalScrollHandler (Sender : TObject ;
                                                 ScrollCode : TScrollCode ;
                                                 var ScrollPos : Integer) ;
  Var
    SB : TScrollBar ;
    NewPos : Integer ;
    Delta : Integer ;
    ScrollRect : TRect ;
  Begin
  SB := Sender as TScrollBar ;

  NewPos := FZoneOffset ;
  case ScrollCode of
    scLineUp:
      Dec (NewPos, SB.SmallChange) ;
    scLineDown:
      Inc (NewPos, SB.SmallChange) ;
    scPageUp:
      Dec (NewPos, SB.LargeChange) ;
    scPageDown:
      Inc (NewPos, SB.LargeChange) ;
    scPosition:
      NewPos := ScrollPos ;
    scTrack:
      NewPos := ScrollPos ;
    scTop:
      NewPos := SB.Max ;
    scBottom:
      NewPos := 0 ;
    scEndScroll:
                ;
    End ;

  If NewPos > SB.Max - SB.PageSize Then
    NewPos := SB.Max - SB.PageSize ;

  If NewPos < 0 Then
    NewPos := 0 ;

  If NewPos = FZoneOffset Then
     Exit ;

  ScrollRect := GetGridRect ;
  Dec (ScrollRect.Right, EffectiveVerticalScrollBarWidth) ;
  { Don't use effective here since we want to redraw everyting. }
  If VisibleScrollBar (FHScrollBar) Then
     Dec (ScrollRect.Bottom, FHScrollBar.Height) ;

  Delta := FZoneOffset - NewPos ;
  FZoneOffset := NewPos ;
  ScrollWindowEx (Handle,
                  Delta,
                  0,
                  @ScrollRect,
                  @ScrollRect,
                  0,
                  Nil,
                  SW_INVALIDATE OR SW_SCROLLCHILDREN) ; 

  SB.Position := FZoneOffset ;

  If Delta < 0 Then
    Begin
    ScrollRect.Left := ScrollRect.Right + Delta ;
    End ;

  PositionZones ;
  End ;

Procedure TColossalGrid.VerticalScrollHandler (Sender : TObject ;
                                               ScrollCode : TScrollCode ;
                                               var ScrollPos : Integer) ;
  Var
    SB : TScrollBar ;
    NewPos : LongInt ;
  Begin
  SB := Sender as TScrollBar ;

  NewPos := RecordIndex ;
  case ScrollCode of
    scLineUp:
      Dec (NewPos, SB.SmallChange) ;
    scLineDown:
      Inc (NewPos, SB.SmallChange) ;
    scPageUp:
      Dec (NewPos, SB.LargeChange) ;
    scPageDown:
      Inc (NewPos, SB.LargeChange) ;
    scPosition:
      NewPos := ScrollPos ;
    scTrack:
      NewPos := ScrollPos ;
    scTop:
      NewPos := SB.Max ;
    scBottom:
      NewPos := 0 ;
    scEndScroll:
                ;
    end ;

  RecordIndex := NewPos ;
  End ;

Procedure TColossalGrid.DisplayHorizontalScrollBar (AShow : Boolean) ;
  Begin
  { We should never have a Nil scroll bar. In the future we may want to
    allocate a scrollbar here. }
  If FHScrollBar = Nil Then
    Exit ;

  FHScrollBar.Visible := AShow ;
  End ;

Procedure TColossalGrid.DisplayVerticalScrollBar (AShow : Boolean) ;
  Begin
  { We should never have a Nil scroll bar. In the future we may want to
    allocate a scrollbar here. }
  If FVScrollBar = Nil Then
    Exit ;

  FVScrollBar.Visible := AShow ;
  End ;


