{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the implementation for the TjmGridRow class. The
  TjmGridRow class represents individual rows within a record. In the
  TColossalGrid a record can be composed of any number of subrows.
}

{
  Property Get Function:  TjmGridRow.GetColor
  Property: Color

}
Function TjmGridRow.GetBackgroundColor : TColor ;
  Begin
  If (osBackgroundColor In FOverride) Then
    Result := FBackgroundColor
  Else
    Result := FZone.BackgroundColor ;
  End ;

{
  Property Set Procedure:  TjmGridRow.SetColor
  Property: Color
}

Procedure TjmGridRow.SetBackgroundColor (AColor : TColor) ;
  Begin
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osBackgroundColor) ;
  FBackgroundColor := AColor ;
  FZone.LayoutChanged ;
  End ;

{
  Property Get Function:  TjmGridRow.GetColumns
  Property: Columns
}
Function TjmGridRow.GetColumns (Index : Cardinal) : TjmGridColumn ;
  Begin
  IF (Index > ColumnCount) Or (Index < 1) Then
    raise Exception.CreateFmt ('Column index (%d) outsize of range 1..%d', [Index, ColumnCount]) ;

  Result := FZone.Columns [Index + FFirstColumn - 1] ;
  End ;

{
  Property Get Function:  TjmGridRow.GetColumnCount
  Property: ColumnCount
  Descriptions: Returns the number of columns within the row.
}
Function TjmGridRow.GetColumnCount : Cardinal ;
  Begin
  If FFirstColumn = 0 Then
    Begin { If the first column is zero then the row has no columns. }
    Result := 0 ;
    End
  Else
    Begin { Row Has Columns }
    If Self = FZone.Rows [FZone.FRows.Count] Then
      Begin { The last row gets all of the remaining columns. }
      Result := FZone.ColumnCount - FFirstColumn + 1 ;
      End
    Else
      Begin { Not the last row. }
      IF FMaxColumns = 0 Then
        Begin { A ColumnCount of zero means the row gets all remaining columns. }
        Result := FZone.ColumnCount - FFirstColumn + 1 ;
        End
      Else
        Begin
        Result := FFirstColumn + FMaxColumns - 1 ;
        If Result > FZone.ColumnCount Then
          Result := FZone.ColumnCount ;
        Dec (Result, FFirstColumn - 1) ;
        End ;
      End ;
    End ;
  End ;

Procedure TjmGridRow.DrawRecords (RecordNumber : LongInt ;
                                  ACanvas : TCanvas ;
                                  ARect : TRect ;
                                  XOffset : Integer) ;
  Var
    ICol : Integer ;
    BackGroundRect : TRect ;
    DrawRect : TRect ;
    CellWidth : Integer ; { The desired with for the cell (Including lines) }
    Offset : Integer ;
  Begin
  DrawRect := Arect ;

  Offset := ARect.Left - XOffset ; {Account for scrolling }
  For ICol := 1 To ColumnCount Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      DrawRect.Left := Offset ;

      If (ICol = LastNonHiddenColumn) And FZone.ExtendRightColumn Then
        Begin { Special case.  Save the leftmost position of any extended columns. }
        If Offset < FZone.FExtendColumnLeftPosition Then
          Begin
          FZone.FExtendColumnLeftPosition := Offset ;
          End ;
        End ;

      If (ICol = LastNonHiddenColumn) And FZone.ExtendRightColumn
         Then
        Begin { This is the last column and it is being extended to the right. }
        CellWidth := ARect.Right - DrawRect.Left ;
        DrawRect.Right := ARect.Right ;
        End
      Else
        Begin
        CellWidth := Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle) ;
        DrawRect.Right := DrawRect.Left + Columns [ICol].Width ;
        End ;

      { Only attempt to draw if the column is actually visible and it is not
        covered by an editor.  If we attempt to draw over the editor then we
        get flickering. }
      If Not IsIntersectionNull (ACanvas.ClipRect, DrawRect)
         And Not (ParentGrid.IsCurrentCell (ParentZone.ZoneNumber, Columns [ICol].ColumnNumber, RecordNumber) And ParentGrid.EditorVisible) Then
        Begin
        BackgroundRect := DrawRect ;
        Columns [ICol].DrawCellbackground (RecordNumber, ACanvas, BackgroundRect) ;
        Columns [ICol].DrawCell (RecordNumber, ACanvas, DrawRect) ;
        End ;

      { Draw the column separator. }
      If (Not FZone.ExtendRightColumn Or (ICol <> LastNonHiddenColumn))
          And (Offset + Columns [ICol].Width < ARect.Right) Then
        Begin
        DrawLine (ParentZone.Canvas,
                  Offset + Columns [ICol].Width, ARect.Top,
                  Offset + Columns [ICol].Width, ARect.Bottom,
                  Columns [ICol].VerticalLineColor,
                  Columns [ICol].VerticalLineStyle) ;
        End ;

      Inc (Offset, CellWidth) ;
      End ;
    End ;

  If Offset < ARect.Right Then
    Begin
    DrawRect.Left := Offset ;
    DrawRect.Right := ARect.Right ;
    ACanvas.Brush.Color := FZone.UnusedColor ;
    ACanvas.FillRect (DrawRect) ;
    End ;

  End ; { TjmGridRow.DrawForeground }

Procedure TjmGridRow.DrawVerticalLines (ACanvas : TCanvas ;
                                           ARect : TRect ;
                                           XOffset : Integer) ;
  Var
    ICol : Integer ;
    Offset : Integer ;
  Begin
  Offset := Arect.Left - XOffset ;
  For ICol := 1 To ColumnCount Do
    Begin
    { Ignore hidden columns. }
    If Not Columns [ICol].Hidden Then
      Begin
      If (ICol = LastNonHiddenColumn) And FZone.ExtendRightColumn Then
        Begin
        Offset := ARect.Right ;
        End
      Else
        Begin
        Inc (Offset, Columns [ICol].Width) ;
        End ;

      { Skip if we are not visible. }
      If (Offset <= ARect.Right) And (Offset >= ARect.Left) Then
        Begin
        DrawLine (ParentZone.Canvas,
                  Offset, ARect.Top,
                  Offset, ARect.Bottom,
                  Columns [ICol].VerticalLineColor,
                  Columns [ICol].VerticalLineStyle) ;
        End ;
      Inc (Offset, LineWidth (Columns [ICol].VerticalLineStyle)) ;
      End ;
    End ;
  End ;

Procedure TjmGridRow.DrawHeader (ACanvas : TCanvas ;
                                 ARect : TRect ;
                                 XOffset : Integer) ;
  Var
    ICol : Integer ;
    DrawRect : TRect ;
    BackgroundRect : TRect ;
    CellWidth : Integer ; { The desired with for the cell (Including lines) }
    Offset : Integer ;
  Begin
  DrawRect := Arect ;
  Offset := ARect.Left - XOffset ;
  For ICol := 1 To ColumnCount Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      DrawRect.Left := Offset ;
      If (ICol = LastNonHiddenColumn)
         And FZone.ExtendRightColumn Then
        Begin { This is the last column and it is being extended to the right. }
        CellWidth := ARect.Right - DrawRect.Left ;
        DrawRect.Right := ARect.Right ;
        End
      Else
        Begin
        CellWidth := Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle) ;
        DrawRect.Right := DrawRect.Left + Columns [ICol].Width ;
        End ;
      { Only attempt to draw if the column is actually visible. }
      If (DrawRect.Left <= ARect.Right) And (DrawRect.Right >= ARect.Left) Then
        Begin

        ACanvas.Brush.Style := bsSolid ;
        If (ParentGrid.HasControl)
            Or (csDesigning in ParentGrid.ComponentState) Then
          Begin
          ACanvas.Brush.Color := Columns [ICol].HeaderColor ;
          ACanvas.Font.Assign (Columns [ICol].EffectiveHeaderFont) ;
          End
        Else
          Begin
          ACanvas.Brush.Color := Columns [ICol].InactiveColor ;
          ACanvas.Font.Assign (Columns [ICol].EffectiveInactiveFont) ;
          End ;


        BackgroundRect := DrawRect ;
        Columns [ICol].DrawHeaderBackground (ACanvas, BackgroundRect) ;
        Columns [ICol].DrawHeader (ACanvas, DrawRect) ;

        { Draw the line between the columns. }
        If (Not FZone.ExtendRightColumn Or (ICol <> LastNonHiddenColumn))
            And (Offset + Columns [ICol].Width < ARect.Right) Then
          Begin
          DrawLine (ParentZone.Canvas,
                    Offset + LongInt (Columns [ICol].Width), ARect.Top,
                    Offset + LongInt (Columns [ICol].Width), ARect.Bottom,
                    Columns [ICol].VerticalLineColor,
                    Columns [ICol].VerticalLineStyle) ;
          End ;

        End ;
      Inc (Offset, CellWidth) ;
      End ;
    End ;

  { Draw the unused area of the header. }
  If Offset < ARect.Right Then
    Begin
    DrawRect.Left := Offset ;
    DrawRect.Right := ARect.Right ;
    ACanvas.Brush.Style := bsSolid ;
    If (ParentGrid.HasControl) Or (csDesigning In ParentGrid.ComponentState) Then
      ACanvas.Brush.Color := FZone.HeaderColor
    Else
      ACanvas.Brush.Color := FZone.InactiveColor ;
    ACanvas.FillRect (DrawRect) ;
    End ;

  End ; { DrawHeader }


Procedure TjmGridRow.DrawFooter (ACanvas : TCanvas ;
                                    ARect : TRect ;
                                    XOffset : Integer) ;
  Var
    ICol : Integer ;
    DrawRect : TRect ;
    CellWidth : Integer ; { The desired with for the cell (Including lines) }
    Offset : Integer ;
  Begin
  DrawRect := Arect ;
  Offset := Arect.Left - FZone.GetScrollPosition ;
  For ICol := 1 To ColumnCount Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      DrawRect.Left := Offset ;
      If (ICol = LastNonHiddenColumn)
         And FZone.ExtendRightColumn Then
        Begin { This is the last column and it is being extended to the right. }
        CellWidth := ARect.Right - DrawRect.Left ;
        DrawRect.Right := ARect.Right ;
        End
      Else
        Begin
        CellWidth := Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle) ;
        DrawRect.Right := DrawRect.Left + Columns [ICol].Width ;
        End ;
      { Only attempt to draw if the column is actually visible. }
      If (DrawRect.Left <= ARect.Right) And (DrawRect.Right >= ARect.Left) Then
        Begin
        ACanvas.Brush.Style := bsSolid ;
        ACanvas.Brush.Color := Columns [ICol].FooterColor ;
        Columns [ICol].DrawFooterBackground (ACanvas, DrawRect) ;

        ACanvas.Font.Assign (Columns [ICol].EffectiveFooterFont) ;
        Columns [ICol].DrawFooter (ACanvas, DrawRect) ;
        End ;
      Inc (Offset, CellWidth) ;
      End ;
    End ;

  { Draw the unused area of the footer. }
  If Offset < ARect.Right Then
    Begin
    DrawRect.Left := Offset ;
    DrawRect.Right := ARect.Right ;
    ACanvas.Brush.Color := FZone.FooterColor ;
    ACanvas.FillRect (DrawRect) ;
    End ;

  End ; { DrawFooter }

{
  This function returns the index of the last column that is not hidden.
}
Function TjmGridRow.LastNonHiddenColumn : Cardinal ;
  Var
    ICol : Integer ;
  Begin
  For ICol := ColumnCount Downto 1 Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      Result := ICol ;
      Exit ;
      End ;
    End ;
    { All columns are hidden so return a very small value. }
    Result := 0 ;
  End ;

Function TjmGridRow.RowWidth : Cardinal ;
  Var
    ICol : Integer ;
  Begin
  Result := 0 ;
  For ICol := 1 To ColumnCount Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      Inc (Result, Columns [ICol].Width) ;
      { The last column in the widest row does not need a line. }
      If ICol <> ColumnCount Then
        Begin
        Inc (Result, LineWidth (Columns [ICol].VerticalLineStyle)) ;
        End ;
      End ;
    End ;
  End ;


{
  Function: TjmGridRow.ColumnAtOffset

  Description:
    This function returns number of the column at the specified offset from
    the left of the row.

  Parameters:
    AOffset : The offset from the left of the row.

  Return Value:
    0 => No column is at the offset
    > 0 => The column number

}
Function TjmGridRow.ColumnAtOffset (AOffset : Cardinal) : Cardinal ;
  Var
    ICol : Cardinal ;
    Offset : Integer ;
  Begin
  Offset := 0 ;
  For ICol := 1 To ColumnCount Do
    Begin
    { Ignore hidden columns. }
    If Not Columns [ICol].Hidden Then
      Begin
      { If the right column is extended and this is the last column then
        we have to measure to the end of the zone. }
      If FZone.ExtendRightColumn
         And (ICol = LastNonHiddenColumn)
         And (AOffset < (FZone.ClientRect.Right - FZone.ClientRect.Left)) Then
        Begin
        Result := ICol ;
        Exit ;
        End
      Else If (Offset + Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle)) > AOffset Then
        Begin
        Result := ICol ;
        Exit ;
        End ;
      Inc (Offset, Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle)) ;
      End ;
    End ;

  Result := 0 ;
  End ;

{
  Function: TjmGridRow.GetColumnOffset

  Description:
    This function returns the offset from the left of the specifed column in
    a row.

  Parameters:
    ACol : The column number whose offset is to be found.

  Return Value:
    The column offset.
}

Function TjmGridRow.GetColumnOffset (ACol : Cardinal) : Cardinal ;
  Var
    ICol : Integer ;
  Begin
  If (ACol > ColumnCount) Or (Acol < 1) Then
    Raise Exception.CreateFmt ('Column (%d) out of range 1..%d', [ACol, ColumnCount]) ;

  If Columns [ACol].Hidden Then
    Begin
    Result := 0 ;
    Exit ;
    End ;

  Result := 0 ;
  For ICol := 1 To ACol - 1  Do
    Begin
    If Not Columns [ICol].Hidden Then
      Begin
      Inc (Result, Columns [ICol].Width + LineWidth (Columns [ICol].VerticalLineStyle)) ;
      End ;
    End ;
  End ;

Procedure TjmGridRow.SetMaxColumns (AMax : Cardinal) ;
  Begin
  If AMax = FMaxColumns Then
    Exit ;

  FMaxColumns := AMax ;
  FZone.LayoutChanged ;
  End ;

Procedure TjmGridRow.SetOverride (AValue : TjmGridOverrideSettings) ;
  Begin
  If AValue = FOverride Then
    Exit ;

  FOverride := AValue ;
  ParentGrid.LayoutChanged ;
  End ;

Function TjmGridRow.StoreAppearanceProperties : Boolean ;
  Begin
  If OverrideSettings = [] Then
    Result := False
  Else
    Result := True ;
  End ;

Procedure TjmGridRow.SetFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FFont.Free ;
    FFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FFOnt = Nil Then
      Begin
      FFOnt := TFOnt.Create ;
      FFOnt.OnChange := FontChanged ;
      End ;
    FFont.Assign (AFont) ;
    End ;
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridRow.EffectiveFont : TFont ;
  Begin
  If FFont <> Nil Then
    Result := FFont
  Else
    Result := ParentZone.EffectiveFont ;
  End ;

Procedure TjmGridRow.FontChanged (Sender : TObject) ;
  Begin
  ParentZone.LayoutChanged ;
  End ;

Function TjmGridRow.GetRowHeight : Cardinal ;
  Begin
  If osRowHeight In FOverride Then
    Result := FRowHeight
  Else
    Result := ParentZone.RowHeight ;
  End ;

Procedure TjmGridRow.SetRowHeight (AValue : Cardinal) ;
  Begin
  If (AValue = FRowHeight) And (osRowHeight in FOverride) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowHeight) ;
  FRowHeight := AValue ;
  ParentGrid.LayoutChanged ;  { Affects grid and zone. }
  End ;

Function TjmGridRow.GetRowLineStyle : TjmGridLineStyle ;
  Begin
  If osRowLineStyle In FOverride THen
    Result := FRowLineStyle
  Else
    Result := ParentZone.RowLineStyle ;
  End ;

Procedure TjmGridRow.SetRowLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If (osRowLineStyle In FOverride) And (AStyle = FRowLineStyle) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowLineStyle) ;
  FRowLineStyle := AStyle ;
  ParentGrid.LayoutChanged ; { Affects grid and zone. }
  End ;

Function TjmGridRow.GetRowLineColor : TColor ;
  Begin
  if osRowLineColor in FOverride Then
    Result := FBackgroundColor
  Else
    Result := ParentZone.RowLineColor ;
  End ;

Procedure TjmGridRow.SetRowLineColor (AColor : TColor) ;
  Begin
  If (osRowLineColor in FOverride) And (AColor = FRowLineColor) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowLineColor) ;
  FRowLineColor := AColor ;

  ParentZone.LayoutChanged ; { Only affects zone. }
  End ;

Function TjmGridRow.GetParentGrid : TColossalGrid ;
  Begin
  Result := FZone.ParentGrid ;
  End ;

