{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains the implementation of TjmGridZone methods that we have
  cagorized as "actions".
}


{
  Procedure: TjmGridZone.AdjustRowCount

  This procedure is called by the cris whenever the number of rows
  changes.

  Parameters:
          None

}
Procedure TjmGridZone.AdjustRowCount ;
  Var
    II : Integer ;
  Begin
  If osRowCount in FOverride Then
    Exit ;

  If FGrid.DefaultRowCount > FRows.Count Then
    Begin
    For II := FRows.Count + 1 To FGrid.DefaultRowCount Do
      TjmGridRow.Create (Self) ;
    End
  Else If FGrid.DefaultRowCount < FRows.Count Then
    Begin
    While FGrid.DefaultRowCount < FRows.Count Do
      begin
      FRows.Delete (FRows.Count) ;
      End ;
    End ;
  PositionColumnsInrows ;
  End ;
  
{
  Procedure: TjmGridZone.Resize

  Procedure to handle zone resizing.

  Parameters:
          None

  All actions that need to be performed after a zone is resized should be
  placed here.

}
Procedure TjmGridZone.ResizeZone ;
  Begin
  SetScrollPosition ;
  SetScrollBarRange ;
  End ;

{
  Procedure: TjmGridZone.RepaintCell

    Procedure to invalidate a single cell.

  Parameters:
          ACol : Cell Column
          ARecord : Cell Record Number

}
Procedure TjmGridZone.RedrawCell (ACol : Cardinal ; ARecord : LongInt) ;
  Var
    Rect : TRect ;
  Begin
  If (ACol < 1) Or (ACol > ColumnCount) Then
    Raise Exception.CreateFmt ('Column number (%d) outside of range 1..%d', [ACol, ColumnCount]) ;

  If (ARecord < 1) Or (ARecord > FGrid.RecordCount) Then
    Raise Exception.CreateFmt ('Record number (%d) outside of range 1..%d', [ARecord, FGrid.RecordCount]) ;

  Rect := GetCellRect (ACol, ARecord) ;
  If Rect.Left <> Rect.Right Then
    RedrawRect (Rect) ;
  End ;

{
  Function: TjmGridZone.MoveLeft

    Procedure to response to request to move left (e.g. left arrow)

  Parameters:
          OldCol : The old current column
          NewCol : The new current column

  Return Value:
          True => The current column show be changed to NewCol
          False => The zone does not know how to change the column.  The
                   grid show decide.

  This procedure is called by the grid to ask the zone how to change the
  current column.

}
Function TjmGridZone.MoveLeft (OldCol : Cardinal ; var NewCol : Cardinal) : Boolean ;
  Begin
  If OldCol > 1 Then
    Begin
    NewCol := OldCol - 1 ;
    MakeColumnVisible (NewCol) ;
    Result := True ;
    End
  Else
    Begin { We are already at the left most column. }
    Result := False ;
    End ;
  End ;

{
  Function: TjmGridZone.MoveRight

    Procedure to response to request to move right (e.g. right arrow)

  Parameters:
          OldCol : The old current column
          NewCol : The new current column

  Return Value:
          True => The current column show be changed to NewCol
          False => The zone does not know how to change the column.  The
                   grid show decide.

  This procedure is called by the grid to ask the zone how to change the
  current column.
}
Function TjmGridZone.MoveRight (OldCol : Cardinal ; var NewCol : Cardinal) : Boolean ;
  Begin
  If OldCol < ColumnCount Then
    Begin
    NewCol := OldCol + 1 ;
    MakeColumnVisible (NewCol) ;
    Result := True ;
    End
  Else
    Begin
    Result := False ;
    End ;
  End ;

{
  Function: TjmGridZone.MoveUp

  This function is called to move up one cell.

  Parameters:
          OldCol : The current column
          OldRecord : The Current Record
          NewCol : The column to move to
  Return Value:
          True => Move to the previous record
          False => Remain on the current record

  This procedure is called by the grid to ask the zone how to change the
  current column in response to a move up request.  For move up requests the
  zone will always determine the column to move to.  The return value tells the
  grid if the record needs to change.
}
Function TjmGridZone.MoveUp (OldCol : Cardinal ;
                                   OldRecord : LongInt ;
                                   var NewCol : Cardinal) : Boolean ;
  Var
    IRow : Integer ;
    Position : Cardinal ;
    Row : Cardinal ;
    Offset : Integer ;
  Begin
  If OldCol = 0 Then
    Begin
    { No Column Selected }
    NewCol := 0 ;
    Result := False ;
    Exit ;
    End ;

  ColumnToRow (OldCol, Row, Position) ;

  Offset := Rows [Row].GetColumnOffset (Position) ;

  { See if there is a row in this record we can move to. }
  For IRow := Row - 1 DownTo 1 Do
    Begin
    If Rows [IRow].ColumnCount <> 0 Then
      Begin
      Position := Rows [IRow].ColumnAtOffset (Offset) ;
      NewCol := RowPositionToColumn (IRow, Position) ;
      If NewCol < 1 Then
        NewCol := Rows [IRow].ColumnCount ;
      Result := False ;
      Exit ;
      End ;
    End ;

  { If we are on the first record then there is nowhere to go. }
  If OldRecord = 1 Then
    Begin
    NewCol := OldCol ;
    Result := True ;
    Exit ;
    End ;

  { Try the previous rows. }
  For IRow := FRows.Count DownTo Row Do
    Begin
    If Rows [IRow].ColumnCount <> 0 Then
      Begin
      Position := Rows [IRow].ColumnAtOffset (Offset) ;
      NewCol := RowPositionToColumn (IRow, Position) ;
      If NewCol < 1 Then
        NewCol := Rows [IRow].ColumnCount ;
      MakeColumnVisible (NewCol) ;
      Result := True ;
      Exit ;
      End ;
    End ;

  { If we get here then something is screwed up. }
  Raise Exception.Create ('No columns in any Rows?') ;

  End ;

{
  Function: TjmGridZone.MoveDown

  This function is called to move down one cell.

  Parameters:
          OldCol : The current column
          OldRecord : The Current Record
          NewCol : The column to move to
  Return Value:
          True => Move to the next record
          False => Remain on the current record

  This procedure is called by the grid to ask the zone how to change the
  current column in response to a move down request.  For move up requests the
  zone will always determine the column to move to.  The return value tells the
  grid if the record needs to change.
}
Function TjmGridZone.MoveDown (OldCol : Cardinal ;
                               OldRecord : LongInt ;
                               var NewCol : Cardinal) : Boolean ;
  Var
    IRow : Integer ;
    Position : Cardinal ;
    Offset : Integer ;
    Row : Cardinal ;
  Begin
  If OldCol = 0 Then
    Begin
    { No Column Selected }
    NewCol := 0 ;
    Result := False ;
    Exit ;
    End ;
    
  ColumnToRow (OldCol, Row, Position) ;

  Offset := Rows [Row].GetColumnOffset (Position) ;

  { See if there is a row in this record we can move to. }
  For IRow := Row + 1 TO FRows.Count Do
    Begin
    If Rows [IRow].ColumnCount <> 0 Then
      Begin
      Position := Rows [IRow].ColumnAtOffset (Offset) ;
      NewCol := RowPositionToColumn (IRow, Position) ;
      MakeColumnVisible (NewCol) ;
      Result := False ;
      Exit ;
      End ;
    End ;

  { If we are on the last record then there is nowhere to go. }
  If OldRecord = FGrid.RecordCount Then
    Begin
    NewCol := OldCol ;
    Result := True ;
    Exit ;
    End ;

  { Try the next row. }
  For IRow := 1 To Row Do
    Begin
    If Rows [IRow].ColumnCount <> 0 Then
      Begin
      Position := Rows [IRow].ColumnAtOffset (Offset) ;
      NewCol := RowPositionToColumn (IRow, Position) ;
      MakeColumnVisible (NewCol) ;
      Result := True ;
      Exit ;
      End ;
    End ;

  { If we get here then something is screwed up. }
  Raise Exception.Create ('Grid Internal Error - No columns in any Rows?') ;

  End ;

{
  Procedure: TjmGridZone.LayoutChanged

  This procedure is called whenever the layout (colors, fonts, columns widths)
  of the zone is changed.

  Parameters:
          None

}
Procedure TjmGridZone.LayoutChanged ;
  Begin
  If csLoading in ComponentState Then
    Exit ;

  If FGrid.Active Then
    Begin
    PositionColumnsInrows ;
    Repaint ;
    End ;
  End ;

{
  Procedure: TjmGridZone.PositionColumnsInRows

    This procedure assigns columns to rows.

  Parameters:
          None

}
Procedure TjmGridZone.PositionColumnsInRows ;
  Var
    Offset : Cardinal ; { The column being assigned. }
    IRow : Integer ;
  Begin
  Offset := 1 ;
  For IRow := 1 To FRows.Count Do
    Begin
    { See if there are any columns remaining. }
    If Offset <= ColumnCount Then
      Begin
      Rows [IRow].FFirstColumn := Offset ;

      { If FMaxColumns is 0 then the row gets all available columns. }
      If Rows [IRow].FMaxColumns = 0 Then
        Begin
        Offset := High (Integer) ;
        End
      Else
        Begin
        Inc (Offset, Rows [IRow].FMaxColumns) ;
        End ;
      End
    Else
      Begin { This row has no columns. They are all in other rows. }
      Rows [IRow].FFirstColumn := 0 ;
      End ;
    End ;
  End ; { TjmGridZone.PositionColumnsInRows }


Procedure TjmGridZone.RedrawHeaderColumn (ACol : Cardinal) ;
  var
    Rect : TRect ;
  Begin
  Rect := GetHeaderColumnRect (ACol) ;
  RedrawRect (Rect) ;
  End ;

Procedure TjmGridZone.RedrawFooterColumn (ACol : Cardinal) ;
  var
    Rect : TRect ;
  Begin
  Rect := GetFooterColumnRect (ACol) ;
  RedrawRect (Rect) ;
  End ;

Procedure TjmGridZone.FontChanged (Sender : TObject) ;
  Begin
  LayoutChanged ;
  End ;

Procedure TjmGridZone.MoveColumn (OldPosition : Cardinal ;
                                        NewPosition : Cardinal) ;
  Var
    ICol : Integer ;
  Begin
  If (OldPosition < 1) Or (OldPosition > ColumnCount) Then
    Raise Exception.CreateFmt ('Column (%d) outside range 1..%d', [OldPosition, ColumnCount]) ;

  If (OldPosition = NewPosition) Or ((OldPosition = ColumnCount) And (NewPosition > ColumnCount)) Then
    Exit ;

  FColumns.Move (OldPosition, NewPosition) ;

  For ICol := 1 To ColumnCount Do
    Columns [ICol].FColumnNumber := ICol ;

  PositionColumnsInRows ;
  Repaint ;
  End ;


Procedure TjmGridZone.RedrawRecordArea ;
  Var
    Rect : TRect ;
  Begin
  Rect := GetRecordRect ;
  RedrawRect (Rect) ;
  End ;

{
  Procedure:  TjmGridZone.GetChildren

  Description:  Determine the controls to be written to the stream.

  Parameters:
      Proc : The procedure to be called for each child control.

  This procedure override the one define in TComponent.  This procedure is used
  by the Delphi classes for storing object to a stream to find all the controls
  that are owned by by the zone.

  If this procedure did not exist the rows and columns would not get written
  to the stream.
}
Procedure TjmGridZone.GetChildren (Proc : TGetChildProc ; Root : TComponent) ;
  Var
    IRow : Integer ;
    ICol : Integer ;
  Begin

  For IRow := 1 To FRows.Count Do
    Proc (Rows [IRow]) ;

  For ICol := 1 To ColumnCount Do
    Proc (Columns [ICol]) ;

  End ;

{
  Function:  TjmGridZone.GetChildOwner

  Description:  Specifies the owner for child controls.

  Parameters:
      None

  Return Value:
      The address of the zone.

  At first glance this function appears to have no reason to exist.  However,
  the Delphi classes for reading an object from a stream use
  TComponent.GetChildOwner to determine which object should own child classes
  stored in a stream.  If this function were not overriden then the form would
  own the zone's columns and row after they were read from the stream.
}
Function TjmGridZone.GetChildOwner : TComponent ;
  Begin
  Result := Self ;
  End ;

{
  Procedure:  TjmGridZone.AddRow

  Description:  Adds a row to the zone.

  Parameters:
      ARow : The row to add

  This procedure is caleld in the row contructor to tell the zone that
  a row has been added.
}
Procedure TjmGridZone.AddRow (ARow : TjmGridRow) ;
  Begin
  FRows.Add (ARow) ;
  End ;

{
  Procedure:  TjmGridZone.AddColumn

  Description:  Adds a column to the zone.

  Parameters:
      AColumn : The column to add

  This procedure is caleld in the row contructor to tell the zone that
  a column has been added.
}
Procedure TjmGridZone.AddColumn (AColumn : TjmGridColumn) ;
  Begin
  FColumns.Add (AColumn) ;
  AColumn.ColumnNumber := FColumns.Count ;
  End ;

{
  Function: TjmGridZone.StoreAppearanceProperties

  Description:

    This function is used in many of the zone's property definitions the control
    the zone's appearance.  If the user does not override any of the zone's
    appreance properties then we save space by not storing them to the stream.
    Doing all or nothing is a compromise but the alternative is to have a
    separate function for each property.

  Parameters: None

  Return Value:
    True => Store the property value in the stream
    False => Do not storethe property value in the stream.
}

Function TjmGridZone.StoreAppearanceProperties : Boolean ;
  Begin
  If OverrideSettings = [] Then
    Result := False
  Else
    Result := True ;
  End ;

Procedure TjmGridZone.PositionEditor (Editor : TWinControl ; Column : Cardinal ; RecordNumber : LongInt) ;
  Var
    Rect : TRect ;
  Begin
  RedrawCell (Column, RecordNumber) ;
  Rect := GetCellRectUnclipped (Column, RecordNumber) ;
  Editor.Parent := Self ;
  Editor.SetBounds (Rect.Left, Rect.Top, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top) ;
  SetEditorText (Editor, Columns [Column].GetCellEditText (RecordNumber)) ;
  If (Editor Is TEdit) Then
    (Editor As TEdit).Modified := False ;
  Editor.Visible := True ;
  End ;


Procedure TjmGridZone.FindFields ;
  Var
    ICol : Cardinal ;
  Begin
  For ICol := 1 To ColumnCount Do
    Columns [ICol].FindField ;
  End ;

