{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains conversion methods for the TjmGridZone class.
}

{
  Procedure:  TjmGridZone.ZonePosition

  Description:

    Translates an X/Y position to a location on the zone.

}
Function TjmGridZone.GetZonePosition (XPos : Integer ;
                                          YPos : Integer) : TjmGridPosition ;
  Var
    Rect : TRect ;

  Procedure FindColumnLocation (XOffset : Cardinal ;
                                YOffset : Cardinal ;
                                Var Column : Cardinal ;
                                Var Location : TjmGridColumnLocation) ;
    Var
      Row : Integer ;
      Offset : Integer ;
      RowColumn : Integer ;
    Begin
    { Remember rows start at 1. }
    Row := RowAtOffset (YOffset) ;
    RowColumn := Rows [Row].ColumnAtOffset (XOffset) ;
    If RowColumn = 0 Then
      Begin
      Result.ColumnLocation := gclNone ;
      Exit ;
      End ;

    { We know the column so now see if we are on the line. }
    Offset := Rows [Row].GetColumnOffset (RowColumn) ;
    If (XOffset < (Offset + Rows [Row].Columns [RowColumn].Width))
      OR ((RowColumn = Rows [Row].LastNonHiddenColumn) And ExtendRightColumn) Then
      Result.ColumnLocation := gclBody
    Else
      Result.ColumnLocation := gclSeparator ;

    Result.Column := RowPositionToColumn (Row, RowColumn) ;
    End ; { TjmGridZone.ZoneLocation.FindColumnLocation }

  Begin { TjmGridZone.ZonePosition }
  Rect := ClientRect ;

  Result.Zone := ZoneNumber ;
  Result.Column := 0 ;
  Result.RecordNumber := 0 ;

  Result.ColumnLocation := gclNone ;


  { See if the point is on the scrollbar. }
  If (YPos >= Rect.Bottom) And (YPos < Height) Then
    Begin
    Result.GridLocation := glHorizontalScrollBar ;
    Exit ;
    End ;

  { See if the point is outside the zone. }
  If (XPos < Rect.Left) Or (XPos >= Rect.Right)
     Or (YPos < Rect.Top) or (YPos >= Rect.Bottom) Then
    Begin
    Result.GridLocation := glOutside ;
    Exit ;
    End ;

  If YPos < ParentGrid.RecordAreaYPosition Then
    Begin { Within the header }
    If YPos >= ParentGrid.RecordAreaYPosition - LineWidth (ParentGrid.HeaderSeparatorStyle) Then
      Begin
      Result.GridLocation := glHeaderSeparator ;
      End
    Else
      Begin
      Result.GridLocation := glHeader ;
      FindColumnLocation (XPos + FLeftOffset,
                          YPos,
                          Result.Column,
                          Result.ColumnLocation) ;
      End ;
    End
  Else If YPos < ParentGrid.FooterYPosition Then
    Begin { Within Record Area }
    Result.RecordNumber := RecordAtYPosition (YPos) ;
    If Result.RecordNumber = 0 Then
      Begin
      Result.GridLocation := glUnused ;
      Exit ;
      End ;

    If (ParentGrid.RecordCount <> 0) and (Result.RecordNumber < 1) Then
      Raise Exception.CreateFmt ('Record Number (%d) Out of range 1..%d', [Result.RecordNumber, ParentGrid.RecordCount]) ;

    { See if the point is on the record separator }
    If ((YPos - ParentGrid.RecordAreaYPosition) Mod (ParentGrid.RecordHeight + LineWidth (ParentGrid.HorizontalLineStyle)))
        >= ParentGrid.RecordHeight Then
      Begin
      Result.GridLocation := glRecordSeparator ;
      Exit ;
      End ;

    FindColumnLocation (XPos - Rect.Left + FLeftOffset,
                        (YPos - ParentGrid.RecordAreaYPosition) Mod (ParentGrid.RecordHeight + LineWidth (Parentgrid.HorizontalLineStyle)),

                         Result.Column,
                         Result.ColumnLocation) ;
    If Result.Column = 0 Then
      Result.GridLocation := glUnused
    Else
      Result.GridLocation := glCell ;

    End
  Else If (YPos < ParentGrid.FooterYPosition + ParentGrid.EffectiveFooterHeight) Then
    Begin { Within Footer }
    If YPos < ParentGrid.FooterYPosition + LineWidth (ParentGrid.FooterSeparatorStyle) Then
      Begin
      Result.GridLocation := glFooterSeparator ;
      End
    Else
      Begin
      Result.GridLocation := glFooter ;
      FindColumnLocation (XPos - Rect.Left + FLeftOffset,
                          YPos - ParentGrid.FooterYPosition - LineWidth (ParentGrid.FooterSeparatorStyle),
                          Result.Column,
                          Result.ColumnLocation) ;
      End ;
    End
  Else
    Begin { Unused Area following the footer. }
    Result.GridLocation := glUnused ;
    End ;

  End ; { TjmGridZone.ZoneLocation }



{
  Finds the Row and position within the Row of a zone column.

  In:
  ACol => Column to search for (starting at 1)
  Out:
  Row => Row column was found in (starting at 1)
  Position => Position of the column in the Row (starting at 1)

}
Procedure TjmGridZone.ColumnToRow (ACol : Cardinal ;
                                         var Row : Cardinal ;
                                         var Position : Cardinal) ;
  Var
    IRow : Integer ;
    Count : Integer ;
  Begin
  Count := 0 ;
  For IRow := 1 to FRows.Count Do
    Begin
    If (Rows[IRow].ColumnCount + Count) >= ACol Then
      Begin
      Row := IRow ;
      Position := ACol - Count ;
      Exit ;
      End ;
    Inc (Count, Rows[IRow].ColumnCount) ;
    End ;

  Raise Exception.Create ('Internal Error - Row out of Range') ;
  End ;

{
  This function converts a row index and position within that row to the
  index of the column within the zone.
}
Function TjmGridZone.RowPositionToColumn (ARow : Cardinal ;
                                          APosition : Cardinal) : Cardinal ;
  Var
    IRow : Integer ;
  Begin

  Result := 0 ;
  For IRow := 1 To ARow - 1 Do
    Begin
    Inc (Result, Rows [IRow].ColumnCount) ;
    End ;
  Inc (Result, APosition) ;
  End ;

{
  This function converts a Column object to the index of the row in which
  the column is located.
}
Function TjmGridZone.ColumnAddressToRow (Col : TjmGridColumn) : Cardinal ;
  Var
    IRow : Integer ;
    Index : Integer ;
  Begin
  Index := FColumns.IndexOf (Col) ;
  If Index < 1 Then
    Raise Exception.Create ('Column Not Found') ;

  For IRow := 1 to FRows.Count - 1 Do
    Begin
    If Index < Rows [IRow].FFirstColumn + Rows [IRow].MaxColumns Then
      Begin
      Result := IRow ;
      Exit ;
      End ;
    End ;
  Result := FRows.Count ;

  End ;

{
  This function converts a grid (X, Y) position to a Zone (X, Y) position.
}
Procedure TjmGridZone.ConvertMousePositionToZonePosition (Var X, Y : Integer) ;
  begin
  Dec (X, Left) ;
  Dec (Y, Top) ;
  End ;

