{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains the definition for the TjmGridZone class.

  This class represents an independent, scrollable window within a grid.
}

  TjmGridZone = class (TCustomControl)
    Private
      FGrid : TColossalGrid ;
      FNeedsHScrollBar : Boolean ;
      FHasHScrollBar : Boolean ;
      FReadOnly : Boolean ;


      FCellFrameColor : TColor ;
      FCellFrameStyle : TjmGridFrameStyle ;
      FHeaderFrameColor : TColor ;
      FHeaderFrameStyle : TjmGridFrameStyle ;
      FFooterFrameColor : TColor ;
      FFooterFrameStyle : TjmGridFrameStyle ;
      FActiveFrameColor : TColor ;
      FActiveFrameStyle : TjmGridFrameStyle ;

      FZoneNumber : Cardinal ;
      FOverride : TjmGridOverrideSettings ;
      { When the rightmost column is extended the entire column needs to be
        redrawn when a left scroll occurs.  This marks the leftmost position
        of any extended columns that are visible. }
      FExtendColumnLeftPosition : Integer ;
      FDisableColumnResize : Boolean ;
      FDisableColumnMove : Boolean ;

      FHitTestPoint : TPoint ;  { Location of last WM_NCHITTEST point }

      FState : TjmGridZoneState ;
      FSizingColumn : Cardinal ;  { Column that is being sized. }
      FSizingColumnOffset : Integer ; { Offset of the left size of the column being sized. }
      FScrollCountAtLastClick : Integer ;

      FRowHeight : Cardinal ;
      FRowLineStyle : TjmGridLineStyle ;

      FRows : TjmGridRowList ;
      FColumns : TjmGridColumnList ;

      { Column Movement State Fields }
      FMoveRect : TRect ;
      FMoveColumn : Cardinal ;
      FMoveXOffset : Integer ;
      FMoveYOffset : Integer ;

      FSelectable : Boolean ;

      { Position Properties }
      FZoneWidth : Cardinal ;

      { Background Color Properties }
      FBackgroundColor : TColor ;
      FFooterColor : TColor ;
      FHeaderColor : TColor ;
      FInactiveColor : Tcolor ;
      FSelectColor  : TColor ;
      FUnusedColor : TColor ;

      { Font Properties }
      FFont : TFont ;
      FSelectFont : TFont ;
      FInactiveFont : TFont ;
      FHeaderFont : TFont ;
      FFooterFont : TFont ;

      { Scroll Properties }
      FLeftOffset : Integer ;         { The amount we have scrolled to the left. }
      FShowHorizontalScrollBar : TjmGridScrollBars ;
      FExtendRightColumn : Boolean ;


      { Line Properties }
      FHorizontalLineColor : TColor ;

      FRowLineColor : TColor ;

      FVerticalLineStyle : TjmGridLineStyle ;
      FVerticalLineColor : TColor ;

      { Property Functions }
      Function GetBackgroundColor : TColor ;
      Procedure SetBackgroundColor (Value : TColor) ;
      Function GetColumns (Index : Cardinal) : TjmGridColumn ;
      Function GetRows (Index : Cardinal) : TjmGridRow ;

      Procedure SetExtendRightColumn (State : Boolean) ;
      Procedure SetFont (AFont : TFont) ;
      Function GetFooterColor : TColor ;
      Procedure SetFooterColor (AColor : TColor) ;
      Procedure SetFooterFont (AFont : TFont) ;
      Procedure SetHeaderFont (AFont : TFont) ;
      Function GetHorizontalLineColor : TColor ;
      Procedure SetHorizontalLineColor (AColor : TColor) ;
      Procedure SetInactiveFont (AFont : TFont) ;
      Function GetLastNonhiddenColumn : Cardinal ;
      Procedure SetOverride (AValue : TjmGridOverrideSettings) ;
      Function GetRowCount : Cardinal ;
      Procedure SetRowCount (ACount : Cardinal) ;
      Procedure SetRowLineColor (AColor : TColor) ;
      Function GetRowLineColor : TColor ;
      Procedure SetRowLineStyle (AStyle : TjmGridLineStyle) ;
      Function GetRowLineStyle : TjmGridLineStyle ;
      Function GetRowHeight : Cardinal ;
      Procedure SetRowHeight (AValue : Cardinal) ;
      Procedure SetSelectColor (AColor : TColor) ;
      Function GetSelectColor : TColor ;
      Procedure SetSelectFont (AFont : TFont) ;
      Procedure SetShowHorizontalScrollBar (AState : TjmGridScrollBars) ;
      Function GetVerticalLineColor : TColor ;
      Procedure SetVerticalLineColor (AColor : TColor) ;
      Function GetVerticalLineStyle : TjmGridLineStyle ;
      Procedure SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;

      Function GetCellFrameColor : TColor ;
      Procedure SetCellFrameColor (AColor : TColor) ;
      Function GetCellFrameStyle : TjmGridFrameStyle ;
      Procedure SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetHeaderFrameColor : TColor ;
      Procedure SetHeaderFrameColor (AColor : TColor) ;
      Function GetHeaderFrameStyle : TjmGridFrameStyle ;
      Procedure SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetFooterFrameColor : TColor ;
      Procedure SetFooterFrameColor (AColor : TColor) ;
      Function GetFooterFrameStyle : TjmGridFrameStyle ;
      Procedure SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
      Function GetActiveFrameColor : TColor ;
      Procedure SetActiveFrameColor (AColor : TColor) ;
      Function GetActiveFrameStyle : TjmGridFrameStyle ;
      Procedure SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;

      { Scroll Functions }
      Function IsScrollBarNeeded : Boolean ;
      Function GetScrollPosition : Integer ;
      Procedure MakeColumnVisible (ACol : Cardinal) ;
      Procedure MakeLeftOffset (NewOffset : Cardinal) ;
      Procedure MakeTopRecord (NewRecord : LongInt) ;
      Function NextLeftScrollPosition (StartPos : Integer) : Integer ;
      Function NextRightScrollPosition (StartPos : Integer) : Integer ;
      Procedure SetScrollBarRange ;
      Procedure SetScrollPosition ;

      { Action Procedures }
      Procedure AddColumn (AColumn : TjmGridColumn) ;
      Procedure AddRow (ARow : TjmGridRow) ;
      Procedure AdjustRowCount (ACount : Integer) ;
      Procedure RedrawCell (ACol : Cardinal ; ARecord : LongInt) ;
      Procedure RedrawFooterColumn (ACol : Cardinal) ;
      Procedure RedrawHeaderColumn (ACol : Cardinal) ;
      Procedure RedrawRecordArea ;
      Procedure LayoutChanged ;
      Procedure PositionColumnsInRows ;
      Function StoreAppearanceProperties : Boolean ;


      { Measurement Functions }
      Function GetCellRect (ACol : Cardinal ; ARecord : LongInt) : TRect ;
      Function GetCellRectUnclipped (ACol : Cardinal ; ARecord : LongInt) : TRect ;
      Function GetRecordRect : TRect ;
      Function GetColumnOffset (ACol : Cardinal) : Integer ;
      Function GetFooterColumnRect (ACol : Cardinal) : TRect ;
      Function GetHeaderColumnRect (ACol : Cardinal) : TRect ;
      Function RowAtOffset (YOffset : Integer) : Integer ;
      Function GetZonePosition (XPos : Integer ;
                                 YPos : Integer) : TjmGridPosition ;
      Function RecordAtYPosition (YPos : Integer) : LongInt ;

      { Drawing Functions }
      Procedure RedrawRect (const ARect : TRect) ;
      Procedure StartColumnMove (Column : Cardinal ;
                                 MouseX : Integer ;
                                 MouseY : Integer) ;
      Procedure MoveColumnTo (MouseX : Integer ;
                              MouseY : Integer ;
                              Position : TjmGridPosition) ;
      Procedure EndColumnMove (MouseX : Integer ; MouseY : Integer) ;
      Procedure DrawMoveRect ;

      { Conversion Functions }
      Procedure ColumnToRow (ACol : Cardinal ;
                                var Row : Cardinal ;
                                var Position : Cardinal) ;
      Function GetRowOffset (ARow : Cardinal) : Integer ;
      Function RowPositionToColumn (ARow : Cardinal ; APosition : Cardinal) : Cardinal ;

      Function ColumnAddressToRow (Col : TjmGridColumn) : Cardinal ;
      Procedure ConvertMousePositionToZonePosition (Var X, Y : Integer) ;

      Procedure FontChanged (Sender : TObject) ;


      Procedure PositionEditor (Editor : TWinControl ; Column : Cardinal ; RecordNumber : LongInt) ;

      Procedure FindFields ;

    Protected  {*** TjmGridZone ***}
      Procedure ResizeZone ; 
      Procedure WMEraseBkgnd (Var Message : TWMEraseBkgnd) ; Message WM_ERASEBKGND ;
      Procedure WMHScroll (Var Message : TWMHScroll) ; Message WM_HSCROLL ;
      Procedure WMSetCursor(var Msg : TWMSetCursor) ; message WM_SETCURSOR ;
      Procedure WMNCHitTest(var Msg : TWMNCHitTest) ; Message WM_NCHITTEST ;
      Procedure WMSetFocus (var msg : TWMSetFocus) ; Message WM_SETFOCUS ;
      Procedure WndProc (var Message : TMessage) ; Override ;
      Procedure GetChildren (Proc : TGetChildProc ; Root : TComponent) ; Override ;
      Function GetChildOwner : TComponent ; Override ;
      Procedure CreateParams (var Params : TCreateParams) ; Override ;

      procedure MouseDown (Button : TMouseButton ;
                           Shift : TShiftState ;
                           X , Y : Integer) ; Override ;
      procedure MouseMove(Shift : TShiftState ; X, Y : Integer) ; Override ;
      procedure MouseUp(Button : TMouseButton ; Shift : TShiftState ; X, Y : Integer) ; Override ;
      Procedure Paint ; Override ;

      Function MoveDown (OldCol : Cardinal ;
                         OldRecord : LongInt ;
                         var NewCol : Cardinal) : Boolean ; Virtual ;
      Function MoveLeft (OldCol : Cardinal ; var NewCol : Cardinal) : Boolean ; Virtual ;
      Function MoveRight (OldCol : Cardinal ; var NewCol : Cardinal) : Boolean ; Virtual ;
      Function MoveUp (OldCol : Cardinal ;
                       OldRecord : LongInt ;
                       var NewCol : Cardinal) : Boolean ; Virtual ;

      Function DesiredRecordHeight : Cardinal ;

      Procedure SetZoneWidth (AZoneWidth : Cardinal) ;

      Function GetColumnCount : Cardinal ;
      Procedure SetColumnCount (ACount : Cardinal) ;

      Function GetHeaderColor : TColor ;
      Procedure SetHeaderColor (AColor : TColor) ;

      Function GetUnusedColor : TColor ;
      Procedure SetUnusedColor (AColor : TColor) ;

      Function RowWidth : Cardinal ;


      Function GetInactiveColor : TColor ;
      Procedure SetInactiveColor (AColor : TColor) ;



      Function EffectiveFont : TFont ;
      Function EffectiveSelectFont : TFont ;
      Function EffectiveHeaderFont : TFont ;
      Function EffectiveFooterFont : TFont ;
      Function EffectiveInactiveFont : TFont ;

    Public     {*** TjmGridZone ***}
      Constructor Create (AGrid : TComponent) ; Override ;
      Destructor Destroy ; Override ;

      Procedure MoveColumn (OldPosition : Cardinal ; NewPosition : Cardinal) ;


      Property ZoneNumber : Cardinal
               Read FZoneNumber ;
      Property ZonePosition [X : Integer ; Y : Integer] : TjmGridPosition
               Read GetZonePosition ;
      Property CellRect [Column : Cardinal ; RecordNumber : LongInt] : TRect
               Read GetCellRect ;
      Property Columns [Index : Cardinal] : TjmGridColumn
               Read GetColumns
               Stored False ;
      Property Rows [Index : Cardinal] : TjmGridRow
               Read GetRows
               Stored False ;
      Property LastNonHiddenColumn : Cardinal
               Read GetLastNonHiddenColumn
               Stored False ;
      Property ParentGrid : TColossalGrid
               Read FGrid
               Stored False ;
    Published
      Property ActiveFrameColor : TColor
               Read GetActiveFrameColor
               Write SetActiveFrameColor
               Stored StoreAppearanceProperties ;
      Property ActiveFrameStyle : TjmGridFrameStyle
               Read GetActiveFrameStyle
               Write SetActiveFrameStyle
               Stored StoreAppearanceProperties ;
      Property BackgroundColor : TColor
               Read GetBackgroundColor
               Write SetBackgroundColor
               Stored StoreAppearanceProperties ;
      Property CellFrameColor : TColor
               Read GetCellFrameColor
               Write SetCellFrameColor
               Stored StoreAppearanceProperties ;
      Property CellFrameStyle : TjmGridFrameStyle
               Read GetCellFrameStyle
               Write SetCellFrameStyle
               Stored StoreAppearanceProperties ;
      Property ColumnCount : Cardinal
               Read GetColumnCount
               Write SetColumnCount
               Stored False ;
      Property DisableColumnMove : Boolean
               Read FDisableColumnMove
               Write FDisableColumnMove
               Default False ;
      Property DisableColumnResize : Boolean
               Read FDisableColumnResize
               Write FDisableColumnResize
               Default False ;
      Property ExtendRightColumn : Boolean
               Read FExtendRightColumn
               Write SetExtendRightColumn
               Default True ;
      Property Font : TFont
               Read FFont
               Write SetFont ;
      Property FooterColor : TColor
               Read GetFooterColor
               Write SetFooterColor
               Stored StoreAppearanceProperties ;
      Property FooterFrameColor : TColor
               Read GetFooterFrameColor
               Write SetFooterFrameColor
               Stored StoreAppearanceProperties ;
      Property FooterFrameStyle : TjmGridFrameStyle
               Read GetFooterFrameStyle
               Write SetFooterFrameStyle
               Stored StoreAppearanceProperties ;
      Property HeaderFrameColor : TColor
               Read GetHeaderFrameColor
               Write SetHeaderFrameColor
               Stored StoreAppearanceProperties ;
      Property HeaderFrameStyle : TjmGridFrameStyle
               Read GetHeaderFrameStyle
               Write SetHeaderFrameStyle
               Stored StoreAppearanceProperties ;
      Property FooterFont : TFont
               Read FFooterFont
               Write SetFooterFont ;
      Property HasHorizontalScrollBar : Boolean
               Read FHasHScrollBar
               Stored False ;
      Property HeaderColor : TColor
               Read GetHeaderColor
               Write SetHeaderColor
               Stored StoreAppearanceProperties ;
      Property HeaderFont : TFont
               Read FHeaderFont
               Write SetHeaderFont ;
      Property HorizontalLineColor : TColor
               Read GetHorizontalLineColor
               Write SetHorizontalLineColor
               Stored StoreAppearanceProperties ;
      Property InactiveColor : TColor
               Read GetInactiveColor
               Write SetInactiveColor
               Stored StoreAppearanceProperties ;
      Property InactiveFont : TFont
               Read FInactiveFont
               Write SetInactiveFont ;
      Property OverrideSettings : TjmGridOverrideSettings
               Read FOverride
               Write SetOverride
               Default [] ;
      Property ReadOnly : Boolean
               Read FReadOnly
               Write FReadOnly ;
      Property RowCount : Cardinal
               Read GetRowCount
               Write SetRowCount
               Stored False ; { Rows get loaded on their own. }
      Property RowHeight : Cardinal
               Read GetRowHeight
               Write SetRowHeight
               Stored StoreAppearanceProperties ;
      Property RowLineColor : TColor
               Read GetRowLineColor
               Write SetRowLineColor
               Stored StoreAppearanceProperties ;
      Property RowLineStyle : TjmGridLineStyle
               Read GetRowLineStyle
               Write SetRowLineStyle ;
      Property Selectable : Boolean
               Read FSelectable
               Write FSelectable
               Default True ;
      Property SelectColor : TColor
               Read GetSelectColor
               Write SetSelectColor
               Stored StoreAppearanceProperties ;
      Property SelectFont : TFont
               Read FSelectFont
               Write SetSelectFont ;
      Property ShowHorizontalScrollBar : TjmGridScrollBars
               Read FShowHorizontalScrollBar
               Write SetShowHorizontalScrollBar
               Default sbWhenNeeded ;
      property UnusedColor : TColor
               Read GetUnusedColor
               Write SetUnusedColor
               Stored StoreAppearanceProperties ;
      Property VerticalLineColor : TColor
               Read GetVerticalLineColor
               Write SetVerticalLineColor
               Stored StoreAppearanceProperties ;
      Property VerticalLineStyle : TjmGridLineStyle
               Read GetVerticalLineStyle
               Write SetVerticalLineStyle
               Stored StoreAppearanceProperties ;
      Property ZoneWidth : Cardinal
               Read FZoneWidth
               Write SetZoneWidth ;
    End ; {*** TjmGridZone ***}

