{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains the implementation of TjmGridZone methods that deal with
  drawing the zone object.
}

{
  Procedure:  TjmGridZone.StartColumnMove

  Description:
      This procedure starts a user initiated column movement.  We perform the
      following steps:

      1. Save the ID of the column we are moving.
      2. Save the location where the mouse was when the movement started.
        This information is used to draw the outline of the column header during
        the movement.
      3. Draw the outline of the header to visually show the user the column
        movement.

  Parameters:
    Column : The column being moved
    MouseX, MouseY : The position of the mouse at the start of the move.

}
Procedure TjmGridZone.StartColumnMove (Column : Cardinal ;
                                             MouseX : Integer ;
                                             MouseY : Integer) ;
  Begin
  FMoveColumn := Column ;
  FMoveRect := GetHeaderColumnRect (Column) ;
  FMoveXOffset := MouseX - FMoveRect.Left ;
  FMoveYOffset := MouseY - FMoveRect.Top ;
  DrawMoveRect ;
  End ;

{
  Procedure:  TjmGridZone.MoveColumnTo

  Description:
    This procedure is called when the user moves the mouse during a column
    move operation.  This procedure moves the column header outline.
}
Procedure TjmGridZone.MoveColumnTo (MouseX : Integer ;
                                          MouseY : Integer ;
                                          Position : TjmGridPosition) ;
  Var
    Delta : Integer ;
  Begin
  { Erase the old rectangle. }
  DrawMoveRect ;
  Delta := MouseX - FMoveRect.Left - FMoveXOffset ;
  Inc (FMoveRect.Left, Delta) ;
  Inc (FMoveRect.Right, Delta) ;

  { Draw the new position. }
  Delta := MouseY - FMoveRect.Top - FMoveYOffset ;
  Inc (FMoveRect.Top, Delta) ;
  Inc (FMoveRect.Bottom, Delta) ;
  DrawMoveRect ;
  End ;

{
  Procedure:  TjmGridZone.EndColumnMove

  Description:
    This proceudre is called when the user ends a column move operation.  This
    is where the column actually gets moved to the new position.

  Parameters:
    MouseX, MouseY : The position of the mouse at the end of the move.
}
Procedure TjmGridZone.EndColumnMove (MouseX : Integer ; MouseY : Integer) ;
  Var
    Position : TjmGridPosition ;
  Begin
  { Erase the outline box. }
  DrawMoveRect ;

  { Determine where the column is being moved to. }
  Position := ZonePosition [MouseX, MouseY] ; 

  If (Position.Zone = FZoneNumber)
     And (Position.Zone = FZoneNumber)
     And (Position.GridLocation = glHeader)
     And (Position.Column <> FMoveColumn)
     And (Position.COlumn > 0) Then
    Begin
    If Assigned (ParentGrid.OnColumnMoved) Then
      Begin
      ParentGrid.OnColumnMoved (ParentGrid, FZoneNumber, Position.Column, FMoveColumn)
      End ;
    MoveColumn (FMoveColumn, Position.Column) ;
    End ;
  FMoveColumn := 0 ;
  End ;

{
  Procedure:  TjmGridZone.DrawMoveRect

  Description:
    This procedure draws the column header outline that gives the user visual
    clues when he is dragging a column to a new position.

  Parameters: None

}
Procedure TjmGridZone.DrawMoveRect ;
  Begin
  Canvas.Pen.Color := clBlack ;
  Canvas.Pen.Style := psSolid ;
  Canvas.Pen.Width := 4 ;
  Canvas.Pen.Mode := pmNotXOr ;
  Canvas.Brush.Style := bsClear ;
  Canvas.Rectangle (FMoveRect.Left, FMoveRect.Top, FMoveRect.Right, FMoveRect.Bottom) ;
  End ;

{
  Procedure:  TjmGridZone.RedrawRect

  Description:
    This procedure repaint or rather tells Windows to redraw a rectangular
    area.  This is simply an encapsulation of InvalidateRect.  We use our own
    procedure so that we can isolate painting problems.

  Parameters:
    ARect : The rectangle to be redrawn.

}
Procedure TjmGridZone.RedrawRect (const ARect : TRect) ;
  Begin
  InvalidateRect (Handle, @ARect, False) ;
  End ;



