{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains TjmGridZone methods that deal with measuring.
}

{
  This function returns a rectangle containing the record area of the grid.
}
Function TjmGridZone.GetRecordRect : TRect ;
  Begin
  Result := ClientRect ;
  Result.Top := ParentGrid.RecordAreaTop ;
  Result.Bottom := Result.Top + ParentGrid.RecordAreaheight ;
  End ;  { TjmGridZone.GetRecordRect }

{
  This function determines the maximum width of all the subrows within a record.
}
Function TjmGridZone.RowWidth : Cardinal ;
  Var
    IRow : Integer ;
    ThisWidth : Integer ;
  Begin
  Result := 0 ;
  For IRow := 1 To FRows.Count Do
    Begin
    ThisWidth := Rows [IRow].RowWidth ;
    If ThisWidth > Result Then
      Result := ThisWidth ;
    End ;
  End ;

{
  TjmGridZone.CellRect:
    Returns the viewable rectangle of the specified cell.
}
Function TjmGridZone.GetCellRect (ACol : Cardinal ; ARecord : LongInt) : TRect ;
  Var
    RecordRect : TRect ;
  Begin
  Result := GetCellRectUnclipped (ACol, ARecord) ;

  RecordRect := GetRecordRect ;
  If (Result.Left < RecordRect.Left) Then
    Result.Left := RecordRect.Left ;

  If (Result.Right > RecordRect.Right) Then
    Result.Right := RecordRect.Right ;

  End ;

{
  This gets the rectangle for the specified cell, including those parts that
  are not visible.
}
Function TjmGridZone.GetCellRectUnclipped (ACol : Cardinal ; ARecord : LongInt) : TRect ;
  Var
    RecordRect : TRect ;
    IRow : Cardinal ;
    Position : Cardinal ;
    RecordStart : Integer ;
  Begin

  { Validations }
  If (ACol < 1) Or (ACol > ColumnCount) Then
    Raise Exception.CreateFmt ('Column number (%d) outside of range 1..%d', [ACol, ColumnCount]) ;

  If (ARecord < 1) Or (ARecord > ParentGrid.RecordCount) Then
    Raise Exception.CreateFmt ('Record number (%d) outside of range 1..%d', [ARecord, ParentGrid.RecordCount]) ;


  { A hidden column has no grid rect. }
  If Columns [ACol].Hidden Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  { See if the cell is off the top. }
  If ARecord < ParentGrid.TopVisibleRecord Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  { See if the cell is off the bottom. }
  If ARecord > ParentGrid.BottomVisibleRecord Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  RecordRect := GetRecordRect ;

  { Find the Row/Position of the column }
  ColumnToRow (ACol, IRow, Position) ;

  Result.Left := Rows [IRow].GetColumnOffset(Position) ;

  Dec (Result.Left, FLeftOffset) ; { Account for scrolling }

  { Find the Y position of the start of the record. }
  RecordStart := (ARecord - ParentGrid.TopVisibleRecord)
                * (ParentGrid.RecordHeight + LineWidth (ParentGrid.HorizontalLineStyle))
                + ParentGrid.EffectiveHeaderheight ;

  Result.Top := RecordStart + GetRowOffset (IRow) ;

  If IRow <> FRows.Count Then
    Result.Bottom := Result.top + Rows [IRow].RowHeight
  Else
    Result.Bottom := RecordStart + ParentGrid.RecordHeight ;

  { If we are extending the right column and this is the last column in the
    row then the right side is the edge of the zone.  Otherwise we use the
    zone width. }
  If ExtendRightColumn And (Rows [IRow].LastNonHiddenColumn = Position)  Then
    Result.Right := RecordRect.Right
  Else
    Result.Right := Result.Left + Rows [IRow].Columns [Position].Width ;

  { Make sure the cell is not completely off of the chart. }
  If IsIntersectionNull (Result, RecordRect) Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  { Make sure the cell does not go off the edges. }
  If (Result.Top < RecordRect.Top) Then
    Result.Top := RecordRect.Top ;

  If (Result.Bottom > RecordRect.Bottom) Then
    Result.Bottom := RecordRect.Bottom ;
  End ;

{
  TjmGridRow.GetRowRect:

    Returns the offset of the top of a Row.
}

Function TjmGridZone.GetRowOffset (ARow : Cardinal) : Integer ;
  Var
    IRow : Integer ;
  Begin

  If (ARow < 1) Or (ARow > FRows.Count) Then
    Raise Exception.CreateFmt ('Row (%d) out of range 1..%d', [ARow, FRows.Count]) ;
  Result := 0 ;
  For IRow := 1 To ARow - 1 Do
    Begin
    Inc (Result, Rows [IRow].RowHeight + LineWidth (Rows [IRow].RowLineStyle)) ;
    End ;
  End ;

{
  Returns the offset from the left of the grid of a column
  taking into account the row the column is in.
}
Function TjmGridZone.GetColumnOffset (ACol : Cardinal) : Integer ;
  Var
    IRow : Cardinal ;
    Position : Cardinal ;
  Begin
  ColumnToRow (ACol, IRow, Position) ;
  Result := Rows [IRow].GetColumnOffset (Position) ;
  End ;
{
Function TjmGridZone.GetClientRect : TRect ;
  Begin
  Result := ClientRect ;
  Dec (Result.Bottom, EffectiveHorizontalScrollBarHeight) ;
  End ;
}

Function TjmGridZone.GetHeaderColumnRect (ACol : Cardinal) : TRect ;
  Var
    HeaderRect : TRect ;
    IRow : Cardinal ;      { Row the column is located in. }
    Position : Cardinal ;  { Position of the column within the row }
  Begin
  If (ACol < 1) Or (ACol > ColumnCount) Then
    Raise Exception.CreateFmt ('Column number (%d) outside of range 1..%d', [ACol, ColumnCount]) ;

  If Columns [ACol].Hidden Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  HeaderRect := ClientRect ;
  HeaderRect.Bottom := HeaderRect.Top + ParentGrid.EffectiveHeaderHeight ;

  Result := HeaderRect ;

  { Find the Row/Position of the column }
  ColumnToRow (ACol, IRow, Position) ;
  Result.Left := Rows [IRow].GetColumnOffset(Position) - FLeftOffset ;

  Inc (Result.Top, GetRowOffset (IRow)) ;
  Result.Bottom := Result.Top + RowHeight ;

  If (Position = Rows [IRow].LastNonHiddenColumn) and ExtendRightColumn Then
    Result.Right := HeaderRect.Right
  Else
    Result.Right := Result.Left + Columns [ACol].Width ;

  { Make sure the cell is not completely off of the chart. }
  If (Result.Left > HeaderRect.Right)
     Or (Result.right < HeaderRect.Left) Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  { Make sure the cell does not go off the edges. }
  If (Result.Left < HeaderRect.Left) Then
    Result.Left := HeaderRect.Left ;

  If (Result.Right > HeaderRect.Right) Then
    Result.Right := HeaderRect.Right ;

  End ;

{
  This function returns the Rect for the footer of a column.
}
Function TjmGridZone.GetFooterColumnRect (ACol : Cardinal) : TRect ;
  Var
    FooterRect : TRect ;
    IRow : Cardinal ;      { Row the column is located in. }
    Position : Cardinal ;  { Position of the column within the row }
  Begin
  If (ACol < 1) Or (ACol > ColumnCount) Then
    Raise Exception.CreateFmt ('Column number (%d) outside of range 1..%d', [ACol, ColumnCount]) ;

  If Columns [ACol].Hidden Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  FooterRect := ClientRect ;
  FooterRect.Top := ParentGrid.FooterYPosition ;
  FooterRect.Bottom := FooterRect.Top + ParentGrid.EffectiveFooterHeight ;

  Result := FooterRect ;

  { Find the Row/Position of the column }
  ColumnToRow (ACol, IRow, Position) ;

  Result.Left := Rows [IRow].GetColumnOffset(Position) - FLeftOffset ;

  Inc (Result.Top, GetRowOffset (IRow)) ;

  If (Position = Rows [IRow].LastNonHiddenColumn) and ExtendRightColumn Then
    Result.Right := FooterRect.Right
  Else
    Result.Right := Result.Left + Columns [ACol].Width ;

  { Make sure the cell is not completely off of the chart. }
  If (Result.Left > FooterRect.Right)
     Or (Result.right < FooterRect.Left) Then
    Begin
    Result := ZeroRect ;
    Exit ;
    End ;

  { Make sure the cell does not go off the edges. }
  If (Result.Left < FooterRect.Left) Then
    Result.Left := FooterRect.Left ;

  If (Result.Right > FooterRect.Right) Then
    Result.Right := FooterRect.Right ;

  End ;

{
  This function determines the Row at a specific offset from the top of a
  record.
}
Function TjmGridZone.RowAtOffset (YOffset : Integer) : Integer ;
  Var
    IRow : Integer ;
    Position : Integer ;
    RowHeight : Integer ;
  Begin
  If FRows.Count = 0 Then
    Begin
    Result := 0 ;
    Exit ;
    End ;

  Position := 0 ;
  For IRow := 1 To FRows.Count Do
    Begin
    RowHeight := Rows [IRow].RowHeight + LineWidth (ParentGrid.RowLineStyle) ;
    If YOffset <= Position + RowHeight Then
      Begin
      Result := IRow ;
      Exit ;
      End ;
    Inc (Position, RowHeight) ;
    End ;
  Result := FRows.Count ;
  End ;

{
  This function returns the maximum height (unclipped) for a record.
}
Function TjmGridZone.DesiredRecordHeight : Cardinal ;
  Var
    IRow : Cardinal ;
  Begin
  Result := 0 ;
  For IRow := 1 To RowCount - 1 Do
    Begin
    Inc (Result, Rows [IRow].RowHeight + LineWidth (Rows [IRow].RowLineStyle)) ;
    End ;
  { The last row does not have a line. }
  If RowCount <> 0 Then
    Inc (Result, Rows [RowCount].RowHeight) ;
  End ;

{
  Function: TjmGridZone.RecordAtYPosition

    This function translates a Y position relative to the top of the zone into
    the record number under that point.

  Parameters:
    YPos : The position to translate to a record number.

  Return Value:
    0 => No record is at the position
    > 0 => The record number
}
Function TjmGridZone.RecordAtYPosition (YPos : Integer) : LongInt ;
   Var
    RecY : Integer ; { Position relative to the start of the record area. }
    RelRect : LongInt ; { Record Offset }
   Begin
   If (YPos < ParentGrid.RecordAreaYPosition)
      Or (YPos >= ParentGrid.FooterYPosition) Then
    Begin
    Result := 0 ;
    End
   Else
    Begin
    RecY := YPos - ParentGrid.RecordAreaYPosition ;
    RelRect := RecY Div
       (ParentGrid.RecordHeight + LineWidth (ParentGrid.HorizontalLineStyle)) ;
    Result := RelRect + Parentgrid.TopVisibleRecord ;
    If Result > ParentGrid.RecordCount Then
      Result := 0 ;
    End ;
   End ;





