{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains VCL methods that are overridden by the TjmGridZone class.
}

{
  Procedure:  TjmGridZone.WndProc (Overridden)

  Description:  Windows Procedure for the Zone

      Functions

      1. Design Mode

      The Zone is a component but it has no owner in the Delphi sense.  The grid
      needs to strictly control the order, creation, and deletion of zones.
      This causes a problem in design mode because when the user clicks on the
      zone nothing will happen because Delphi will not be able to find the form
      the zone is on.  That is why we need to override this procedure.  The
      code is similar to TControl.WndProc.

      2. Run-Time

      We need to pass mouse messages to the grid in order to trigger user events.

  Parameters:
      Message : The message being passed to the zone.
}

Procedure TjmGridZone.WndProc (var Message : TMessage) ;
  Var
    Form : TCustomForm ;
  Begin
  If csDesigning In ParentGrid.ComponentState Then
    Begin
    { The form knows about the grid but not the zone so we help the form out
      a little here by telling it who our theoretical owner is. }
    Form := GetParentForm (ParentGrid) ;
    If (Form <> Nil) And (Form.Designer <> Nil)
       And Form.Designer.IsDesignMsg (Self, Message) Then
      Begin
      Exit ;
      End ;
    End ;

  { Pass mouse messages up to the grid. }
  Case Message.Msg Of
    WM_MOUSEFIRST .. WM_MOUSELAST:
      Begin
      With TWmMouse (Message) Do
        Begin
        { Correct for Zone offset. }
        Inc (XPos, Left) ;
        Inc (YPos, Top) ;
        ParentGrid.Wndproc (Message) ;
        Inherited ;
        Exit ;
        End ;
      End ;
    CM_DRAG:
      Begin
      ParentGrid.WndProc (Message) ;
      Exit ;  { We do not want inherited processing here. }
      End ;
    WM_KEYFIRST..WM_KEYLAST:
      Begin
      ParentGrid.WndProc (Message) ;
      Inherited ;
      End ;
    End ;

  Inherited ;
  End ;


{
  Procedure:  TjmGridZone.Paint (Overridden)

  Description:  Draws the zone

    The following areas of the zone are drawn:

    o Header

    o Record Area

    o Footer

    o Unused Area below the footer of this zone does not have a scroll bar
    and at least other zone in the grid does.

  Parameters: None
}

Procedure TjmGridZone.Paint ;
  Var
    Rect : TRect ;

  {
    Function TjmGridZone.Paint.AdjustScrollState

    Description:  Enables or disables the scroll bar as needed.

    Return Value:
        True => The scroll bar state was changed
        False => The scroll bar state remained the same.
  }
  Function AdjustScrollState : Boolean ;
    Begin
    Result := False ;
    FNeedsHScrollBar := IsScrollBarNeeded ;
    If FNeedsHScrollBar <> FHasHScrollBar Then
      Begin
      RecreateWnd ;
      FGrid.ResizeGrid ;
      Result := True ;
      End ;
    End ; { AdjustScrollState }

  {
    Procedure TjmGridZone.Paint.DrawRecord

    Description:  Draws a single record in the zone.
  }
  Procedure DrawRecord (RecordNumber : LongInt ; ARect : TRect) ;
    Var
      IRow : Integer ;
      YY : Integer ;
      DrawRect : TRect ;
    Begin
    YY := ARect.Top ;
    DrawRect := ARect ;
    For IRow := 1 To FRows.Count Do
      Begin
      If IRow <> 1 Then
        Begin
        { Draw the line between the Rows. }
        DrawLine (Canvas,
                  DrawRect.Left, YY,
                  DrawRect.Right, YY,
                  Rows [IRow - 1].RowLineColor,
                  Rows [IRow - 1].RowLineStyle) ;
        Inc (YY, LineWidth (Rows [IRow - 1].RowLineStyle)) ;
        End ;
      DrawRect.Top := YY ;
      If IRow <> FRows.Count Then
        DrawRect.Bottom := YY + Rows [IRow].RowHeight
      Else
        DrawREct.Bottom := ARect.Bottom ;

      If Not IsIntersectionNull (Canvas.ClipRect, DrawRect) Then
        Rows [IRow].DrawRecords (RecordNumber, Canvas, DrawRect, FLeftOffset) ;

      YY := DrawRect.Bottom ;
      End ;
    End ;  { DrawRecord. }

  {
    Procedure TjmGridZone.Paint.DrawRecords

    Description:  Draws all of the visibles records in the zone.
  }
  Procedure DrawRecords ;
    Var
      II : LongInt ;
      YY : Integer ;
      RowRect : TRect ; { Space for drawing rows in the grid. }
      DrawRect : TRect ;
      ClipRegion : HRGN ;
    Begin
    RowRect := GetRecordRect ;

    { Create a clip region to prevent drawing outside the lines. }

    ClipRegion := WinProcs.CreateRectRgnIndirect (RowRect) ;
    { Protect the clip region }
    Try
      SelectClipRgn (Canvas.Handle, ClipRegion) ;

      { Set the left and right }
      DrawRect := RowRect ;

      FExtendColumnLeftPosition := High (Integer) ;
      YY := RowRect.Top ;
      For II := ParentGrid.TopVisibleRecord To ParentGrid.BottomVisibleRecord Do
        Begin
        DrawRect.Top := YY ;
        DrawRect.Bottom := YY + ParentGrid.RecordHeight ;
        DrawRecord (II, DrawRect) ;

        Inc (YY, ParentGrid.RecordHeight) ;

        { Draw the line separating the records. }
        If (YY < RowRect.Bottom) Then
          Begin
          DrawLine (Canvas,
                    DrawRect.Left, YY,
                    DrawRect.Right, YY,
                    HorizontalLineColor,
                    ParentGrid.HorizontalLineStyle) ;
          Inc (YY, LineWidth (ParentGrid.HorizontalLineStyle)) ;
          End ;

        End ;

        { Fill In the unused areas }
        DrawRect.Top := YY ;
        DrawRect.Bottom := RowRect.Bottom ;
        Canvas.Brush.Color := UnusedColor ;
        Canvas.FillRect (DrawRect) ;

      Finally
        SelectClipRgn (Canvas.Handle, 0) ;
        DeleteObject (ClipRegion) ;
      End ;
    End ;

  {
    Procedure TjmGridZone.Paint.DrawHeader

    Description:  Draws the zone header.
  }
  Procedure DrawHeader ;
    Var
      DrawRect : TRect ;
      IRow : Integer ;
      HeaderRect : TRect ;
      ClipRegion : HRGN ;
    Begin

    HeaderRect := ClientRect ;
    HeaderRect.Bottom := HeaderRect.Top + ParentGrid.EffectiveHeaderHeight ; ;
    If IsIntersectionNull (Canvas.ClipRect, HeaderRect) Then
      Exit ;  { It would not be painted anyway. }

    ClipRegion := WinProcs.CreateRectRgnIndirect (HeaderRect) ;
    { Protect the clip region. }
    Try
      SelectClipRgn (Canvas.Handle, ClipRegion) ;
      DrawRect := HeaderRect ;
      DrawRect.Bottom := DrawRect.Top ;
      For IRow := 1 To FRows.Count Do
        Begin
        If IRow <> 1 Then
          Begin
          { Draw the line between the rows. }
          DrawLine (Canvas,
                    DrawRect.Left, DrawRect.Bottom,
                    DrawRect.Right, DrawRect.Bottom,
                    clBlack,
                    Rows [IRow - 1].RowLineStyle) ;
          Inc (DrawRect.Bottom, LineWidth (lsSingle)) ;
          End ;
        DrawRect.Top := DrawRect.Bottom ;

        If IRow <> FRows.Count Then
          Inc (DrawRect.Bottom, Rows [IRow].RowHeight)
        Else
          DrawRect.Bottom := HeaderRect.Top + ParentGrid.HeaderHeight ;

        Rows [IRow].DrawHeader (Canvas, DrawRect, FLeftOffset) ;
        End ;

      { Draw the header separator. }
      DrawLine (Canvas,
                DrawRect.Left, HeaderRect.Top + ParentGrid.HeaderHeight,
                DrawRect.Right, HeaderRect.Top + ParentGrid.HeaderHeight,
                ParentGrid.HorizontalLineColor,
                ParentGrid.HeaderSeparatorStyle) ;
    Finally
      SelectClipRgn (Canvas.Handle, 0) ;
      DeleteObject (ClipRegion) ;
      End ;
    End ; { Paint.DrawHeader }

  {
    Procedure TjmGridZone.Paint.DrawFooter

    Description:  Draws the zone footer.
  }
  Procedure DrawFooter ;
    Var
      DrawRect : TRect ;
      IRow : Integer ;
      LineRect : TRect ;
      ClipRegion : HRGN ;
      FooterRect : TRect ;
    Begin
    FooterRect := ClientRect ;
    FooterRect.Top := ParentGrid.FooterYPosition ;
    If IsIntersectionNull (Canvas.ClipRect, FooterRect) Then
      Exit ;  { It would not be painted anyway. }


    ClipRegion := WinProcs.CreateRectRgnIndirect (FooterRect) ;
    SelectClipRgn (Canvas.Handle, ClipRegion) ;

    Try
      DrawRect := FooterRect ;
      DrawRect.Bottom := FooterRect.Top ;
      { Draw the line separating the records from the footer }
      DrawLine (Canvas,
                DrawRect.Left, DrawRect.Bottom,
                DrawRect.Right, DrawRect.Bottom,
                ParentGrid.FooterSeparatorColor,
                ParentGrid.FooterSeparatorStyle) ;
      Inc (DrawRect.Bottom, LineWidth (ParentGrid.FooterSeparatorStyle)) ;

      For IRow := 1 To FRows.Count Do
        Begin
        { Draw the lines between the rows }
        If IRow <> 1 Then
          Begin
          DrawLine (Canvas,
                    DrawRect.Left, DrawRect.Bottom,
                    DrawRect.Right, DrawRect.Bottom,
                    clBlack,
                    Rows [IRow - 1].RowLineStyle) ;
          Inc (DrawRect.Bottom, LineWidth (lsSingle)) ;
          End ;

        DrawRect.Top := DrawRect.Bottom ;

        If IRow <> FRows.Count Then
           Inc (DrawRect.Bottom, Rows [IRow].RowHeight)
        Else
          DrawRect.Bottom := FooterRect.Top + ParentGrid.RecordHeight + LineWidth (ParentGrid.FooterSeparatorStyle);

        If IRow = 1 Then
          Begin
          LineRect := DrawRect ;
          Rows [IRow].DrawVerticalLines (Canvas, LineRect, FLeftOffset) ;
          End
        Else
          Begin
          Rows [IRow].DrawVerticalLines (Canvas, DrawRect, FLeftOffset) ;
          End ;

        Rows [IRow].DrawFooter (Canvas, DrawRect, FLeftOffset) ;
        End ;

        DrawLine (Canvas, DrawRect.Left, DrawRect.Bottom, DrawRect.Right, DrawRect.Bottom, clBlack, lsSingle) ;

      Finally
        SelectClipRgn (Canvas.Handle, 0) ;
        DeleteObject (ClipRegion) ;
      End ;
    End ; { TjmGridZone.Paint.DrawFooter }

  Begin { Paint }

  { If the scroll bar state was changed then the window got recreated in which
    case the window from here on does not exist.  If we do not abort then we
    will write to the screen. }
  If AdjustScrollState Then
    Exit ;

  If ParentGrid.HasHeader Then
    DrawHeader ;
  DrawRecords ;
  If ParentGrid.HasFooter Then
    DrawFooter ;

  Canvas.Brush.Color := ParentGrid.Color ;
  Canvas.Brush.Style := bsSolid ;
  Rect := ClientRect ;
  Rect.Top := ParentGrid.FooterYPosition
              + ParentGrid.FooterHeight
              + LineWidth (ParentGrid.FooterSeparatorStyle) + LineWidth (lsSingle) ;
  Canvas.FillRect (Rect) ;

  { Draw the line at the bottom of the zone. }
  If HasHorizontalScrollBar Or ParentGrid.AnyZonesHaveScrollBars Then
    Begin
    Rect := ClientRect ;
    DrawLine (Canvas,
              Rect.Left, Rect.Bottom - LineWidth (ParentGrid.HorizontalScrollBarSeparatorStyle),
              Rect.Right, Rect.Bottom - LineWidth (ParentGrid.HorizontalScrollBarSeparatorStyle),
              ParentGrid.HorizontalScrollBarSeparatorColor,
              ParentGrid.HorizontalScrollBarSeparatorStyle) ;
    End ;

  SetScrollPosition ;
  End ; { TjmGridZone.Paint }

{
  Procedure:    TjmGridZone.MouseDown (Overridden)

  Description:

    This method is called in response to mouse down events.  We have to
    worry about the following things here:

    o Setting focus to the grid when it is clicked.
    o If a partially hidden column or record is clicked then it needs to be
      made completely visible.
    o Starting Column Resizing
    o Starting Column Movement

  Parameters:
    Button : The mouse button that was pressed
    Shift : The shift state
    X, Y : Click position

}

procedure TjmGridZone.MouseDown (Button : TMouseButton ;
                                   Shift : TShiftState ;
                                   X , Y : Integer) ;
  var
    Position : TjmGridPosition ;
    Cell : TjmGridCell ;
  Const
    NullCell : TjmGridCell = () ;
  Begin
  ConvertMousePositionToZonePosition (X, Y) ;
  { Find where the mouse was clicked. }
  Position := ZonePosition [X, Y] ;
  Case Button Of
    mbLeft:
      Begin
      If (Position.GridLocation = glCell)
          And (Position.ColumnLocation = gclBody)
          And Not (ParentGrid.UpdateTextFromEditor)
          And Selectable Then
        Begin

        { A cell has been clicked so it becomes the current cell and needs to
          be made completely visible. }
        Cell.Zone := ZoneNumber ;
        Cell.Column := Position.Column ;
        Cell.RecordNumber := Position.RecordNumber ;
        MakeColumnVisible (Cell.Column) ;
        If Position.RecordNumber = ParentGrid.CurrentCell.RecordNumber Then
          Begin { No record change required. }
          { Don't bother if we are not changing cells. }
          If (Cell.Zone <>  Parentgrid.CurrentCell.Zone)
             Or (Cell.Column <> ParentGrid.CurrentCell.Column) Then
            Begin
            ParentGrid.ChangeCurrentCell (Cell) ;
            End ;
          End
        Else
          Begin { Record change needed. }
          Cell.RecordNumber := ParentGrid.CurrentCell.RecordNumber ;
          Cell.Zone := ZoneNumber ;
          ParentGrid.ChangeCurrentCell (Cell) ;
          ParentGrid.FCurrentCell.Zone := ZoneNumber ;
          ParentGrid.FCurrentCell.Column := Position.Column ;
          Parentgrid.ChangeCurrentRecord (Position.RecordNumber) ;
          End ;

        { Double Click Handler - Display the editor.
          The trick here is the setting of FScrollCount.  We do not display the
          editor if the double click caused the grid to scroll vertically since
          this would mean the second click would hit a different grid position
          than the first. }
        If (ssDouble In Shift)
           And (ParentGrid.FScrollCount = FScrollCountAtLastClick) Then
          Begin
          ParentGrid.EditorVisible := True ;
          End ;
        End
      Else If (Position.ColumnLocation = gclSeparator)
         And Not ParentGrid.DisableColumnResize
         And Not DisableColumnResize Then
        Begin
        ParentGrid.HideEditor (true) ;
        FState := gsColumnSizing ;
        FSizingColumnOffset := GetColumnOffset (Position.Column) ;
        FSizingColumn := Position.Column ;
        End
      { The one trick in this test is that if HasControl is false it will
        be changed to true below.  If we start drawing movement rectangles now
        then things are going to get messed up since the background color of the
        header will change.
      }
      Else If (Position.ColumnLocation = gclBody)
         And (Position.GridLocation = glHeader)
         And (ColumnCount > 1)
         And Not ParentGrid.DisableColumnMove
         And Not DisableColumnMove
         And ParentGrid.HasControl
         Then
        Begin
        { The mouse down has given focus to the window.  We need to redraw
          the window before doing the column move. }
        UpdateWindow (Handle) ;
        FState := gsColumnMoving ;
        StartColumnMove (Position.Column, X, y) ;
        End ;
      End ;
    End ;

  { We save the number of times the grid has been scrolled vertically.  If we
    double click on a partial row the row gets scrolled on the first click
    then the second click comes on a different row.
  }
  FScrollCountAtLastClick := ParentGrid.FScrollCount ;
  End ;

{
  Procedure:  TjmGridZone.MouseMove (Overridden)

  Description:

    In the mouse move event we need to handle:

    o Column Resizing
    o Column Moving

    Right now column resizing is immediate.  On our systems this is fine but
    for slower systems we may need to implement a delay to limit the number
    of repaints.

  Parameters:
    Shift : Shift State
    X, Y : Mouse Position

}
procedure TjmGridZone.MouseMove(Shift : TShiftState ; X, Y : Integer) ;

  Procedure ColumnSizing ;
    Const
      MinWidth = 1 ;
    Var
      Width : Integer ;
    Begin
    Width := X - FSizingColumnOffset + FLeftOffset ;
    If Width < MinWidth Then
      Width := MinWidth ;
    Columns [FSizingColumn].Width := Width ;
    End ;

  Procedure ColumnMoving ;
    var
      Position : TjmGridPosition ;
    Begin
    Position := ZonePosition [X, Y] ;
    If (Position.GridLocation <> glHeader) Then
      Exit ;
    MoveColumnTo (X, Y, Position) ;
    End ;

  Begin
  ConvertMousePositionToZonePosition (X, Y) ;
  Case FState Of
    gsColumnSizing:
      ColumnSizing ;
    gsColumnMoving:
      ColumnMoving ;
    End ;
  End ;

{
  Procedure:  TjmGridZone.MouseUp (Overridden)

  Description:

    In the MouseUp handler we need to take care of:

    o Ending column movement

    In the future we made need to and column resizing as well.

  Parameters:
    Button : The mouse buttun that was lifted
    Shift : Shift State
    X, Y : Mouse Position
}
procedure TjmGridZone.MouseUp(Button : TMouseButton ; Shift : TShiftState ; X, Y : Integer) ;
  Begin
  ConvertMousePositionToZonePosition (X, Y) ;
  Case Button Of
    mbLeft:
      Begin
      Case FState Of
        gsColumnMoving:
          Begin
          EndColumnMove (X, Y)  ;
          End ;
        End ;
      End ;
    End ;

  FState := gsNormal ;
  End ;


{
  Procedure:  TjmGridZone.CreateParams (Overridden)

  Description:

    This procedure sets up the window creation parameters.  We need to take
    care of:

    o Horizontal Scroll Bar

  Parameters:
    Params : The window creation parameters
}
Procedure TjmGridZone.CreateParams (var Params : TCreateParams) ;
  Begin
  Inherited CreateParams (Params) ;
  If FNeedsHScrollBar Then
    Params.Style := Params.Style OR WS_HSCROLL
  Else
    Params.Style := Params.Style AND NOT WS_HSCROLL ;
  FHasHScrollBar := FNeedsHScrollBar ;
  End ;

