{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}

{
  This file contains methods for getting and setting properties for the
  TjmGridZone class.
}

Function TjmGridZone.GetColumns (Index : Cardinal) : TjmGridColumn ;
  Begin
  Result := FColumns [Index] ;
  End ;

Function TjmGridZone.GetRows (Index : Cardinal) : TjmGridRow ;
  Begin
  Result := FRows [Index] ;
  End ;

Function TjmGridZone.GetInactiveColor : TColor ;
  Begin
  If osInactiveColor In FOverride Then
    Result := FInactiveColor
  Else
    Result := ParentGrid.InactiveColor ;
  End ;

Procedure TjmGridZone.SetInactiveColor (AColor : TColor) ;
  Begin
  If (AColor = FInactiveColor) And (osInactiveColor In FOverride) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osInactiveColor) ;
  FInactiveColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetRowLineColor (AColor : TColor) ;
  Begin
  If (AColor = FRowLineColor) And (osRowLineColor In FOverride) Then
    Exit ;

  FRowLineColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowLineColor) ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetRowLineColor : TColor ;
  Begin
  If osRowLineColor in FOverride Then
    Result := FRowLineColor
  Else
    Result := ParentGrid.RowLineColor ;
  End ;

Function TjmGridZone.GetVerticalLineColor : TColor ;
  Begin
  If osVerticalLineColor in FOverride Then
    Result := FVerticalLineColor
  Else
    Result := ParentGrid.VerticalLineColor ;
  End ;

Procedure TjmGridZone.SetVerticalLineColor (AColor : TColor) ;
  Begin
  If (AColor = FVerticalLineColor) And (osVerticalLineColor In FOverride) Then
    Exit ;

  FVerticalLineColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOVerride, osVerticalLineColor) ;
  LayoutChanged ;
  End ;


Function TjmGridZone.GetUnusedColor : TColor ;
  Begin
  If osUnusedColor in FOverride Then
    Result := FUnusedColor
  Else
    Result := ParentGrid.UnusedColor ;
  End ;

Procedure TjmGridZone.SetUnusedColor (AColor : TColor) ;
  Begin
  If (AColor = FUnusedColor) And (osUnusedColor In FOverride) Then
    Exit ;

  FUnusedColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osUnusedColor) ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FFont.Free ;
    FFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FFOnt = Nil Then
      Begin
      FFOnt := TFOnt.Create ;
      FFOnt.OnChange := FontChanged ;
      End ;
    FFont.Assign (AFont) ;
    End ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetHeaderFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FHeaderFont.Free ;
    FHeaderFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FHeaderFont = Nil Then
      Begin
      FHeaderFont := TFOnt.Create ;
      FHeaderFont.OnChange := FontChanged ;
      End ;
    FHeaderFont.Assign (AFont) ;
    End ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetFooterFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FFooterFont.Free ;
    FFooterFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FFooterFont = Nil Then
      Begin
      FFooterFont := TFOnt.Create ;
      FFooterFont.OnChange := FontChanged ;
      End ;
    FFooterFont.Assign (AFont) ;
    End ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetSelectFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FSelectFont.Free ;
    FSelectFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FSelectFont = Nil Then
      Begin
      FSelectFont := TFOnt.Create ;
      FSelectFont.OnChange := FontChanged ;
      End ;
    FSelectFont.Assign (AFont) ;
    End ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetInactiveFont (AFont : TFont) ;
  Begin
  If AFont = Nil Then
    Begin
    FInactiveFont.Free ;
    FInactiveFont := Nil ;
    FontChanged (Nil) ;
    End
  Else
    Begin
    If FInactiveFont = Nil Then
      Begin
      FInactiveFont := TFOnt.Create ;
      FInactiveFont.OnChange := FontChanged ;
      End ;
    FInactiveFont.Assign (AFont) ;
    End ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetBackgroundColor : TColor ;
  Begin
  If osBackgroundColor in FOverride Then
    Result := FBackgroundColor
  Else
    Result := ParentGrid.BackgroundColor ;
  End ;

Procedure TjmGridZone.SetBackgroundColor (Value : TColor) ;
  Begin
  If (Value = FBackgroundColor) And (osBackgroundColor In FOverride) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osBackgroundColor) ;
  FBackgroundColor := Value ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetHorizontalLineColor : TColor ;
  Begin
  If osHorizontalLineColor In FOverride Then
    Result := FHorizontalLineColor
  Else
    Result := ParentGrid.HorizontalLineColor ;
  End ;

Procedure TjmGridZone.SetHorizontalLineColor (AColor : TColor) ;
  Begin
  IF (AColor = FHorizontalLineColor) And (osHorizontalLineColor In FOverride) Then
    Exit ;

  FHorizontalLineColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHorizontalLineColor) ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetHeaderColor : TColor ;
  Begin
  If osHeaderColor In FOverride Then
    Result := FHeaderColor
  Else
    Result := ParentGrid.HeaderColor ;
  End ;

Procedure TjmGridZone.SetHeaderColor (AColor : TColor) ;
  Begin
  If (AColor = FHeaderColor) And (osHeaderColor In FOverride) Then
    Exit ;

  FHeaderColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderColor) ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetFooterColor : TColor ;
  Begin
  If osFooterColor In FOverride Then
    Result := FFooterColor
  Else
    Result := ParentGrid.FooterColor ;
  End ;

Procedure TjmGridZone.SetFooterColor (AColor : TColor) ;
  Begin
  If (AColor = FFooterColor) And (OsFooterColor In FOverride) Then
    Exit ;

  FFooterColor := AColor ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterColor) ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetVerticalLineStyle : TjmGridLineStyle ;
  Begin
  If osVerticalLineStyle In FOverride Then
    Result := FVerticalLineStyle
  Else
    Result := ParentGrid.VerticalLineStyle ;
  End ;

Procedure TjmGridZone.SetVerticalLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If (AStyle = FVerticalLineStyle) And (osVerticalLineStyle In FOverride) THen
    Exit ;

  FVerticalLineStyle := AStyle ;
  If Not (csLoading In ComponentState) Then
    Include (FOverride, osVerticalLineStyle) ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetColumnCount (ACount : Cardinal) ;
  Var
    Delta : Integer ;
    II : Integer ;
  Begin

  If (csLoading In ComponentState) Then
    Exit ;

  Delta := ACount - ColumnCount ;
  If Delta = 0 Then
    Begin
    Exit ;
    End
  Else If Delta > 0 Then
    Begin
    For II := 1 To Delta Do
      Begin
      ParentGrid.CreateColumn (Self) ;
      End ;
    End
  Else
    Begin
    For II := 1 To -Delta Do
      Begin
      FColumns.Delete (ColumnCount) ;
      End ;
    End ;

  LayoutChanged ;
  End ;

Function TjmGridZone.GetColumnCount : Cardinal ;
  Begin
  Result := FColumns.Count ;
  End ;

  Procedure TjmGridZone.SetZoneWidth (AZoneWidth : Cardinal) ;
  Begin
  If FZoneWidth = AZoneWidth Then
    Exit ;

  FZoneWidth := AZoneWidth ;
  ParentGrid.LayoutChanged ;
  End ;

Function TjmGridZone.EffectiveFont : TFont ;
  Begin
  If FFont <> Nil Then
    Result := FFont
  Else
    Result := ParentGrid.Font ;
  End ;

Function TjmGridZone.EffectiveHeaderFont : TFont ;
  Begin
  If FHeaderFont <> Nil Then
    Result := FHeaderFont
  Else
    Result := ParentGrid.HeaderFont ;
  End ;

Function TjmGridZone.EffectiveFooterFont : TFont ;
  Begin
  If FFooterFont <> Nil Then
    Result := FFooterFont
  Else
    Result := ParentGrid.FooterFont ;
  End ;

Function TjmGridZone.EffectiveSelectFont : TFont ;
  Begin
  If FSelectFont <> Nil Then
    Result := FSelectFont
  Else
    Result := ParentGrid.SelectFont ;
  End ;

Function TjmGridZone.EffectiveInactiveFont : TFont ;
  Begin
  If FInactiveFont <> Nil Then
    Result := FInactiveFont
  Else
    Result := ParentGrid.InactiveFont ;
  End ;

Function TjmGridZone.GetLastNonhiddenColumn : Cardinal ;
  Var
    ICol : Integer ;
  Begin
  Result := 0 ;
  For ICol := 1 to ColumnCount Do
    Begin
    If Not Columns [ICol].Hidden Then
      Result := ICol ;
    End ;
  End ;

Procedure TjmGridZone.SetExtendRightColumn (State : Boolean) ;
  Begin
  IF State = FExtendRightColumn Then
    Exit ;

  FExtendRightColumn := State ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetOverride (AValue : TjmGridOverrideSettings) ;
  Begin
  FOverride := AValue ;
  ParentGrid.LayoutChanged ;
  End ;

Function TjmGridZone.GetSelectColor : TColor ;
  Begin
  If osSelectColor in FOverride Then
    Result := FSelectColor
  Else
    Result := ParentGrid.SelectColor ;
  End ;

Procedure TjmGridZone.SetSelectColor (AColor : TColor) ;
  Begin
  If (FSelectColor = AColor) And (osSelectColor In FOverride) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osSelectColor) ;
  FSelectColor := AColor ;
  LayoutChanged ;
  End ;

Procedure TjmGridZone.SetShowHorizontalScrollBar (AState : TjmGridScrollBars) ;
  Begin
  If AState = FShowHorizontalScrollBar Then
    Exit ;

  FShowHorizontalScrollBar := AState ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetRowCount : Cardinal ;
  Begin
  If osRowCount in FOverride Then
    Result := FRows.Count
  Else
    Result := ParentGrid.DefaultRowCount ;
  End ;

Procedure TjmGridZone.SetRowCount (ACount : Cardinal) ;
  Var
    II : Integer ;
  Begin
  If (osRowCount in FOverride) And (FRows.Count = ACount) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowCount) ;

  If ACount > FRows.Count Then
    Begin
    For II := FRows.Count + 1 To ACount Do
      TjmGridRow.Create (Self) ;
    End
  Else If ACount < FRows.Count Then
    Begin
    While ACount < FRows.Count Do
      begin
      FRows.Delete (FRows.Count) ;
      End ;
    End ;

  ParentGrid.LayoutChanged ;
  End ;

Procedure TjmGridZone.SetRowLineStyle (AStyle : TjmGridLineStyle) ;
  Begin
  If (osRowLineStyle In FOverride) And (FRowLineStyle = AStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowLineStyle) ;
  FRowLineStyle := AStyle ;
  ParentGrid.LayoutChanged ;
  End ;

Function TjmGridZone.GetRowLineStyle : TjmGridLineStyle ;
  Begin
  If (osRowLineStyle In FOVerride) Then
    Result := FRowLineStyle
  Else
    Result := ParentGrid.RowLineStyle ;
  End ;

Function TjmGridZone.GetRowHeight : Cardinal ;
  Begin
  If osRowHeight in FOverride Then
    Result := FRowHeight
  Else
    Result := ParentGrid.RowHeight ;
  End ;

Procedure TjmGridZone.SetRowHeight (AValue : Cardinal) ;
  Begin
  If (osRowHeight In FOverride) And (AValue = FRowHeight) Then
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osRowHeight) ;
  FRowHeight := AValue ;
  ParentGrid.LayoutChanged ;
  End ;

Function TjmGridZone.GetCellFrameColor : TColor ;
  Begin
  If osCellFrameColor In FOverride Then
    Result := FCellFrameColor
  Else
    Result := ParentGrid.CellFrameColor ;
  End ;

Procedure TjmGridZone.SetCellFrameColor (AColor : TColor) ;
  Begin
  If (osCellFrameColor In FOverride) And (AColor = FCellFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osCellFrameColor) ;
  FCellFrameColor := AColor ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetCellFrameStyle : TjmGridFrameStyle ;
  Begin
  If osCellFrameStyle In FOverride Then
    Result := FCellFrameStyle
  Else
    Result := ParentGrid.CellFrameStyle ;
  End ;

Procedure TjmGridZone.SetCellFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osCellFrameStyle In FOverride) And (AStyle = FCellFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osCellFrameStyle) ;
  FCellFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetHeaderFrameColor : TColor ;
  Begin
  If osHeaderFrameColor In FOverride Then
    Result := FHeaderFrameColor
  Else
    Result := ParentGrid.HeaderFrameColor ;
  End ;

Procedure TjmGridZone.SetHeaderFrameColor (AColor : TColor) ;
  Begin
  If (osHeaderFrameColor In FOverride) And (AColor = FHeaderFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderFrameColor) ;
  FHeaderFrameColor := AColor ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetHeaderFrameStyle : TjmGridFrameStyle ;
  Begin
  If osHeaderFrameStyle In FOverride Then
    Result := FHeaderFrameStyle
  Else
    Result := ParentGrid.HeaderFrameStyle ;
  End ;

Procedure TjmGridZone.SetHeaderFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osHeaderFrameStyle In FOverride) And (AStyle = FHeaderFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osHeaderFrameStyle) ;
  FHeaderFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetFooterFrameColor : TColor ;
  Begin
  If osFooterFrameColor In FOverride Then
    Result := FFooterFrameColor
  Else
    Result := ParentGrid.FooterFrameColor ;
  End ;

Procedure TjmGridZone.SetFooterFrameColor (AColor : TColor) ;
  Begin
  If (osFooterFrameColor In FOverride) And (AColor = FFooterFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterFrameColor) ;
  FFooterFrameColor := AColor ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetFooterFrameStyle : TjmGridFrameStyle ;
  Begin
  If osFooterFrameStyle in FOverride Then
    Result := FFooterFrameStyle
  Else
    Result := ParentGrid.FooterFrameStyle ;
  End ;

Procedure TjmGridZone.SetFooterFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osFooterFrameStyle In FOverride) And (AStyle = FFooterFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osFooterFrameStyle) ;
  FFooterFrameStyle := AStyle ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetActiveFrameColor : TColor ;
  Begin
  If osActiveFrameColor In FOverride Then
    Result := FActiveFrameColor
  Else
    Result := ParentGrid.ActiveFrameColor ;
  End ;

Procedure TjmGridZone.SetActiveFrameColor (AColor : TColor) ;
  Begin
  If (osActiveFrameColor In FOverride) And (AColor = FActiveFrameColor) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osActiveFrameColor) ;
  FActiveFrameColor := AColor ;
  LayoutChanged ;
  End ;

Function TjmGridZone.GetActiveFrameStyle : TjmGridFrameStyle ;
  Begin
  If osActiveFrameStyle In FOverride Then
    Result := FActiveFrameStyle
  Else
    Result := ParentGrid.ActiveFrameStyle ;
  End ;

Procedure TjmGridZone.SetActiveFrameStyle (AStyle : TjmGridFrameStyle) ;
  Begin
  If (osActiveFrameStyle In FOverride) And (AStyle = FActiveFrameStyle) THen
    Exit ;

  If Not (csLoading In ComponentState) Then
    Include (FOverride, osActiveFrameStyle) ;
  FActiveFrameStyle := AStyle ;
  LayoutChanged ;
  End ;


