{
  Copyright 1997, Colosseum Builders, Inc.

  This software may be freely reistributed as long as this copyright notice
  is included and as long as modifications are documented at the top of the
  file.

  This software may be used under licence from Colosseum Builders, Inc. No
  transfer of ownership is made.

  This software may only be used on an "as-is" basis. There is no warranty of
  any kind.

  This software may only be included in other software if the author assumes
  all liability. Colosseum Builders, Inc. cannot ensure that its software will
  work correctly in any system.

  This software may be used without a fee in software that is distributed
  with complete source code to all users.

  If this software is incorporated in a system that is not distributed with
  complete source code then a licencing fee of $75 per developer workstation
  must be paid to Colosseum Builders, Inc. NJ customers must include sales
  tax (As of 1-Jan-1997 $4.50 for a total of $79.50). Checks should be made
  out to Colosseum Builders, Inc and mailed to

  Colosseum Builders, Inc
  E101 103 Park Ave
  Summit NJ 07901

}
{
  This file contains the implementation of TjmGridZone methods that deal
  with scrolling.
}
{
  Function: TjmGridZone.IsScrollBarNeeded

  Determines if a zone needs a horizontal scroll bar.

  Parameters:
          None

  Return Value:
         True => The zone should display a scroll bar
         False => The zone should not display a scroll bar

}
Function TjmGridZone.IsScrollBarNeeded : Boolean ;
  Begin
  Case FShowHorizontalScrollBar Of
    sbWhenNeeded:
      If (FLeftOffset <> 0) Or (RowWidth > Width) Then
        Result := True
      Else
        Result := False ;
    sbAlways:
      Result := True ;
    sbNever: { ssNone }
      Result := False ;
    Else
      Raise Exception.Create ('Unknown Horizontal Scroll Bar Type') ;
    End ;
  End ; { TjmGridZone.IsScrollBarNeeded }

{
  Procedure: TjmGridZone.MakeLeftOffset

  Scrolls the zone horizontally.

  Parameters:
          NewOffset: The position where the zone should be scrolled to.

}
Procedure TjmGridZone.MakeLeftOffset (NewOffset : Cardinal) ;
  Var
    ScrollRect, ClipRect : TRect ;
    Shift : Integer ; { Relative Distance to scroll }
  Begin
  If NewOffset = FLeftOffset Then
    Exit ;

  Shift := FLeftOffset - NewOffset ;
  FLeftOffset := NewOffset ;

  ScrollRect := GetClientRect ;
  ClipRect := ScrollRect ;

  If (FExtendColumnLeftPosition < ScrollRect.Right) Then
    Begin { Special case of visible extended columns begin scrolled left.
            They need to be stretched so the entire column must be invalidated. }
    If (Shift < 0) Then
      Begin { Left Scroll }
      { Do not include any extended columns in the scroll area. }
      ScrollRect.Right := FExtendColumnLeftPosition ;
      ScrollWindowEx (Handle,
                      Shift,
                      0,
                      @ScrollRect,
                      @ClipRect,
                      0,
                      Nil,
                      SW_SCROLLCHILDREN) ;
      { We do not do an invalidate in the ScrollWindowEx since we need to
        invalidate the entire column here.  If all items in the column were
        always right justified this would not be necessary but this is not
        always the case. }
      { Now invalidate all the areas that include extended columns. }
      ScrollRect := GetClientRect ;
      ScrollRect.Left := FExtendColumnLeftPosition + Shift ;
      RedrawRect (ScrollRect) ;
      End
    Else
      Begin { Right Scroll }
      { Do not include any extended columns in the scroll area. }
      ScrollRect.Right := FExtendColumnLeftPosition ;
      ScrollWindowEx (Handle,
                      Shift,
                      0,
                      @ScrollRect,
                      @ClipRect,
                      0,
                      Nil,
                      SW_INVALIDATE OR SW_SCROLLCHILDREN) ;
      { Now invalidate all the areas that include extended columns. }
      ScrollRect := GetClientRect ;
      ScrollRect.Left := FExtendColumnLeftPosition ;
      RedrawRect (ScrollRect) ;
      End
    End
  Else
    Begin { Normal Scroll Handling }
    ScrollWindowEx (Handle,
                    Shift,
                    0,
                    @ScrollRect,
                    @ClipRect,
                    0,
                    Nil,
                    SW_INVALIDATE OR SW_SCROLLCHILDREN) ;
    End ;

  If ParentGrid.EditorVisible Then
    ParentGrid.PositionEditor ;

  SetScrollPosition ;
  End ;


{
  Procedure: TjmGridZone.MakeTopRecord

  Scrolls the zone vertically.

  Parameters:
          NewRecord: The record to be positioned at the top of the zone.

}
Procedure TjmGridZone.MakeTopRecord (NewRecord : LongInt) ;
  Var
    RecordRect : TRect ;

    YDelta : Integer ;
    ClipRect : TRect ;
  Begin
  If NewRecord = FGrid.TopVisibleRecord Then
    Exit ;  { No Change. }

  RecordRect := GetRecordRect ;

  If (NewRecord > FGrid.BottomVisibleRecord + 1)
     Or (NewRecord + (FGrid.BottomVisibleRecord - FGrid.TopVisibleRecord)
         < FGrid.TopVisibleRecord) Then
    Begin
    { No Amount of scrolling will help. }
    RedrawRect (RecordRect) ;
    Exit ;
    End ;

  { Determine how much we are scrolling by. }
  YDelta := (ParentGrid.TopVisibleRecord - NewRecord)
            * (ParentGrid.RecordHeight + LineWidth (FGrid.HorizontalLineStyle)) ;
            
  { Scroll the Window }
  ClipRect := RecordRect ;
  ScrollWindowEx (Handle,
                  0,
                  YDelta,
                  @RecordRect,
                  @ClipRect,
                  0,
                  Nil,
                  SW_INVALIDATE OR SW_SCROLLCHILDREN) ;

  End ; { TjmGridZone.MakeTopRow }

Procedure TjmGridZone.SetScrollBarRange ;
  Var
    SI : TScrollInfo ;
  Begin
  If (Not FHasHScrollBar) Or (Not HandleAllocated) Then
    Exit ;

  If RowWidth > Width Then
    Begin
    SetScrollRange (Handle, SB_HORZ, 0, RowWidth - Width, True) ;
    SI.cbSize := Sizeof (SI) ;
    SI.fMask := SIF_ALL ;
    SI.nMin := 0 ;
    SI.nMax := RowWidth ;
    SI.nPage := Width ;
    SI.nPos := FLeftOffset ;
    SetScrollInfo (Handle, SB_HORZ, SI, False) ;
    End ;
  End ;

Procedure TjmGridZone.MakeColumnVisible (ACol : Cardinal) ;
  Var
    Offset : Integer ;
  Begin
  Offset := GetColumnOffset (ACol) ;

  If Offset < FLeftOffset Then
    Begin
    MakeLeftOffset (Offset) ;
    End
  Else If (Offset - FLeftOffset + Columns [ACol].Width) > Width Then
    Begin
    If Columns [ACol].Width > Width Then
      MakeLeftOffset (Offset)
    Else
      MakeLeftOffset (Offset - Width + Columns [ACol].Width) ;
    End ;
  End ;


Procedure TjmGridZone.SetScrollPosition ;
  Begin
  If Not HandleAllocated Then
    Exit ;

  SetScrollPos (Handle, SB_HORZ, FLeftOffset, True) ;
  End ;

Function TjmGridZone.GetScrollPosition : Integer ;
  Begin
  Result := FLeftOffset ;
  End ;

{
  This function finds the next desirable left scroll position from a given
  starting point.  We want to align the next column along the left.
}

Function TjmGridZone.NextLeftScrollPosition (StartPos : Integer) : Integer ;
  Var
    IRow : Integer ;
    Offset : Integer ;
    ICol : Cardinal ;
  Begin
  Result := 0 ;
  If StartPos <= 0 Then
    Exit ;

  For IRow := 1 To FRows.Count Do
    Begin
    ICol := Rows [IRow].ColumnAtOffset (StartPos) ;
    IF ICol <> 0 Then
      Begin
      Offset := Rows [IRow].GetColumnOffset (ICol) ;
      If Offset > Result Then
        Result := Offset ;
      End ;
    End ;
  End ;

{
  This function finds the next desirable left scroll position from a given
  starting point.  We want to align the next column along the right.
}
Function TjmGridZone.NextRightScrollPosition (StartPos : Integer) : Integer ;
  Var
    IRow : Integer ;
    Offset : Integer ;
    ICol : Integer ;
  Begin
  Result := High (Integer) ;
  For IRow := 1 To FRows.Count Do
    Begin
    ICol := Rows [IRow].ColumnAtOffset (StartPos + Width) ;
    IF ICol <> 0 Then
      Begin
      Offset := Rows [IRow].GetColumnOffset (ICol) + Rows [IRow].Columns [ICol].Width ;
      If Offset < Result Then
        Result := Offset ;
      End ;
    End ;
  Result := Result - Width ;
  End ;


